@@ -60,28 +60,7 @@ def format_crash_report(when, exc_info, details, log_report=True):
     header += "When:       %s\n" % when
     header += "\n"
 
-    header += "Exception\n---------\n"
-    if exc_info:
-        header += ''.join(traceback.format_exception(*exc_info))
-        header += "\n"
-        # Print out a stack trace too.  The exception stack only contains
-        # calls between the try and the exception.
-        try:
-            stack = util.get_nice_stack()
-        except StandardError:
-            stack = traceback.extract_stack()
-        header += "Call Stack\n---------\n"
-        header += ''.join(traceback.format_list(stack))
-        header += "\n"
-    else:
-        # fake an exception with our call stack
-        try:
-            stack = util.get_nice_stack()
-        except StandardError:
-            stack = traceback.extract_stack()
-        header += ''.join(traceback.format_list(stack))
-        header += 'UnknownError: %s\n' % details
-        header += "\n"
+    header += format_stack_report(details, exc_info)
     if details:
         header += "Details: %s\n" % (details, )
 
@@ -137,6 +116,32 @@ def format_crash_report(when, exc_info, details, log_report=True):
 
     return report
 
+def format_stack_report(details, exc_info):
+    header = ''
+    header += "Exception\n---------\n"
+    if exc_info:
+        header += ''.join(traceback.format_exception(*exc_info))
+        header += "\n"
+        # Print out a stack trace too.  The exception stack only contains
+        # calls between the try and the exception.
+        try:
+            stack = util.get_nice_stack()
+        except StandardError:
+            stack = traceback.extract_stack()
+        header += "Call Stack\n---------\n"
+        header += ''.join(traceback.format_list(stack))
+        header += "\n"
+    else:
+        # fake an exception with our call stack
+        try:
+            stack = util.get_nice_stack()
+        except StandardError:
+            stack = traceback.extract_stack()
+        header += ''.join(traceback.format_list(stack))
+        header += 'UnknownError: %s\n' % details
+        header += "\n"
+    return header
+
 def extract_headers(report):
     """Takes the headers out of the report which includes the log
     files.  This makes it easier to log the headers and only the
