@@ -30,11 +30,16 @@
 from datetime import datetime
 from glob import glob
 from fnmatch import fnmatch
-import json
-import codecs
-import logging
 import os, os.path
 import shutil
+import fnmatch
+try:
+    import simplejson as json
+except ImportError:
+    import json
+import codecs
+import logging
+import re
 import time
 
 from miro import app
@@ -64,6 +69,49 @@ def unicode_to_path(path):
     """
     return utf8_to_filename(path.encode('utf8')).replace('/', os.path.sep)
 
+class GlobSet(object):
+    """
+    A set()like object which allows some of its values to be glob-style pattern
+    matches.
+    """
+    def __init__(self, values):
+        self.frozenset = frozenset(v.lower() for v in values
+                                   if v and '*' not in v)
+        patterns = [v.lower() for v in values if '*' in v]
+        self.repr = repr([v.lower() for v in values])
+        if patterns:
+            self.regex = re.compile('|'.join(fnmatch.translate(v)
+                                             for v in patterns))
+        else:
+            self.regex = None
+
+    def __repr__(self):
+        return "GlobSet(%s)" % (self.repr,)
+
+    def __contains__(self, value):
+        if not value:
+            return False
+        value = value.lower()
+        if value in self.frozenset:
+            return True
+        elif self.regex:
+            return bool(self.regex.match(value))
+        return False
+
+    def __iter__(self):
+        return iter(self.frozenset)
+
+    def __eq__(self, other):
+        return self.frozenset == frozenset(other)
+
+    def __and__(self, other):
+        if isinstance(other, GlobSet):
+            return NotImplemented
+        other = set(other)
+        if self.frozenset & other:
+            return True
+        return any(v for v in other if self.regex.match(v))
+
 class BaseDeviceInfo(object):
     """
     Base class for device information.
@@ -73,7 +121,8 @@ class BaseDeviceInfo(object):
     # might as well save a bit of memory anyways.
     __slots__ = ['name', 'device_name', 'vendor_id', 'product_id',
                  'video_conversion', 'video_path',
-                 'audio_conversion', 'audio_path', 'audio_types',
+                 'audio_conversion', 'audio_path',
+                 'container_types', 'audio_types', 'video_types',
                  'mount_instructions']
     def update(self, kwargs):
         for key, value in kwargs.items():
@@ -81,6 +130,8 @@ class BaseDeviceInfo(object):
                 self.audio_path = unicode_to_path(value)
             elif key == 'video_path':
                 self.video_path = unicode_to_path(value)
+            elif key.endswith('_types'):
+                setattr(self, key, GlobSet(value))
             else:
                 setattr(self, key, value)
 
@@ -110,7 +161,9 @@ class DeviceInfo(BaseDeviceInfo):
     video_path: mount-relative path to where the videos should be placed
     audio_conversion: the Miro conversion name for audio to this device
     audio_path: mount-relative path to where audio files should be placed
-    audio_types: audio MIME types this device supports
+    container_types: FFmpeg container formats this device supports
+    audio_types: FFmpeg audio codecs this device supports
+    video_types: FFmpeg video codecs this device supports
     mount_instructions: text to show the user about how to mount their device
     parent (optional): a MultipleDeviceInfo instance which has this device's
                        info
@@ -239,7 +292,7 @@ class DeviceManager(object):
                 else:
                     key = (info.vendor_id, info.product_id)
                     existing = self.device_by_id[key]
-                    self.device_by_id[key] = self.merge(existing, info)
+                    self.device_by_id[key] = self._merge(existing, info)
 
     def remove_device(self, info):
         # FIXME - need this
@@ -284,7 +337,7 @@ class DeviceManager(object):
             info = self.device_by_name[device_name]
         except KeyError:
             for info in self.generic_devices:
-                if fnmatch(device_name, info.device_name):
+                if fnmatch.fnmatch(device_name, info.device_name):
                     break
             else:
                 raise
