@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,25 +31,25 @@ import datetime
 import os
 import random
 import logging
-from base64 import b64encode
 
 from miro.gtcache import gettext as _
 from miro.database import DDBObject, ObjectNotFoundError
 from miro.dl_daemon import daemon, command
 from miro.download_utils import (next_free_filename, get_file_url_path,
-                                 filter_directory_name)
+        next_free_directory, filter_directory_name)
 from miro.util import (get_torrent_info_hash, returns_unicode, check_u,
-                       returns_filename, unicodify, check_f, to_uni)
-from miro import config
+                       returns_filename, unicodify, check_f, to_uni, is_magnet_uri)
+from miro import app
 from miro import dialogs
 from miro import displaytext
 from miro import eventloop
 from miro import httpclient
 from miro import models
 from miro import prefs
-from miro.plat.utils import samefile, FilenameType, unicode_to_filename
+from miro.plat.utils import samefile, unicode_to_filename
 from miro import flashscraper
 from miro import fileutil
+from miro.fileobject import FilenameType
 
 daemon_starter = None
 
@@ -83,7 +84,7 @@ class RemoteDownloader(DDBObject):
         self.main_item_id = None
         self.dlid = generate_dlid()
         self.status = {}
-        self.metainfo = self.fast_resume_data = None
+        self.metainfo = None
         self.state = u'downloading'
         if contentType is None:
             # HACK: Some servers report the wrong content-type for
@@ -178,9 +179,7 @@ class RemoteDownloader(DDBObject):
             try:
                 self.contentType = info['content-type'].decode('ascii',
                                                                'replace')
-            except (SystemExit, KeyboardInterrupt):
-                raise
-            except:
+            except (KeyError, UnicodeDecodeError):
                 self.contentType = None
             self.run_downloader()
         else:
@@ -206,6 +205,9 @@ class RemoteDownloader(DDBObject):
         self.run_downloader()
 
     def get_content_type(self):
+        if is_magnet_uri(self.url):
+            self.contentType = u'application/x-magnet'
+            return 
         httpclient.grab_headers(self.url, self.on_content_type,
                                 self.on_content_type_error)
 
@@ -240,27 +242,21 @@ class RemoteDownloader(DDBObject):
     def update_status(cls, data):
         for field in data:
             if field not in ['filename', 'shortFilename', 'channelName',
-                             'metainfo', 'fastResumeData']:
+                             'metainfo']:
                 data[field] = unicodify(data[field])
         self = get_downloader_by_dlid(dlid=data['dlid'])
-        # print data
+
         if self is not None:
             # FIXME - this should get fixed.
             metainfo = data.pop('metainfo', self.metainfo)
-            fast_resume_data = data.pop('fastResumeData',
-                    self.fast_resume_data)
-            # for metainfo and fast_resume_data, the downloader process
-            # doesn't send the keys if they haven't changed.  Therefore, use
-            # our current values if the key isn't present
-            current = (self.status, self.metainfo, self.fast_resume_data)
-            new = (data, metainfo, fast_resume_data)
-            try:
-                if current == new:
-                    return
-            except Exception:
-                # This is a known bug with the way we used to save
-                # fast resume data
-                logging.exception("RemoteDownloader.update_status: exception when comparing status")
+
+            # For metainfo, the downloader process doesn't send the
+            # keys if they haven't changed.  Therefore, use our
+            # current values if the key isn't present.
+            current = (self.status, self.metainfo)
+            new = (data, metainfo)
+            if current == new:
+                return
 
             was_finished = self.is_finished()
             old_filename = self.get_filename()
@@ -277,8 +273,6 @@ class RemoteDownloader(DDBObject):
                 self.status = data
             if metainfo != self.metainfo:
                 self.metainfo = metainfo
-            if fast_resume_data != self.fast_resume_data:
-                self.fast_resume_data = fast_resume_data
             self._recalc_state()
 
             # Store the time the download finished
@@ -290,8 +284,8 @@ class RemoteDownloader(DDBObject):
 
             if ((self.get_state() == u'uploading'
                  and not self.manualUpload
-                 and (config.get(prefs.LIMIT_UPLOAD_RATIO)
-                      and self.get_upload_ratio() > config.get(prefs.UPLOAD_RATIO)))):
+                 and (app.config.get(prefs.LIMIT_UPLOAD_RATIO)
+                      and self.get_upload_ratio() > app.config.get(prefs.UPLOAD_RATIO)))):
                 self.stop_upload()
 
             self.signal_change(needs_signal_item=needs_signal_item,
@@ -335,6 +329,7 @@ class RemoteDownloader(DDBObject):
                                                 self.channelName)
             c.send()
             _downloads[self.dlid] = self
+            self.status["state"] = u"downloading"
         else:
             self.status["state"] = u'failed'
             self.status["shortReasonFailed"] = _('File not found')
@@ -347,11 +342,10 @@ class RemoteDownloader(DDBObject):
             c = command.PauseDownloadCommand(RemoteDownloader.dldaemon,
                                              self.dlid)
             c.send()
-        else:
-            self.before_changing_status()
-            self.status["state"] = u"paused"
-            self.after_changing_status()
-            self.signal_change()
+        self.before_changing_status()
+        self.status["state"] = u"paused"
+        self.after_changing_status()
+        self.signal_change()
 
     def stop(self, delete):
         """Stops the download and removes the partially downloaded
@@ -364,11 +358,11 @@ class RemoteDownloader(DDBObject):
                                                 self.dlid, delete)
                 c.send()
                 del _downloads[self.dlid]
-        else:
-            if delete:
-                self.delete()
-            self.status["state"] = u"stopped"
-            self.signal_change()
+
+        if delete:
+            self.delete()
+        self.status["state"] = u"stopped"
+        self.signal_change()
 
     def delete(self):
         if "filename" in self.status:
@@ -384,9 +378,9 @@ class RemoteDownloader(DDBObject):
         parent = os.path.join(fileutil.expand_filename(filename),
                               os.path.pardir)
         parent = os.path.normpath(parent)
-        moviesDir = fileutil.expand_filename(config.get(prefs.MOVIES_DIRECTORY))
-        if ((os.path.exists(parent) and os.path.exists(moviesDir)
-             and not samefile(parent, moviesDir)
+        movies_dir = fileutil.expand_filename(app.config.get(prefs.MOVIES_DIRECTORY))
+        if ((os.path.exists(parent) and os.path.exists(movies_dir)
+             and not samefile(parent, movies_dir)
              and len(os.listdir(parent)) == 0)):
             try:
                 os.rmdir(parent)
@@ -416,10 +410,10 @@ class RemoteDownloader(DDBObject):
                 c = command.StartDownloadCommand(RemoteDownloader.dldaemon,
                                                  self.dlid)
                 c.send()
-            else:
-                self.status['state'] = u'downloading'
-                self.restart()
-                self.signal_change()
+
+            self.status['state'] = u'downloading'
+            self.restart()
+            self.signal_change()
 
     def migrate(self, directory):
         if _downloads.has_key(self.dlid):
@@ -453,12 +447,21 @@ class RemoteDownloader(DDBObject):
                 newfilename = os.path.join(directory, short_filename)
                 if newfilename == filename:
                     return
-                newfilename = next_free_filename(newfilename)
+                # create a file or directory to serve as a placeholder before
+                # we start to migrate.  This helps ensure that the destination
+                # we're migrating too is not already taken.
+                if fileutil.isdir(filename):
+                    newfilename = next_free_directory(newfilename)
+                    fp = None
+                else:
+                    newfilename, fp = next_free_filename(newfilename)
                 def callback():
                     self.status['filename'] = newfilename
                     self.signal_change(needs_signal_item=False)
                     self._file_migrated(filename)
                 fileutil.migrate_file(filename, newfilename, callback)
+                if fp is not None:
+                    fp.close() # clean up if we called next_free_filename()
         for i in self.item_list:
             i.migrate_children(directory)
 
@@ -494,7 +497,8 @@ class RemoteDownloader(DDBObject):
         "bittorrent".
         """
         self.confirm_db_thread()
-        if self.contentType == u'application/x-bittorrent':
+        if ((self.contentType == u'application/x-bittorrent'
+             or self.contentType == u'application/x-magnet')):
             return u"bittorrent"
 
         return u"http"
@@ -543,7 +547,7 @@ class RemoteDownloader(DDBObject):
         if self.status['retryTime'] > datetime.datetime.now():
             retry_delta = self.status['retryTime'] - datetime.datetime.now()
             time_str = displaytext.time_string(retry_delta.seconds)
-            return _('no connection - retrying in %s') % time_str
+            return _('no connection - retrying in %(time)s', {"time": time_str})
         else:
             return _('no connection - retrying soon')
 
@@ -657,10 +661,10 @@ class RemoteDownloader(DDBObject):
             return
         if self.get_state() in (u'downloading', u'offline'):
             self.restart()
-        if self.get_state() in (u'uploading'):
+        if self.get_state() == u'uploading':
             if ((self.manualUpload
-                 or (config.get(prefs.LIMIT_UPLOAD_RATIO)
-                     and self.get_upload_ratio() < config.get(prefs.UPLOAD_RATIO)))):
+                 or (app.config.get(prefs.LIMIT_UPLOAD_RATIO)
+                     and self.get_upload_ratio() < app.config.get(prefs.UPLOAD_RATIO)))):
                 self.restart()
             else:
                 self.stop_upload()
@@ -674,11 +678,17 @@ class RemoteDownloader(DDBObject):
         else:
             _downloads[self.dlid] = self
             dler_status = self.status
+            # FIXME: not sure why this is necessary
+            if self.contentType ==  u'application/x-magnet':
+                dler_status['url'] = self.url
             dler_status['metainfo'] = self.metainfo
-            dler_status['fastResumeData'] = self.fast_resume_data
+
             c = command.RestoreDownloaderCommand(RemoteDownloader.dldaemon,
                                                  dler_status)
             c.send()
+            self.before_changing_status()
+            self.status['state'] = u'downloading'
+            self.after_changing_status()
 
     def start_upload(self):
         """
@@ -738,7 +748,7 @@ class RemoteDownloader(DDBObject):
         self.signal_change()
 
 def cleanup_incomplete_downloads():
-    download_dir = os.path.join(config.get(prefs.MOVIES_DIRECTORY),
+    download_dir = os.path.join(app.config.get(prefs.MOVIES_DIRECTORY),
                                 'Incomplete Downloads')
     if not fileutil.exists(download_dir):
         return
@@ -754,44 +764,62 @@ def cleanup_incomplete_downloads():
                     filename = os.path.join(download_dir, filename)
                 files_in_use.add(filename)
 
-    for f in fileutil.listdir(download_dir):
+    try:
+        entries = fileutil.listdir(download_dir)
+    except OSError:
+        entries = []
+
+    for f in entries:
         f = os.path.join(download_dir, f)
         if f not in files_in_use:
             try:
                 if fileutil.isfile(f):
-                    fileutil.remove (f)
+                    fileutil.remove(f)
                 elif fileutil.isdir(f):
-                    fileutil.rmtree (f)
+                    fileutil.rmtree(f)
             except OSError:
                 # FIXME - maybe a permissions error?
                 pass
 
 def kill_uploaders(*args):
-    torrent_limit = config.get(prefs.UPSTREAM_TORRENT_LIMIT)
+    torrent_limit = app.config.get(prefs.UPSTREAM_TORRENT_LIMIT)
     auto_uploads = list(RemoteDownloader.auto_uploader_view())
     for dler in auto_uploads[torrent_limit:]:
         dler.stop_upload()
 
-def config_change_uploaders(key, value):
+def _on_config_change(obj, key, value):
     if key == prefs.UPSTREAM_TORRENT_LIMIT.key:
         kill_uploaders()
 
-def limit_uploaders():
-    tracker = RemoteDownloader.auto_uploader_view().make_tracker()
-    tracker.connect('added', kill_uploaders)
-    config.add_change_callback(config_change_uploaders)
-    kill_uploaders()
 
 class DownloadDaemonStarter(object):
     def __init__(self):
         RemoteDownloader.initialize_daemon()
         self.downloads_at_startup = list(RemoteDownloader.make_view())
         self.started = False
+        self._config_callback_handle = None
+        self._download_tracker = None
+
+    def limit_uploaders(self):
+        view = RemoteDownloader.auto_uploader_view()
+        self._download_tracker = view.make_tracker()
+        self._download_tracker.connect('added', kill_uploaders)
+        self._config_callback_handle = app.backend_config_watcher.connect(
+                "changed", _on_config_change)
+        kill_uploaders()
+
+    def disconnect_signals(self):
+        if self._download_tracker is not None:
+            self._download_tracker.unlink()
+            self._download_tracker = None
+        if self._config_callback_handle is not None:
+            app.backend_config_watcher.disconnect(self._config_callback_handle)
+            self._config_callback_handle = None
 
     def startup(self):
         cleanup_incomplete_downloads()
         RemoteDownloader.dldaemon.start_downloader_daemon()
-        limit_uploaders()
+        self.limit_uploaders()
         self.restart_downloads()
         self.started = True
 
@@ -800,6 +828,7 @@ class DownloadDaemonStarter(object):
             downloader.restart_on_startup_if_needed()
 
     def shutdown(self, callback):
+        self.disconnect_signals()
         self.shutdown_callback = callback
         if not self.started:
             self._on_shutdown()
@@ -870,11 +899,13 @@ def get_downloader_for_item(item):
             get_torrent_info_hash(path)
         except ValueError:
             raise ValueError("Don't know how to handle %s" % url)
-        except IOError:
+        except (OSError, IOError):
             return None
         else:
             return RemoteDownloader(url, item, u'application/x-bittorrent',
                                     channelName=channelName)
+    elif is_magnet_uri(url):
+        return RemoteDownloader(url, item, u'application/x-magnet')
     else:
         return RemoteDownloader(url, item, channelName=channelName)
 
