@@ -39,6 +39,9 @@ from miro.clock import clock
 
 from miro import eventloop
 from miro import feedparser
+from miro import filetypes
+from miro import flashscraper
+from miro import util
 from miro.datastructures import Fifo
 
 # values from feedparser dicts that don't have to convert in
@@ -77,78 +80,32 @@ def _convert_if_feedparser_dict(obj):
         return normalize_feedparser_dict(obj)
     return obj
 
-from miro import app
-from miro import prefs
-USER_AGENT = (feedparser.USER_AGENT + " %s/%s (%s)" %
-              (app.config.get(prefs.SHORT_APP_NAME),
-               app.config.get(prefs.APP_VERSION),
-               app.config.get(prefs.PROJECT_URL)))
-
 def parse(url_file_stream_or_string):
-    return feedparser.parse(url_file_stream_or_string, USER_AGENT)
+    """Parse a feed.
 
-class _QueueParseProcessor(threading.Thread):
-    """Object that handles the queue_parse() call."""
-
-    WAIT_TIME = 0.2 # time to wait between parse() calls
-
-    def __init__(self):
-        self.queue = Fifo()
-        self.last_end_time = 0
-        self.current_call_info = None
-
-    def add_to_queue(self, url_file_stream_or_string, callback, errback):
-        call_info = (url_file_stream_or_string, callback, errback)
-        self.queue.enqueue(call_info)
-        self._schedule_next()
-
-    def _schedule_next(self):
-        if len(self.queue) == 0:
-            return # nothing to schedule
-        if self.current_call_info is not None:
-            return # we'll run again once that call is finished
-
-        self.current_call_info = self.queue.dequeue()
-        wait_left = (self.last_end_time + self.WAIT_TIME) - clock()
-        if wait_left <= 0:
-            self._run_current()
-        else:
-            eventloop.add_timeout(wait_left, self._run_current,
-                    "feedparser queue timeout")
-
-    def _run_current(self):
-        url_file_stream_or_string = self.current_call_info[0]
-        eventloop.call_in_thread(self._callback, self._errback,
-                feedparser.parse, "Feedparser callback",
-                url_file_stream_or_string)
-
-    def _finish_call(self, callback, *args, **kwargs):
-        self.last_end_time = clock()
-        self.current_call_info = None
-        callback(*args, **kwargs)
-        self._schedule_next()
-
-    def _callback(self, *args, **kwargs):
-        callback = self.current_call_info[1]
-        self._finish_call(callback, *args, **kwargs)
-
-    def _errback(self, *args, **kwargs):
-        errback = self.current_call_info[2]
-        self._finish_call(errback, *args, **kwargs)
-
-_queue_parse_proccessor = _QueueParseProcessor()
-
-def queue_parse(url_file_stream_or_string, callback, errback):
-    """Call parse in a separet thread.
-
-    This method tries to ensure that feedparser doesn't hog the whole CPU
-    using a few methods:
-      - Only feedparser.parse() call is running at a given time.
-      - After a parse() call, we wait for a bit to give other things time to
-        access the CPU.
+    This method runs the feed data through feedparser.parse, then does some
+    other things like unicodify it and fix issues with certain feed providers.
     """
-    _queue_parse_proccessor.add_to_queue(url_file_stream_or_string, callback,
-            errback)
+    parsed = feedparser.parse(url_file_stream_or_string)
+    parsed = util.unicodify(parsed)
+    _yahoo_hack(parsed['entries'])
+    return parsed
+
+def _yahoo_hack(feedparser_entries):
+    """Hack yahoo search to provide enclosures"""
+    for entry in feedparser_entries:
+        if 'enclosures' not in entry:
+            try:
+                url = entry['link']
+            except KeyError:
+                continue
+            mimetype = filetypes.guess_mime_type(url)
+            if mimetype is not None:
+                entry['enclosures'] = [{'url': util.to_uni(url),
+                                        'type': util.to_uni(mimetype)}]
+            elif flashscraper.is_maybe_flashscrapable(url):
+                entry['enclosures'] = [{'url': util.to_uni(url),
+                                        'type': util.to_uni("video/flv")}]
 
 def sanitizeHTML(htmlSource, encoding):
     return feedparser.sanitizeHTML(htmlSource, encoding)
