@@ -37,7 +37,7 @@ from miro import httpclient
 import urlparse
 import cgi
 from xml.dom import minidom
-from urllib import unquote_plus
+from urllib import unquote_plus, urlencode
 from miro.util import check_u
 
 def is_maybe_flashscrapable(url):
@@ -124,11 +124,16 @@ def _youtube_callback_step2(info, video_id, callback):
         # build the format codes.
         fmt_list = [x.split('/')[0] for x in params['fmt_list'][0].split(',')]
         # build the list of available urls.
-        fmt_url_map = params["url_encoded_fmt_stream_map"][0].split(",")
-        # strip url= from url=xxxxxx, strip trailer.
-        fmt_url_map = [unquote_plus(x[4:]).split(';')[0] for x in fmt_url_map]
-        # now build the actual fmt_url_map ...
-        fmt_url_map = dict(zip(fmt_list, fmt_url_map))
+        stream_map = params["url_encoded_fmt_stream_map"][0].split(",")
+        fmt_url_map = dict()
+        # strip url= from url=xxxxxx, strip trailer.  Strip duplicate params.
+        for fmt, u in zip(fmt_list, stream_map):
+            o = urlparse.urlsplit(unquote_plus(u[4:]).split(';')[0])
+            qs = urlencode(list(set(urlparse.parse_qsl(o.query))))
+            # Let's put humpty dumpty back together again
+            fmt_url_map[fmt] = urlparse.urlunsplit(
+              urlparse.SplitResult(o.scheme, o.netloc, o.path, qs, o.fragment))
+            
 
         title = params.get("title", ["No title"])[0]
         try:
@@ -289,7 +294,7 @@ def _scrape_vimeo_video_url(url, callback):
             lambda x: _scrape_vimeo_callback(x, callback),
             lambda x: _scrape_vimeo_errback(x, callback))
     except StandardError:
-        logging.warning("Unable to scrape vimeo.com video URL: %s", url)
+        logging.exception("Unable to scrape vimeo.com video URL: %s", url)
         callback(None)
 
 MEGALOOP_RE = re.compile(r'http://([^/]+\.)?vimeo.com/moogaloop.swf\?clip_id=(\d+)')
@@ -303,24 +308,27 @@ def _scrape_vimeo_moogaloop_url(url, callback):
             lambda x: _scrape_vimeo_callback(x, callback),
             lambda x: _scrape_vimeo_errback(x, callback))
     except StandardError:
-        logging.warning("Unable to scrape vimeo.com moogaloop URL: %s", url)
+        logging.exception("Unable to scrape vimeo.com moogaloop URL: %s", url)
         callback(None)
 
-VIMEO_CLIP_RE = re.compile(r'http://www.vimeo.com/moogaloop/load/clip:(\d+)')
+VIMEO_CLIP_RE = re.compile(r'http://(?:www\.)?vimeo\.com/moogaloop/load/clip:(?P<id_>\d+)')
 
 def _scrape_vimeo_callback(info, callback):
     url = info['redirected-url']
     try:
         doc = minidom.parseString(info['body'])
-        id_ = VIMEO_CLIP_RE.match(url).group(1)
+        id_ = VIMEO_CLIP_RE.match(url).group('id_')
         req_sig = doc.getElementsByTagName('request_signature').item(0).firstChild.data.decode('ascii', 'replace')
         req_sig_expires = doc.getElementsByTagName('request_signature_expires').item(0).firstChild.data.decode('ascii', 'replace')
-        url = (u"http://www.vimeo.com/moogaloop/play/clip:%s/%s/%s/?q=sd" %
+        url = (u"http://www.vimeo.com/moogaloop/play/clip:%s/%s/%s/?q=" %
                (id_, req_sig, req_sig_expires))
-        # TODO: HD support
-        callback(url)
+        hd_url = url + 'hd'
+        sd_url = url + 'sd'
+        httpclient.grab_headers(hd_url,
+                                lambda x: callback(hd_url),
+                                lambda x: callback(sd_url))
     except StandardError:
-        logging.warning("Unable to scrape XML for vimeo.com video URL: %s", url)
+        logging.exception("Unable to scrape XML for vimeo.com video URL: %s", url)
         callback(None)
 
 def _scrape_vimeo_errback(err, callback):
