@@ -202,6 +202,13 @@ class MiroInterpreter(cmd.Cmd):
         print "FEEDS"
         self._print_feeds(self.feed_tabs.get_all_tabs())
 
+    @run_in_event_loop
+    def do_update(self, line):
+        """update -- Updates all the feeds."""
+        for mem in self.feed_tabs.get_all_tabs():
+            print "telling %s to update" % mem.get_title()
+            mem.update()
+
     @run_in_event_loop
     def do_play(self, line):
         """play <name> -- Plays an item by name in an external player."""
@@ -213,7 +220,7 @@ class MiroInterpreter(cmd.Cmd):
             print "No item named %r" % line
             return
         if item_.is_downloaded():
-            resources.open_file(item_.get_video_filename())
+            resources.open_file(item_.get_filename())
         else:
             print '%s is not downloaded' % item_.get_title()
 
