@@ -210,26 +210,63 @@ class Application:
         call_on_ui_thread(self._handle_movies_directory_gone, continue_callback)
 
     def _handle_movies_directory_gone(self, continue_callback):
+        # Make sure we close the upgrade dialog before showing a new one
+        self.message_handler.close_upgrade_dialog()
+
         title = _("Movies directory gone")
-        description = _(
-            "%(shortappname)s can't use the primary video directory "
-            "located at:\n"
-            "\n"
-            "%(moviedirectory)s\n"
-            "\n"
-            "This may be because it is located on an external drive that "
-            "is not connected, is a directory that %(shortappname)s does "
-            "not have write permission to, or there is something that is "
-            "not a directory at that path.\n"
-            "\n"
-            "Miro will now exit. You can connect the drive or otherwise "
-            "fix the problem and relaunch %(shortappname)s.",
-            {"shortappname": app.config.get(prefs.SHORT_APP_NAME),
-             "moviedirectory": app.config.get(prefs.MOVIES_DIRECTORY)}
-        )
-        dialogs.show_message(title, description,
-                                   alert_type=dialogs.CRITICAL_MESSAGE)
-        self.do_quit()
+        movies_directory = app.config.get(prefs.MOVIES_DIRECTORY)
+        if os.path.isdir(movies_directory):
+            # allow users to continue if the directory exists
+            description = _(
+                "%(shortappname)s can't use the primary video directory "
+                "located at:\n"
+                "\n"
+                "%(moviedirectory)s\n"
+                "\n"
+                "This may be because it is located on an external drive "
+                "that is not connected, is a directory that "
+                "%(shortappname)s does not have write permission to, or "
+                "there is something that is not a directory at that "
+                "path.\n"
+                "\n"
+                "If you continue, the primary video directory will be "
+                "reset to a location on this drive.  If you had videos "
+                "downloaded this will cause %(shortappname)s to lose "
+                "details about those videos.\n "
+                "\n"
+                "If you quit, then you can connect the drive or otherwise "
+                "fix the problem and relaunch %(shortappname)s.",
+                {"shortappname": app.config.get(prefs.SHORT_APP_NAME),
+                 "moviedirectory": movies_directory})
+            choice = dialogs.show_choice_dialog(title, description,
+                    [dialogs.BUTTON_CONTINUE, dialogs.BUTTON_QUIT])
+        else:
+            # if the directory doesn't exist, don't let the user continue
+            description = _(
+                "%(shortappname)s can't use the primary video directory "
+                "located at:\n"
+                "\n"
+                "%(moviedirectory)s\n"
+                "\n"
+                "This may be because it is located on an external drive "
+                "that is not connected, is a directory that "
+                "%(shortappname)s does not have write permission to, or "
+                "there is something that is not a directory at that "
+                "path.\n"
+                "\n"
+                "%(shortappname)s will now exit. You can connect the drive "
+                "or otherwise fix the problem and relaunch "
+                "%(shortappname)s.",
+                {"shortappname": app.config.get(prefs.SHORT_APP_NAME),
+                    "moviedirectory": movies_directory})
+            dialogs.show_message(title, description,
+                                       alert_type=dialogs.CRITICAL_MESSAGE)
+            choice = dialogs.BUTTON_QUIT
+
+        if choice == dialogs.BUTTON_CONTINUE:
+            continue_callback()
+        else:
+            self.do_quit()
 
     def handle_first_time(self, continue_callback):
         call_on_ui_thread(lambda: self._handle_first_time(continue_callback))
@@ -260,6 +297,7 @@ class Application:
         videobox.controls.previous.connect('released', self.on_stop_fast_playback)
         videobox.playback_mode.shuffle.connect('clicked', self.on_shuffle_clicked)
         videobox.playback_mode.repeat.connect('clicked', self.on_repeat_clicked)
+        self.set_left_width(app.widget_state.get_tabs_width())
         self.window.show()
         messages.TrackPlaylists().send_to_backend()
         messages.TrackDownloadCount().send_to_backend()
@@ -281,7 +319,20 @@ class Application:
     def get_right_width(self):
         """Returns the width of the right side of the splitter.
         """
-        return self.window.get_frame().get_width() - self.window.splitter.get_left_width()
+        left_width = self.get_left_width()
+        if left_width:
+            return self.window.get_frame().get_width() - left_width
+        else:
+            return None
+
+    def get_left_width(self):
+        if hasattr(self.window, 'splitter'):
+            return self.window.splitter.get_left_width()
+        else:
+            return None
+
+    def set_left_width(self, width):
+        self.window.splitter.set_left_width(width)
 
     def on_volume_change(self, slider, volume):
         app.playback_manager.set_volume(volume)
@@ -512,7 +563,11 @@ class Application:
         def playback_finished_if_playing_selection():
             if app.playback_manager.is_playing:
                 item = app.playback_manager.get_playing_item()
-                if item in selection:
+                # bz:17555
+                # Snarf the id out.  Turns out that while the ItemInfo id
+                # is the same but the object is different so the boolean
+                # expression fails.
+                if item is not None and item.id in [s.id for s in selection]:
                     app.playback_manager.on_movie_finished()
 
         external_count = len([s for s in selection if s.is_external])
@@ -1144,6 +1199,9 @@ class Application:
             app.item_list_controller_manager.undisplay_controller()
         if self.window is not None:
             self.window.destroy()
+        width = self.get_left_width()
+        if width:
+            app.widget_state.set_tabs_width(width)
         app.controller.shutdown()
         self.quit_ui()
 
@@ -1341,6 +1399,7 @@ class WidgetsMessageHandler(messages.MessageHandler):
             if (item and item.remote and
               item.host == host and item.port == port):
                 app.playback_manager.stop()
+        logging.debug('SHARING DISAPPEARED')
         message = messages.TabsChanged('connect', [], [], [share.id])
         typ, selected_tabs = app.tabs.selection
         if typ == u'connect' and share in selected_tabs:
@@ -1391,10 +1450,11 @@ class WidgetsMessageHandler(messages.MessageHandler):
     def handle_sharing_connect_failed(self, message):
         name = message.share.name
         title = _('Connect failed')
-        fmtargs = dict(name=name)
         description = _('Connection to share %(name)s failed.\n\n'
                         'The share is either unreachable or incompatible '
-                        'with Miro sharing.' % fmtargs)
+                        'with %(appname)s sharing.',
+                        {"name": name,
+                         "appname": app.config.get(prefs.SHORT_APP_NAME)})
         dialogs.show_message(title, description, dialogs.INFO_MESSAGE)
         app.tabs.select_guide()
 
@@ -1415,14 +1475,17 @@ class WidgetsMessageHandler(messages.MessageHandler):
         if self.dbupgrade_progress_dialog is None:
             if message.doing_db_upgrade:
                 title = _('Upgrading database')
-                text = _("Miro is upgrading your database of podcasts and "
-                "files.  This one-time process can take a long time if you  "
-                "have a large number of items in Miro (it can even take more "
-                "than 30 minutes).")
+                text = _(
+                    "%(appname)s is upgrading your database of podcasts "
+                    "and files.  This one-time process can take a long "
+                    "time if you have a large number of items in "
+                    "%(appname)s (it can even take more than 30 minutes).",
+                    {"appname": app.config.get(prefs.SHORT_APP_NAME)})
             else:
                 title = _('Preparing Items')
-                text = _("Miro shutdown improperly and needs to prepare "
-                        "your items for display.")
+                text = _("%(appname)s shutdown improperly and needs to "
+                         "prepare your items for display.",
+                         {"appname": app.config.get(prefs.SHORT_APP_NAME)})
             self.dbupgrade_progress_dialog = dialogs.DBUpgradeProgressDialog(
                     title, text)
             self.dbupgrade_progress_dialog.run()
@@ -1448,6 +1511,8 @@ class WidgetsMessageHandler(messages.MessageHandler):
         if hasattr(self, "_startup_failure_mode"):
             return
         self._startup_failure_mode = True
+        # We may still have the DB upgrade dialog open.  If so, close it.
+        self.close_upgrade_dialog()
 
         dialogs.show_message(message.summary, message.description,
                 dialogs.CRITICAL_MESSAGE)
@@ -1458,6 +1523,8 @@ class WidgetsMessageHandler(messages.MessageHandler):
             logging.info("already in db failure mode--skipping")
             return
         self._database_failure_mode = True
+        # We may still have the DB upgrade dialog open.  If so, close it.
+        self.close_upgrade_dialog()
 
         ret = dialogs.show_choice_dialog(
             message.summary, message.description,
