@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,14 +32,26 @@ handles laying out complex objects for the custom drawing code like
 text blocks and buttons.
 """
 
-import itertools
 import math
 
 import cairo
 import gtk
 import pango
 
+from miro import util
 from miro.plat.frontends.widgets import use_native_buttons
+from miro.frontends.widgets.gtk import pygtkhacks
+
+class FontCache(util.Cache):
+    def get(self, context, description, scale_factor, bold, italic):
+        key = (context, description, scale_factor, bold, italic)
+        return util.Cache.get(self, key)
+
+    def create_new_value(self, key):
+        (context, description, scale_factor, bold, italic) = key
+        return Font(context, description, scale_factor, bold, italic)
+
+_font_cache = FontCache(512)
 
 class LayoutManager(object):
     def __init__(self, widget):
@@ -53,9 +66,14 @@ class LayoutManager(object):
     def reset(self):
         self.current_font = self.font(1.0)
         self.text_color = (0, 0, 0)
+        self.text_shadow = None
 
     def on_style_set(self, widget, previous_style):
+        old_font_desc = self.style_font_desc
         self.update_style(widget.style)
+        if self.style_font_desc != old_font_desc:
+            # bug #17423 font changed, so the widget's width might have changed
+            widget.queue_resize()
 
     def on_direction_changed(self, widget, previous_direction):
         self.update_direction(widget.get_direction())
@@ -71,8 +89,8 @@ class LayoutManager(object):
             self.pango_context.set_base_dir(pango.DIRECTION_LTR)
 
     def font(self, scale_factor, bold=False, italic=False, family=None):
-        return Font(self.pango_context, self.style_font_desc, scale_factor,
-                bold, italic)
+        return _font_cache.get(self.pango_context, self.style_font_desc,
+                scale_factor, bold, italic)
 
     def set_font(self, scale_factor, bold=False, italic=False, family=None):
         self.current_font = self.font(scale_factor, bold, italic)
@@ -81,11 +99,11 @@ class LayoutManager(object):
         self.text_color = color
 
     def set_text_shadow(self, shadow):
-        self.shadow = shadow
+        self.text_shadow = shadow
 
     def textbox(self, text, underline=False):
         textbox = TextBox(self.pango_context, self.current_font,
-                self.text_color)
+                self.text_color, self.text_shadow)
         textbox.set_text(text, underline=underline)
         return textbox
 
@@ -127,16 +145,18 @@ class Font(object):
 
     def line_height(self):
         metrics = self.get_font_metrics()
-        return pango.PIXELS(metrics.get_ascent() + metrics.get_descent())
+        # the +1: some glyphs can be slightly taller than ascent+descent (#17329)
+        return pango.PIXELS(metrics.get_ascent() + metrics.get_descent()) + 1
 
 class TextBox(object):
-    def __init__(self, context, font, color):
+    def __init__(self, context, font, color, shadow):
         self.layout = pango.Layout(context)
         self.layout.set_wrap(pango.WRAP_WORD_CHAR)
         self.font = font
         self.color = color
         self.layout.set_font_description(font.description.copy())
-        self.width = None
+        self.width = self.height = None
+        self.shadow = shadow
 
     def set_text(self, text, font=None, color=None, underline=False):
         self.text_chunks = []
@@ -171,6 +191,14 @@ class TextBox(object):
             self.layout.set_width(-1)
         self.width = width
 
+    def set_height(self, height):
+        if height is not None:
+            # not sure why set_height isn't in the python bindings, but it
+            # isn't
+            pygtkhacks.set_pango_layout_height(self.layout,
+                int(height * pango.SCALE))
+        self.height = height
+
     def set_wrap_style(self, wrap):
         if wrap == 'word':
             self.layout.set_wrap(pango.WRAP_WORD_CHAR)
@@ -178,6 +206,10 @@ class TextBox(object):
             self.layout.set_wrap(pango.WRAP_CHAR)
         else:
             raise ValueError("Unknown wrap value: %s" % wrap)
+        if wrap == 'truncated-char':
+            self.layout.set_ellipsize(pango.ELLIPSIZE_END)
+        else:
+            self.layout.set_ellipsize(pango.ELLIPSIZE_NONE)
 
     def set_alignment(self, align):
         if align == 'left':
@@ -191,13 +223,31 @@ class TextBox(object):
 
     def ensure_layout(self):
         if not self.text_set:
-            self.layout.set_text(''.join(self.text_chunks))
+            text = ''.join(self.text_chunks)
+            if len(text) > 100:
+                text = text[:self._calc_text_cutoff()]
+            self.layout.set_text(text)
             attr_list = pango.AttrList()
             for attr in self.attributes:
                 attr_list.insert(attr)
             self.layout.set_attributes(attr_list)
             self.text_set = True
 
+    def _calc_text_cutoff(self):
+        """This method is a bit of a hack...  GTK slows down if we pass too
+        much text to the layout.  Even text that falls below our height has a
+        performance penalty.  Try not to have too much more than is necessary.
+        """
+        if None in (self.width, self.height):
+            return -1
+
+        chars_per_line = (self.width * pango.SCALE //
+                self.font.get_font_metrics().get_approximate_char_width())
+        lines_available = self.height // self.font.line_height()
+        # overestimate these because it's better to have too many characters
+        # than too little.
+        return int(chars_per_line * lines_available * 1.2)
+
     def line_count(self):
         self.ensure_layout()
         return self.layout.get_line_count()
@@ -231,15 +281,30 @@ class TextBox(object):
 
     def draw(self, context, x, y, width, height):
         self.set_width(width)
+        self.set_height(height)
         self.ensure_layout()
         cairo_context = context.context
         cairo_context.save()
-        cairo_context.set_source_rgb(*self.color)
         underline_drawer = UnderlineDrawer(self.underlines)
+        if self.shadow:
+            # draw shadow first so that it's underneath the regular text
+            # FIXME: we don't use the blur_radius setting
+            cairo_context.set_source_rgba(self.shadow.color[0],
+                    self.shadow.color[1], self.shadow.color[2],
+                    self.shadow.opacity)
+            self._draw_layout(context, x + self.shadow.offset[0], 
+                    y + self.shadow.offset[1], width, height,
+                    underline_drawer)
+        cairo_context.set_source_rgb(*self.color)
+        self._draw_layout(context, x, y, width, height, underline_drawer)
+        cairo_context.restore()
+        cairo_context.new_path()
+
+    def _draw_layout(self, context, x, y, width, height, underline_drawer):
         line_height = 0
         alignment = self.layout.get_alignment()
         for i in xrange(self.layout.get_line_count()):
-            line = self.layout.get_line(i)
+            line = self.layout.get_line_readonly(i)
             extents = line.get_pixel_extents()[1]
             next_line_height = line_height + extents[3]
             if next_line_height > height:
@@ -252,11 +317,9 @@ class TextBox(object):
                 line_x = x
             baseline = y + line_height + pango.ASCENT(extents)
             context.move_to(line_x, baseline)
-            cairo_context.show_layout_line(line)
+            context.context.show_layout_line(line)
             underline_drawer.draw(context, line_x, baseline, line)
             line_height = next_line_height
-        cairo_context.restore()
-        cairo_context.new_path()
 
 class UnderlineDrawer(object):
     """Class to draw our own underlines because cairo's don't look
@@ -428,7 +491,8 @@ class StyledButton(object):
         inner_width = width - radius * 2
         context.move_to(x + radius, y)
         context.rel_line_to(inner_width, 0)
-        context.arc(x + width - radius, y+radius, radius, -math.pi/2, math.pi/2)
+        context.arc(x + width - radius, y+radius, radius, -math.pi/2,
+                    math.pi/2)
         context.rel_line_to(-inner_width, 0)
         context.arc(x + radius, y+radius, radius, math.pi/2, -math.pi/2)
 
