@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,33 +33,31 @@ import time
 
 import gobject
 import gtk
-import logging
 
 from miro import app
 from miro import player
 from miro.gtcache import gettext as _
-from miro import signals
-from miro import util
 from miro import messages
-from miro import displaytext
 from miro.plat import resources
 from miro.plat import screensaver
 from miro.frontends.widgets.gtk.window import Window, WrappedWindow
-from miro.frontends.widgets.gtk.widgetset import (Widget, VBox, Label, HBox,
-                                                  Alignment, Background,
-                                                  DrawingArea, ImageSurface,
-                                                  Image, CustomButton)
+from miro.frontends.widgets.gtk.widgetset import (
+    Widget, VBox, Label, HBox, Alignment, Background, DrawingArea,
+    ClickableImageButton)
 from miro.frontends.widgets.gtk.persistentwindow import PersistentWindow
 
 BLACK = (0.0, 0.0, 0.0)
 WHITE = (1.0, 1.0, 1.0)
+GREEN = (159.0 / 255.0, 202.0 / 255.0, 120.0 / 255.0)
 
-# Global VideoWidget object.  We re-use so we can re-use our PersistentWindow
+# Global VideoWidget object.  We re-use so we can re-use our
+# PersistentWindow
 video_widget = None
 
+
 class ClickableLabel(Widget):
-    """This is like a label and reimplements many of the Label things, but
-    it's an EventBox with a Label child widget.
+    """This is like a label and reimplements many of the Label things,
+    but it's an EventBox with a Label child widget.
     """
     def __init__(self, text, size=0.77, color=WHITE):
         Widget.__init__(self)
@@ -81,7 +80,7 @@ class ClickableLabel(Widget):
         self.create_signal('clicked')
 
     def on_click(self, widget, event):
-        self.emit('clicked', event)
+        self.emit('clicked')
         return True
 
     def on_enter_notify(self, widget, event):
@@ -110,31 +109,6 @@ class ClickableLabel(Widget):
     def show(self):
         self.label._widget.show()
 
-class ClickableImageButton(CustomButton):
-    def __init__(self, image_path):
-        CustomButton.__init__(self)
-        self.image = ImageSurface(Image(image_path))
-
-        self.wrapped_widget_connect('enter-notify-event', self.on_enter_notify)
-        self.wrapped_widget_connect('leave-notify-event', self.on_leave_notify)
-        self.wrapped_widget_connect('button-release-event', self.on_click)
-
-    def size_request(self, layout):
-        return self.image.width, self.image.height
-
-    def draw(self, context, layout):
-        self.image.draw(context, 0, 0, self.image.width, self.image.height)
-
-    def on_click(self, widget, event):
-        self.emit('clicked', event)
-        return True
-
-    def on_enter_notify(self, widget, event):
-        self._widget.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND1))
-
-    def on_leave_notify(self, widget, event):
-        if self._widget.window:
-            self._widget.window.set_cursor(None)
 
 class NullRenderer:
     def __init__(self):
@@ -143,11 +117,13 @@ class NullRenderer:
     def reset(self):
         pass
 
+
 def make_hidden_cursor():
     pixmap = gtk.gdk.Pixmap(None, 1, 1, 1)
     color = gtk.gdk.Color()
     return gtk.gdk.Cursor(pixmap, pixmap, color, color, 0, 0)
 
+
 def make_label(text, handler, visible=True):
     if visible:
         lab = ClickableLabel(text)
@@ -161,11 +137,13 @@ def make_label(text, handler, visible=True):
     lab.set_size(0.77)
     return lab
 
+
 def make_image_button(image_path, handler):
     b = ClickableImageButton(resources.path(image_path))
     b.connect('clicked', handler)
     return b
 
+
 def _align_left(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     """Align left and pad."""
     alignment = Alignment(0, 0, 0, 1)
@@ -173,6 +151,7 @@ def _align_left(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     alignment.add(widget)
     return alignment
 
+
 def _align_right(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     """Align right and pad."""
     alignment = Alignment(1, 0, 0, 1)
@@ -180,6 +159,7 @@ def _align_right(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     alignment.add(widget)
     return alignment
 
+
 def _align_center(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     """Align center (horizontally) and pad."""
     alignment = Alignment(0.5, 0, 0, 1)
@@ -187,6 +167,7 @@ def _align_center(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     alignment.add(widget)
     return alignment
 
+
 def _align_middle(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     """Align center (vertically) and pad."""
     alignment = Alignment(0, 0.5, 0, 0)
@@ -194,11 +175,13 @@ def _align_middle(widget, top_pad=0, bottom_pad=0, left_pad=0, right_pad=0):
     alignment.add(widget)
     return alignment
 
-# Couple of utility functions to grab GTK widgets out of the widget tree for
-# fullscreen code
+
+# Couple of utility functions to grab GTK widgets out of the widget
+# tree for fullscreen code
 def _videobox_widget():
     return app.widgetapp.window.videobox._widget
 
+
 def _window():
     """Returns the window used for playback.  This is either the main window
     or the detached window.
@@ -207,6 +190,7 @@ def _window():
         return app.playback_manager.detached_window._window
     return app.widgetapp.window._window
 
+
 class VideoOverlay(Window):
     def __init__(self):
         Window.__init__(self, 'Miro Video Overlay')
@@ -229,6 +213,7 @@ class VideoOverlay(Window):
         window.move(screen_rect.x, screen_rect.y + screen_rect.height -
                 my_height)
 
+
 class VideoWidget(Widget):
     def __init__(self, renderer):
         Widget.__init__(self)
@@ -241,6 +226,7 @@ class VideoWidget(Widget):
     def destroy(self):
         self._widget.destroy()
 
+
 class Divider(DrawingArea):
     def size_request(self, layout):
         return (1, 25)
@@ -252,12 +238,21 @@ class Divider(DrawingArea):
         context.rel_line_to(0, context.height)
         context.stroke()
 
+
 class VideoDetailsWidget(Background):
     def __init__(self):
         Background.__init__(self)
         self.item_info = None
         self.rebuild_video_details()
         self._delete_link = None
+        self._delete_image = None
+        self._will_play_handle = app.playback_manager.connect(
+            'will-play', self.on_will_play)
+
+    def on_will_play(self, widget, duration):
+        # we need to update the video details now that the file is
+        # open and we know more about subtitle track info.
+        self.rebuild_video_details()
 
     def rebuild_video_details(self):
         # this removes the child widget if there is one
@@ -267,84 +262,98 @@ class VideoDetailsWidget(Background):
             self.add(HBox())
             return
 
-        info = self.item_info
-
-        outer_hbox = HBox(5)
-
-        if not info.is_external:
-            if info.expiration_date is not None:
-                text = displaytext.expiration_date(info.expiration_date)
-                self._expiration_label = Label(text)
-                self._expiration_label.set_size(0.77)
-                self._expiration_label.set_color((152.0 / 255.0, 152.0 / 255.0, 152.0 / 255.0))
-                outer_hbox.pack_start(_align_middle(self._expiration_label))
-                outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
-
-            lab = make_label(_("Keep"), self.handle_keep, info.expiration_date is not None)
-            outer_hbox.pack_start(_align_middle(lab))
-            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
-
-        self._subtitles_link = make_label(_("Subtitles"),
-                                          self.handle_subtitles)
-        outer_hbox.pack_start(_align_middle(self._subtitles_link))
-        subtitles_image = make_image_button('images/subtitles_down.png',
-                                            self.handle_subtitles)
-        outer_hbox.pack_start(_align_middle(subtitles_image))
-
-        outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
-
-        self._delete_link = make_label(_("Delete"), self.handle_delete)
-        outer_hbox.pack_start(_align_middle(self._delete_link))
-
-        if not info.is_external:
-            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
-
-            self._share_link = make_label(_("Share"), self.handle_share,
-                                          info.has_sharable_url)
-            outer_hbox.pack_start(_align_middle(self._share_link))
-            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
-
-            if info.commentslink:
-                self._permalink_link = make_label(_("Comments"),
-                                                  self.handle_commentslink,
-                                                  info.commentslink)
-            else:
-                self._permalink_link = make_label(_("Permalink"),
-                                                  self.handle_permalink,
-                                                  info.permalink)
-            outer_hbox.pack_start(_align_middle(self._permalink_link))
-
-        outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
+        left_side_hbox = HBox(5)
+        right_side_hbox = HBox(5)
 
+        # fullscreen
         if app.playback_manager.is_fullscreen:
-            fullscreen_link = make_label(_("Exit fullscreen"),
-                                         self.handle_fullscreen)
-            outer_hbox.pack_start(_align_middle(fullscreen_link))
-            fullscreen_image = make_image_button('images/fullscreen_exit.png',
-                                                 self.handle_fullscreen)
-            outer_hbox.pack_start(_align_middle(fullscreen_image))
+            fullscreen_image = make_image_button(
+                'images/fullscreen_exit.png', self.handle_fullscreen)
         else:
-            fullscreen_link = make_label(_("Fullscreen"),
-                                         self.handle_fullscreen)
-            outer_hbox.pack_start(_align_middle(fullscreen_link))
-            fullscreen_image = make_image_button('images/fullscreen_enter.png',
-                                                 self.handle_fullscreen)
-            outer_hbox.pack_start(_align_middle(fullscreen_image))
+            fullscreen_image = make_image_button(
+                'images/fullscreen_enter.png', self.handle_fullscreen)
+
+        fullscreen_link = make_label(_("Fullscreen"), self.handle_fullscreen)
+        left_side_hbox.pack_start(_align_middle(fullscreen_image))
+        left_side_hbox.pack_start(_align_middle(fullscreen_link))
+
+        left_side_hbox.pack_start(_align_middle(
+                Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
 
         if app.playback_manager.detached_window is not None:
-            popin_link = make_label(_("Pop-in"), self.handle_popin_popout)
-            outer_hbox.pack_start(_align_middle(popin_link))
-            popin_image = make_image_button('images/popin.png',
-                                            self.handle_popin_popout)
-            outer_hbox.pack_start(_align_middle(popin_image))
+            pop_link = make_label(
+                _("Pop-in"), self.handle_popin_popout)
+            # FIXME - need popin image
+            pop_image = make_image_button(
+                'images/popin.png', self.handle_popin_popout)
         else:
-            popout_link = make_label(_("Pop-out"), self.handle_popin_popout)
-            outer_hbox.pack_start(_align_middle(popout_link))
-            popout_image = make_image_button('images/popout.png',
-                                             self.handle_popin_popout)
-            outer_hbox.pack_start(_align_middle(popout_image))
+            pop_link = make_label(
+                _("Pop-out"), self.handle_popin_popout)
+            pop_image = make_image_button(
+                'images/popout.png', self.handle_popin_popout)
 
-        self.add(_align_right(outer_hbox, left_pad=15, right_pad=15))
+        left_side_hbox.pack_start(_align_middle(pop_image))
+        left_side_hbox.pack_start(_align_middle(pop_link))
+
+        left_side_hbox.pack_start(_align_middle(
+                Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
+
+
+        right_side_hbox.pack_start(_align_middle(
+                Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
+
+        right_side_hbox.pack_start(self.make_subtitles_button())
+
+        right_side_hbox.pack_start(_align_middle(
+                Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
+
+        self._delete_image = make_image_button(
+            'images/delete.png', self.handle_delete)
+        right_side_hbox.pack_start(_align_middle(self._delete_image))
+        self._delete_link = make_label(_("Delete"), self.handle_delete)
+        right_side_hbox.pack_start(_align_middle(self._delete_link))
+
+        outer_hbox = HBox()
+        outer_hbox.pack_start(_align_left(left_side_hbox, left_pad=10),
+                              expand=True)
+        outer_hbox.pack_start(_align_right(right_side_hbox, right_pad=10))
+        self.add(outer_hbox)
+
+        # self.add(_align_right(outer_hbox, left_pad=15, right_pad=15))
+
+
+    def make_subtitles_button(self):
+        hbox = HBox(5)
+
+        current_track = app.video_renderer.get_enabled_subtitle_track()
+
+        # None, -1 and 0 all mean there is no current track.
+        if current_track is not None and current_track > 0:
+            ccimage = 'images/cc-on.png'
+            cccolor = GREEN
+            cctext = _("Subtitles On")
+
+        else:
+            tracks = app.video_renderer.get_subtitle_tracks()
+            if tracks is not None and len(tracks) > 0:
+                ccimage = 'images/cc-available.png'
+                cctext = _("Subtitles Found")
+                cccolor = WHITE
+            else:
+                ccimage = 'images/cc-available.png'
+                cctext = _("Subtitles")
+                cccolor = WHITE
+
+        cc_image_button = make_image_button(ccimage, self.handle_subtitles)
+        hbox.pack_start(_align_middle(cc_image_button))
+        subtitles_link = make_label(cctext, self.handle_subtitles)
+        subtitles_link.set_color(cccolor)
+        hbox.pack_start(_align_middle(subtitles_link))
+
+        subtitles_image = make_image_button(
+            'images/subtitles_down.png', self.handle_subtitles)
+        hbox.pack_start(_align_middle(subtitles_image))
+        return hbox
 
     def hide(self):
         self._widget.hide()
@@ -352,25 +361,24 @@ class VideoDetailsWidget(Background):
     def show(self):
         self._widget.show()
 
-    def handle_fullscreen(self, widget, event=None):
+    def handle_fullscreen(self, widget):
         app.playback_manager.toggle_fullscreen()
 
-    def handle_popin_popout(self, widget, event=None):
+    def handle_popin_popout(self, widget):
         if app.playback_manager.is_fullscreen:
             app.playback_manager.exit_fullscreen()
         app.playback_manager.toggle_detached_mode()
 
-    def handle_keep(self, widget, event):
+    def handle_keep(self, widget):
         messages.KeepVideo(self.item_info.id).send_to_backend()
         self._widget.window.set_cursor(None)
 
-    def handle_delete(self, widget, event):
+    def handle_delete(self, widget):
         item_info = self.item_info
-        self.reset()
-        app.playback_manager.on_movie_finished()
         app.widgetapp.remove_items([item_info])
+        self.reset()
 
-    def handle_subtitles(self, widget, event):
+    def handle_subtitles(self, widget):
         tracks = []
         menu = gtk.Menu()
 
@@ -411,22 +419,25 @@ class VideoDetailsWidget(Background):
         menu.append(sep)
 
         child = gtk.MenuItem(_("Select a Subtitles file..."))
-        child.set_sensitive(True)
+        sensitive = 'PlayingLocalVideo' in app.menu_manager.enabled_groups
+        child.set_sensitive(sensitive)
         child.connect('activate', self.handle_select_subtitle_file)
         child.show()
         menu.append(child)
 
-        menu.popup(None, None, None, event.button, event.time)
+        menu.popup(None, None, None, 1, gtk.get_current_event_time())
 
     def handle_disable_subtitles(self, widget):
         if widget.active:
             app.video_renderer.disable_subtitles()
             app.widgetapp.window.on_playback_change(app.playback_manager)
+            self.rebuild_video_details()
 
     def handle_subtitle_change(self, widget, index):
         if widget.active:
             app.video_renderer.enable_subtitle_track(index)
             app.widgetapp.window.on_playback_change(app.playback_manager)
+            self.rebuild_video_details()
 
     def handle_select_subtitle_file(self, widget):
         app.playback_manager.open_subtitle_file()
@@ -467,12 +478,15 @@ class VideoDetailsWidget(Background):
     def reset(self):
         if self._delete_link:
             self._delete_link.on_leave_notify(None, None)
+        if self._delete_image:
+            self._delete_image.on_leave_notify(None, None)
 
 class VideoPlayer(player.Player, VBox):
     """Video renderer widget.
 
-    Note: ``app.video_renderer`` must be initialized before instantiating this
-    class.  If no renderers can be found, set ``app.video_renderer`` to ``None``.
+    Note: ``app.video_renderer`` must be initialized before
+    instantiating this class.  If no renderers can be found, set
+    ``app.video_renderer`` to ``None``.
     """
     HIDE_CONTROLS_TIMEOUT = 2000
 
@@ -486,6 +500,7 @@ class VideoPlayer(player.Player, VBox):
             self.renderer = NullRenderer()
 
         self.overlay = None
+        self.screensaver_manager = None
 
         if video_widget is None:
             video_widget = VideoWidget(self.renderer)
@@ -504,7 +519,8 @@ class VideoPlayer(player.Player, VBox):
                 self._on_items_changed)
         self._item_id = None
 
-        self._video_widget.wrapped_widget_connect('button-press-event', self.on_button_press)
+        self._video_widget.wrapped_widget_connect(
+            'button-press-event', self.on_button_press)
 
     def teardown(self):
         self.renderer.reset()
@@ -562,14 +578,14 @@ class VideoPlayer(player.Player, VBox):
         duration = self.get_total_playback_time()
         if duration is None:
             return
-        time = duration * position
-        self.seek_to_time(time)
+        self.seek_to_time(duration * position)
 
     def seek_to_time(self, time_pos):
         self.renderer.set_current_time(time_pos)
 
     def enter_fullscreen(self):
-        self.screensaver_manager = screensaver.create_manager()
+        self.screensaver_manager = screensaver.create_manager(
+                app.widgetapp.window._window)
         if self.screensaver_manager is not None:
             self.screensaver_manager.disable()
         self.rebuild_video_details()
@@ -583,7 +599,7 @@ class VideoPlayer(player.Player, VBox):
         self.schedule_hide_controls(self.HIDE_CONTROLS_TIMEOUT)
         # make sure all hide() calls go through, otherwise we get the wrong
         # size on windows (#10810)
-        while gtk.events_pending(): 
+        while gtk.events_pending():
             gtk.main_iteration()
         _window().fullscreen()
 
@@ -598,14 +614,15 @@ class VideoPlayer(player.Player, VBox):
         self.overlay.show()
 
     def _destroy_overlay(self):
-        main_window = app.widgetapp.window
-        self.overlay.vbox.remove(self._video_details)
-        self.overlay.vbox.remove(main_window.controls_hbox)
-        self.pack_start(self._video_details)
-        main_window.main_vbox.pack_start(main_window.controls_hbox)
+        if self.overlay:
+            main_window = app.widgetapp.window
+            self.overlay.vbox.remove(self._video_details)
+            self.overlay.vbox.remove(main_window.controls_hbox)
+            self.pack_start(self._video_details)
+            main_window.main_vbox.pack_start(main_window.controls_hbox)
 
-        self.overlay.destroy()
-        self.overlay = None
+            self.overlay.destroy()
+            self.overlay = None
 
     def rebuild_video_details(self):
         self._video_details.rebuild_video_details()
@@ -626,18 +643,48 @@ class VideoPlayer(player.Player, VBox):
         if not self.overlay:
             return
         if not self.overlay.is_visible():
-            self.show_controls()
+            show_it_all = False
+
+            if event is None:
+                show_it_all = True
+            else:
+                # figures out the monitor that miro is fullscreened on and
+                # gets the monitor geometry for that.
+                # if app.playback_manager.detached_window is not None:
+                    # gtkwindow = app.playback_manager.detached_window._window
+                # else:
+                    # gtkwindow = app.widgetapp.window._window
+                # gdkwindow = gtkwindow.window
+                # screen = gtkwindow.get_screen()
+
+                # monitor = screen.get_monitor_at_window(gdkwindow)
+                # monitor_geom = screen.get_monitor_geometry(monitor)
+                # if event.y > monitor_geom.height - 200:
+                    # show_it_all = True
+
+                # Hack to fix #17213.  Eventually we should remove this and
+                # uncomment the code above to implement #8655
+                show_it_all = True
+
+            if show_it_all:
+                self.show_controls()
+            else:
+                self.show_mouse()
             self.schedule_hide_controls(self.HIDE_CONTROLS_TIMEOUT)
         else:
             self.last_motion_time = time.time()
 
-    def show_controls(self):
+    def show_mouse(self):
         _window().window.set_cursor(None)
+
+    def show_controls(self):
+        self.show_mouse()
         self.overlay.show()
 
     def hide_controls(self):
         _window().window.set_cursor(self.hidden_cursor)
-        self.overlay.close()
+        if self.overlay and self.overlay.is_visible():
+            self.overlay.close()
 
     def on_hide_controls_timeout(self):
         # Check if the mouse moved before the timeout
@@ -654,6 +701,8 @@ class VideoPlayer(player.Player, VBox):
             gobject.source_remove(self.hide_controls_timeout)
 
     def schedule_hide_controls(self, time):
+        if self.hide_controls_timeout is not None:
+            gobject.source_remove(self.hide_controls_timeout)
         self.hide_controls_timeout = gobject.timeout_add(time,
                 self.on_hide_controls_timeout)
         self.last_motion_time = 0
@@ -673,9 +722,9 @@ class VideoPlayer(player.Player, VBox):
 
     def select_subtitle_file(self, sub_path, handle_successful_select):
         app.video_renderer.select_subtitle_file(
-            app.playback_manager.get_playing_item(), 
+            app.playback_manager.get_playing_item(),
             sub_path,
             handle_successful_select)
-        
+
     def select_subtitle_encoding(self, encoding):
         app.video_renderer.select_subtitle_encoding(encoding)
