@@ -318,6 +318,10 @@ class PlaybackManager (signals.SignalEmitter):
                 self.playlist.set_shuffle(self.shuffle)
             self.emit('update-shuffle')
 
+    def reshuffle(self):
+        if self.playlist:
+            self.playlist.reshuffle()
+
     def toggle_repeat(self):
         if self.repeat == WidgetStateStore.get_repeat_playlist():
             self.set_repeat(WidgetStateStore.get_repeat_track())
@@ -757,14 +761,35 @@ class PlaybackPlaylist(signals.SignalEmitter):
         self.model = None
         self.disconnect_all()
 
+    # This recalculates the upcoming shuffle items.
+    # This needs to be done in case the playlist changes
+    # without any items-changed signal being emitted.
+    # Note that the new upcoming shuffle items do not take
+    # already played items into consideration.
+    def reshuffle(self):
+        # only reshuffle if we are currently using self.shuffle_upcoming
+        if self.shuffle:
+            self.shuffle_upcoming = self.generate_upcoming_shuffle_items()
+            # Remove any items in the shuffle history which are not currently
+            # in the playlist. The last item isn't removed since that is the 
+            # currently playing item.
+            if self.shuffle_history:
+                playable_items = set(self.get_all_playable_items())
+                current_item = self.shuffle_history.pop()
+                self.shuffle_history = [item for item in self.shuffle_history 
+                                                    if item in playable_items]
+                self.shuffle_history.append(current_item)
+
     def prev_shuffle_item(self):
         while len(self.shuffle_history) > 0:
             try:
                 return self.model.get_info(self.shuffle_history[-1])
             except KeyError:
-                # Item was removed from our InfoList by a ItemList filter
-                # (#17500).  Try the previous item in the list.
+                # Item was removed from our InfoList and the shuffle history
+                # was not updated. This should not happen.
                 self.shuffle_history.pop()
+                logging.warning("trying to play non-existent shuffle history "
+                                 "item, skipping to previous item.")
                 continue
         # no items in our history, return None
         return
@@ -775,8 +800,11 @@ class PlaybackPlaylist(signals.SignalEmitter):
             try:
                 return self.model.get_info(next_id)
             except KeyError:
-                # Item was removed from our InfoList by a ItemList filter
-                # (#17500).  Try the next item in the list.
+                # Item was removed from our InfoList and the upcoming shuffle
+                # items were not updated. This should not happen.
+                logging.warning("trying to play non-existent upcoming shuffle "
+                                "item, skipping to next item.")
+
                 continue
         # no items left in shuffle_upcoming
         return None
@@ -799,6 +827,10 @@ class PlaybackPlaylist(signals.SignalEmitter):
                 self.shuffle_history = []
                 return None #stop playback 
             else:
+                # Remove currently playing item from history if it
+                # was removed from the playlist.
+                if self._is_playing_filtered_item():
+                    self.shuffle_history.pop()
                 self.shuffle_history.append(next_item.id)
                 return next_item
         elif self.shuffle and WidgetStateStore.get_repeat_playlist():
@@ -810,6 +842,10 @@ class PlaybackPlaylist(signals.SignalEmitter):
                 if next_item is None:
                     #17492 - nothing playable in list
                     return None
+            # Remove currently playing item from history if it
+            # was removed from the playlist.
+            if self._is_playing_filtered_item():
+                self.shuffle_history.pop()
             self.shuffle_history.append(next_item.id)
             return next_item
         else:
@@ -824,7 +860,10 @@ class PlaybackPlaylist(signals.SignalEmitter):
             if not self.shuffle_history:
                 return None
             current_item = self.shuffle_history.pop()
-            self.shuffle_upcoming.append(current_item)
+            # Only add the currently playing item to upcoming shuffle items
+            # if it exists in the playlist
+            if not self._is_playing_filtered_item():
+                self.shuffle_upcoming.append(current_item)
             return self.prev_shuffle_item()
         elif (not self.shuffle 
               and self.repeat == WidgetStateStore.get_repeat_playlist()
