@@ -307,6 +307,13 @@ class TabList(signals.SignalEmitter):
         with self.removing():
             iter_ = self.iter_map.pop(name)
             self.view.model.remove(iter_)
+        if name in app.tabs.selected_ids:
+            # hack for 17653: on OS X, deleting the selected tab doesn't
+            # send selection-changed - so if the tab(s) we've 
+            # deleted is selected, explicitly change the selection to
+            # this list's root. This preempts GTK's handling of this 
+            # case, which is behaviorally the same.
+            app.tabs._handle_no_tabs_selected(self)
 
     def get_tab(self, name):
         return self.view.model[self.iter_map[name]][0]
@@ -666,6 +673,7 @@ class HideableTabList(TabList):
             self.emit('tab-name-changed', old_name, info.name)
 
     def remove(self, id_list):
+        deleted_ids = []
         with self.removing():
             for id_ in id_list:
                 try:
@@ -673,16 +681,31 @@ class HideableTabList(TabList):
                 except KeyError:
                     # child of a tab we already deleted
                     continue
-                self.forget_child_iters(iter_)
+                # override default of nil
+                deleted_ids = self.forget_child_iters(iter_)
                 self.view.model.remove(iter_)
+        if set(deleted_ids + id_list).intersection(set(app.tabs.selected_ids)):
+            # hack for 17653: on OS X, deleting the selected tab doesn't
+            # send selection-changed - so if the tab(s) we've 
+            # deleted is selected, explicitly change the selection to
+            # this list's root. This preempts GTK's handling of this 
+            # case, which is behaviorally the same.
+            app.tabs._handle_no_tabs_selected(self)
 
     def forget_child_iters(self, parent_iter):
         model = self.view.model
         iter_ = model.child_iter(parent_iter)
+        deleted_ids = []
         while iter_ is not None:
+            # No need to wrap these around a try...except.  If we can get
+            # the child iter (from above) then this must exist.  If it doesn't
+            # then there is some sort of internal inconsistency.
             id_ = model[iter_][0].id
             del self.iter_map[id_]
-            iter_ = model.next_iter(iter_)
+            deleted_ids += self.forget_child_iters(iter_)
+            iter_ = self.view.model.remove(iter_)
+            deleted_ids.append(id_)
+        return deleted_ids
 
     def model_changed(self):
         self.view.model_changed()
@@ -832,27 +855,37 @@ class SharingTabListHandler(object):
     def init_info(self, info):
         info.type = u'sharing'
         info.unwatched = info.available = 0
-        info.video_playlist_id = unicode(md5(
-                                    repr((u'video',
-                                    info.host,
-                                    info.port, u'video'))).hexdigest())
-        info.audio_playlist_id = unicode(md5(
-                                    repr((u'audio',
-                                    info.host,
-                                    info.port, u'audio'))).hexdigest())
-        if info.is_folder:
+        active = None
+        if info.is_folder and info.playlist_id is None:
             thumb_path = resources.path('images/sharing.png')
         # Checking the name instead of a supposedly unique id is ok for now
         # because
         elif info.playlist_id == u'video':
             thumb_path = resources.path('images/icon-video.png')
+            active = resources.path('images/icon-video_active.png')
             info.name = _('Video')
         elif info.playlist_id == u'audio':
             thumb_path = resources.path('images/icon-audio.png')
+            active = resources.path('images/icon-audio_active.png')
             info.name = _('Music')
-        else:
+        elif info.playlist_id == u'playlist':
             thumb_path = resources.path('images/icon-playlist.png')
+            active = resources.path('images/icon-playlist_active.png')
+            info.name = _('Playlists')
+        elif info.playlist_id == u'podcast':
+            thumb_path = resources.path('images/icon-podcast.png')
+            active = resources.path('images/icon-podcast_active.png')
+            info.name = _('Podcasts')
+        else:
+            if info.podcast:
+                thumb_path = resources.path('images/icon-podcast-small.png')
+                active = resources.path('images/icon-podcast-small_active.png')
+            else:
+                thumb_path = resources.path('images/icon-playlist-small.png')
+                active = resources.path('images/icon-playlist-small_active.png')
         info.icon = imagepool.get_surface(thumb_path)
+        if active:
+            info.active_icon = imagepool.get_surface(active)
 
 class ConnectList(TabUpdaterMixin, HideableTabList):
     name = _('Connect')
