@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,13 +31,18 @@
 video controls).
 """
 
+import logging
+
 from miro import app
 from miro import displaytext
+from miro.gtcache import gettext as _
 from miro.frontends.widgets import style
 from miro.frontends.widgets import imagepool
 from miro.frontends.widgets import widgetutil
 from miro.frontends.widgets import imagebutton
+from miro.frontends.widgets import separator
 from miro.frontends.widgets.widgetconst import MAX_VOLUME
+from miro.frontends.widgets.widgetstatestore import WidgetStateStore
 from miro.plat.frontends.widgets import widgetset
 from miro.plat import resources
 
@@ -47,11 +53,9 @@ class PlaybackControls(widgetset.HBox):
         self.stop = self.make_button('stop', False)
         self.play = self.make_button('play', False)
         self.forward = self.make_button('skip_forward', True)
-        self.fullscreen = self.make_button('play_fullscreen', True)
         self.pack_start(widgetutil.align_middle(self.previous))
         self.pack_start(widgetutil.align_middle(self.stop))
         self.pack_start(widgetutil.align_middle(self.play))
-        self.pack_start(widgetutil.align_middle(self.fullscreen))
         self.pack_start(widgetutil.align_middle(self.forward))
         app.playback_manager.connect('selecting-file', self.handle_selecting)
         app.playback_manager.connect('will-play', self.handle_play)
@@ -64,6 +68,7 @@ class PlaybackControls(widgetset.HBox):
             button.set_delays(0.6, 0.3)
         else:
             button = imagebutton.ImageButton(name)
+        button.set_can_focus(False)
         button.disable()
         return button
     
@@ -79,8 +84,6 @@ class PlaybackControls(widgetset.HBox):
         self.play.disable()
         self.play.set_image('pause')
         self.forward.enable()
-        if item_info.file_type == 'video':
-            self.fullscreen.enable()
         self.queue_redraw()
     
     def handle_play(self, obj, duration):
@@ -89,8 +92,6 @@ class PlaybackControls(widgetset.HBox):
         self.play.set_image('pause')
         self.play.enable()
         self.forward.enable()
-        if not obj.is_playing_audio:
-            self.fullscreen.enable()
         self.queue_redraw()
 
     def handle_pause(self, obj):
@@ -103,27 +104,36 @@ class PlaybackControls(widgetset.HBox):
         self.stop.disable()
         self.play.disable()
         self.forward.disable()
-        self.fullscreen.disable()
         self.queue_redraw()
 
-class PlaybackInfo(widgetset.DrawingArea):
+class PlaybackInfo(widgetset.CustomButton):
     LEFT_MARGIN = 8
     RIGHT_MARGIN = 8
     TOTAL_MARGIN = LEFT_MARGIN + RIGHT_MARGIN
     
     def __init__(self):
-        widgetset.DrawingArea.__init__(self)
+        widgetset.CustomButton.__init__(self)
+        self.set_can_focus(False)
         self.video_icon = imagepool.get_surface(resources.path('images/mini-icon-video.png'))
         self.audio_icon = imagepool.get_surface(resources.path('images/mini-icon-audio.png'))
         self.reset()
-        app.playback_manager.connect('selecting-file', self.handle_selecting)
+        app.playback_manager.connect('selecting-file', self.on_info_change)
+        app.playback_manager.connect('playing-info-changed',
+                self.on_info_change)
         app.playback_manager.connect('will-play', self.handle_play)
         app.playback_manager.connect('will-stop', self.handle_stop)
 
-    def handle_selecting(self, obj, item_info):
+    def on_info_change(self, obj, item_info):
         self.item_name = item_info.name
         self.feed_name = item_info.feed_name
+        self.is_feed = not item_info.is_external
+        self.album = item_info.album
+        self.artist = item_info.artist
+        # XXX Possibly dodgy?  What about other if we choose to allow playback
+        # in future?
         self.is_audio = (item_info.file_type == 'audio')
+        self.is_video = (item_info.file_type == 'video')
+        self.queue_redraw()
 
     def handle_play(self, obj, duration):
         self.queue_redraw()
@@ -134,19 +144,33 @@ class PlaybackInfo(widgetset.DrawingArea):
     
     def reset(self):
         self.item_name = ""
-        self.feed_name = ""
-        self.is_audio = False
-    
+        self.feed_name = self.album = self.artist = None
+        self.is_audio = self.is_video = self.is_feed = None
+
+    def get_details(self):
+        if self.feed_name and self.is_feed:
+            details = self.feed_name
+        elif self.is_audio:
+            # non-feed audio ~= music
+            album = self.album or _("Unknown Album")
+            artist = self.artist or _("Unknown Artist")
+            details = '%s - %s' % (album, artist)
+        else:
+            details = None
+        return details
+
     def get_full_text(self):
-        if self.feed_name is None:
+        details = self.get_details()
+        if details:
+            return '%s - %s' % (self.item_name, details)
+        else:
             return self.item_name
-        return "%s - %s" % (self.item_name, self.feed_name)
     
     def size_request(self, layout):
         layout.set_font(0.8)
         sizer_text = layout.textbox(self.get_full_text())
         width, height = sizer_text.get_size()
-        if self.is_audio:
+        if self.is_audio or self.is_video:
             width = width + 20
         return width, height
 
@@ -163,6 +187,9 @@ class PlaybackInfo(widgetset.DrawingArea):
         if self.is_audio:        
             self.audio_icon.draw(context, x, 0, 15, 12, 1.0)
             x = x + 20
+        elif self.is_video:
+            self.video_icon.draw(context, x, 0, 15, 12, 1.0)
+            x = x + 20
 
         layout.set_text_color((0.9, 0.9, 0.9))
         text = layout.textbox(self.item_name)
@@ -172,9 +199,16 @@ class PlaybackInfo(widgetset.DrawingArea):
         text.set_width(width1)
         text.draw(context, x, 0, width1, height1)
 
-        if self.feed_name is not None:
+        details = self.get_details()
+        if details:
+            if self.get_window().is_active():
+                layout.set_text_color((0.7, 0.7, 0.7))
+            else:
+                layout.set_text_color((0.9, 0.9, 0.9))
+            # XXX bz:17136 override the color: Windows things we are inactive
+            # embedded web browser is displayed.
             layout.set_text_color((0.7, 0.7, 0.7))
-            text = layout.textbox(" - %s" % self.feed_name)
+            text = layout.textbox(" - %s" % details)
             width2, height2 = text.get_size()
             width2 = min(width2, context.width - self.TOTAL_MARGIN - width1 - x)
             text.set_wrap_style('truncated-char')
@@ -218,8 +252,11 @@ class ProgressTime(widgetset.DrawingArea):
             text.draw(context, context.width-width, 0, width, height)
 
 class ProgressTimeRemaining(widgetset.CustomButton):
+    PADDING_LEFT = 10
+
     def __init__(self):
         widgetset.CustomButton.__init__(self)
+        self.set_can_focus(False)
         self.duration = self.current_time = None
         self.display_remaining = True
         app.playback_manager.connect('selecting-file', self.handle_selecting)
@@ -230,7 +267,8 @@ class ProgressTimeRemaining(widgetset.CustomButton):
     def size_request(self, layout):
         layout.set_font(0.75)
         sizer_text = layout.textbox('-9999:99')
-        return sizer_text.get_size()
+        width, height = sizer_text.get_size()
+        return width + self.PADDING_LEFT, height
 
     def handle_play(self, obj, duration):
         self.set_duration(duration)
@@ -272,14 +310,19 @@ class ProgressTimeRemaining(widgetset.CustomButton):
         layout.set_text_color(widgetutil.WHITE)
         text = layout.textbox(text)
         width, height = text.get_size()
-        text.draw(context, 10, 0, width, height)
+        text.draw(context, self.PADDING_LEFT, 0, width, height)
 
 class ProgressSlider(widgetset.CustomSlider):
     def __init__(self):
         widgetset.CustomSlider.__init__(self)
+        self.set_can_focus(False)
         self.background_surface = widgetutil.ThreeImageSurface('playback_track')
         self.progress_surface = widgetutil.ThreeImageSurface('playback_track_progress')
         self.progress_cursor = widgetutil.make_surface('playback_cursor')
+        self.background_surface_inactive = widgetutil.ThreeImageSurface('playback_track_inactive')
+        self.progress_surface_inactive = widgetutil.ThreeImageSurface('playback_track_progress_inactive')
+        self.progress_cursor_inactive = widgetutil.make_surface('playback_cursor_inactive')
+
         app.playback_manager.connect('playback-did-progress', self.handle_progress)
         app.playback_manager.connect('selecting-file', self.handle_selecting)
         app.playback_manager.connect('will-play', self.handle_play)
@@ -314,7 +357,7 @@ class ProgressSlider(widgetset.CustomSlider):
         return False
 
     def size_request(self, layout):
-        return (60, 11)
+        return (60, 17)
 
     def slider_size(self):
         return 1
@@ -322,18 +365,29 @@ class ProgressSlider(widgetset.CustomSlider):
     def draw(self, context, layout):
         if not app.playback_manager.is_playing:
             return
-        min, max = self.get_range()
-        progress_width = int(round(self.get_value() / max * context.width))
-        self.progress_surface.draw(context, 0, 0, progress_width)
-        if progress_width == 0:
-            self.background_surface.draw(context, 0, 0, context.width)
+        if self.get_window().is_active():
+            background, progress, cursor = (self.background_surface,
+                                            self.progress_surface,
+                                            self.progress_cursor)
         else:
-            self.background_surface.draw_right(context, progress_width, 0, context.width - progress_width)
+            background, progress, cursor = (self.background_surface_inactive,
+                                            self.progress_surface_inactive,
+                                            self.progress_cursor_inactive)
+
+        # XXX bz:17136 override the color: Windows things we are inactive
+        # embedded web browser is displayed.
+        background, progress, cursor = (self.background_surface,
+                                        self.progress_surface,
+                                        self.progress_cursor)
+        min, max = self.get_range()
+        scale = (self.get_value() - min) / (max - min)
+        cursor_pos = int(scale * (context.width - 18))
+        background.draw(context, 0, 1, context.width, context.height - 1)
+        if cursor_pos:
+            progress.draw(context, 0, 1, cursor_pos + 9)
+
         if self.duration > 0:
-            if progress_width <= 3:
-                self.progress_cursor.draw(context, 0, 0, self.progress_cursor.width, self.progress_cursor.height)
-            else:
-                self.progress_cursor.draw(context, progress_width-3, 0, self.progress_cursor.width, self.progress_cursor.height)
+            cursor.draw(context, cursor_pos, 0, cursor.width, cursor.height)
 
 class ProgressTimeline(widgetset.Background):
     def __init__(self):
@@ -348,6 +402,11 @@ class ProgressTimeline(widgetset.Background):
         self.slider.connect('released', self.on_slider_released)
         self.remaining_time = ProgressTimeRemaining()
         self.remaining_time.connect('clicked', self.on_remaining_clicked)
+
+        self.active = widgetutil.ThreeImageSurface('progress_timeline')
+        self.inactive = widgetutil.ThreeImageSurface(
+            'progress_timeline_inactive')
+
         vbox = widgetset.VBox()
         vbox.pack_start(widgetutil.align_middle(self.info, top_pad=6))
         slider_box = widgetset.HBox()
@@ -379,35 +438,25 @@ class ProgressTimeline(widgetset.Background):
         self.time.set_current_time(current_time)
         self.remaining_time.set_current_time(current_time)
 
-    def get_height(self, layout):
-        # this assumes that the height is 14 pixels of space plus the
-        # height of two labels--one for the time and one for the
-        # title.  we do the max thing so there's a minimum height that
-        # comes out of this.
-        layout.set_font(0.8)
-        sizer_text = layout.textbox("Foo")
-        dummy, height = sizer_text.get_size()
-        return max(40, 14 + (height * 2))
-
     def size_request(self, layout):
-        layout.set_font(0.8)
-        sizer_text = layout.textbox("Foo")
-        dummy, height = sizer_text.get_size()
-        return -1, self.get_height(layout)
+        return -1, 46
 
     def draw(self, context, layout):
         if self.get_window().is_active():
-            c = (40.0 / 255.0, 40.0 / 255.0, 40.0 / 255.0)
+            surface = self.active
         else:
-            c = (145.0 / 255.0, 145.0 / 255.0, 145.0 / 255.0)
+            surface = self.inactive
+        # XXX bz:17136 override the color: Windows things we are inactive
+        # embedded web browser is displayed.
+        surface = self.active
+
+        surface.draw(context, 0, 0, context.width, 46)
 
-        widgetutil.round_rect(context, 0, 0, context.width, context.height, 5)
-        context.set_color(c)
-        context.fill()
 
 class VolumeSlider(widgetset.CustomSlider):
     def __init__(self):
         widgetset.CustomSlider.__init__(self)
+        self.set_can_focus(False)
         self.set_range(0.0, MAX_VOLUME)
         self.set_increments(0.05, 0.20)
         self.track = widgetutil.make_surface('volume_track')
@@ -420,7 +469,7 @@ class VolumeSlider(widgetset.CustomSlider):
         return True
 
     def size_request(self, layout):
-        return (self.track.width, self.knob.height)
+        return (self.track.width, max(self.track.height, self.knob.height))
 
     def slider_size(self):
         return self.knob.width
@@ -446,24 +495,106 @@ class VideoBox(style.LowerBox):
         style.LowerBox.__init__(self)
         self.controls = PlaybackControls()
         self.timeline = ProgressTimeline()
-        if hasattr(widgetset, 'VolumeMuter'):
-            self.volume_muter = widgetset.VolumeMuter()
-        else:
-            self.volume_muter = imagebutton.ImageButton('volume')
-        if hasattr(widgetset, 'VolumeSlider'):
-            self.volume_slider = widgetset.VolumeSlider()
-        else:
-            self.volume_slider = VolumeSlider()
+        app.playback_manager.connect('will-start', self.on_playback_started)
+        app.playback_manager.connect('did-stop', self.on_playback_stopped)
+        app.playback_manager.connect('selecting-file', self.on_file_selected)
+        self.timeline.info.connect('clicked', self.on_title_clicked)
+        self.playback_mode = PlaybackModeControls()
+        self.volume_muter = imagebutton.ImageButton('volume')
+        self.volume_muter.set_can_focus(False)
+        self.volume_slider = VolumeSlider()
         self.time_slider = self.timeline.slider
 
         hbox = widgetset.HBox(spacing=20)
         hbox.pack_start(self.controls, expand=False)
-        hbox.pack_start(widgetutil.align_middle(self.timeline), expand=True)
+        hbox.pack_start(widgetutil.align_middle(self.timeline),
+                        expand=True)
         volume_hbox = widgetset.HBox(spacing=4)
         volume_hbox.pack_start(widgetutil.align_middle(self.volume_muter))
         volume_hbox.pack_start(widgetutil.align_middle(self.volume_slider))
         hbox.pack_start(volume_hbox)
-        self.add(widgetutil.align_middle(hbox, 0, 0, 25, 25))
+        hbox.pack_start(self.playback_mode)
+
+        vbox = widgetset.VBox()
+        hline = separator.HSeparator(widgetutil.BLACK)
+        vbox.pack_start(hline)
+        vbox.pack_start(widgetutil.align_middle(hbox, 0, 0, 25, 25),
+                        expand=True)
+
+        self.add(vbox)
+
+        self.selected_tab_list = self.selected_tabs = None
+        self.selected_file = None
+
+    def on_file_selected(self, manager, info):
+        self.selected_file = info
+
+    def on_playback_started(self, manager):
+        list_type, selected = app.tabs.selection
+        self.selected_tab_list = app.tabs[list_type]
+        self.selected_tabs = selected
+
+    def on_playback_stopped(self, manager):
+        self.selected_file = None
+
+    def on_title_clicked(self, button):
+        if not self.selected_tab_list or not self.selected_file:
+            return
+        if app.playback_manager.is_playing and not (
+            app.playback_manager.is_playing_audio or
+            app.playback_manager.detached_window):
+            # playing a video in the app, so don't bother
+            return
+        try:
+            tab_iter = self.selected_tab_list.iter_map[self.selected_tabs[0].id]
+        except KeyError:
+            #17495 - item may be from a tab that no longer exists
+            self.selected_tab_list = self.selected_tabs = None
+            return
+        app.tabs._select_from_tab_list(self.selected_tab_list.type, tab_iter)
+        display = app.display_manager.current_display
+        if hasattr(display, 'controller'):
+            controller = display.controller
+            controller.scroll_to_item(self.selected_file, manual=True, recenter=True)
+        else:
+            #17488 - GuideDisplay doesn't have a controller
+            logging.debug("current display doesn't have a controller - "
+                    "can't switch to")
 
     def handle_new_selection(self, has_playable):
         self.controls.handle_new_selection(has_playable)
+
+class PlaybackModeControls(widgetset.HBox):
+    def __init__(self):
+        widgetset.HBox.__init__(self, spacing=0)
+        self.shuffle = self._make_button('shuffle')
+        self.repeat = self._make_button('repeat')
+        self.pack_start(widgetutil.align_middle(self.shuffle))
+        self.pack_start(widgetutil.align_middle(self.repeat))
+        app.playback_manager.connect('update-shuffle', self.handle_shuffle)
+        app.playback_manager.connect('update-repeat', self.handle_repeat)
+
+    def _make_button(self, image_name):
+        button = imagebutton.ImageButton(image_name)
+        button.set_squish_width(True)
+        button.set_can_focus(False)
+        return button
+
+    def handle_shuffle(self, obj):
+        if app.playback_manager.shuffle:
+            self.shuffle.set_image('shuffle-on')
+            self.queue_redraw()
+        else:
+            self.shuffle.set_image('shuffle')
+            self.queue_redraw()
+
+    def handle_repeat(self, obj):
+        if app.playback_manager.repeat == WidgetStateStore.get_repeat_playlist():
+            self.repeat.set_image('repeat-on')
+            self.queue_redraw()
+        elif app.playback_manager.repeat == WidgetStateStore.get_repeat_track():
+            self.repeat.set_image('repeat-1')
+            self.queue_redraw()
+        else:
+            self.repeat.set_image('repeat')
+            self.queue_redraw()
