@@ -33,8 +33,14 @@ Widgets frontend. See WidgetState design doc.
 
 from miro.messages import (SaveDisplayState, SaveViewState, SaveGlobalState,
         DisplayInfo, ViewInfo)
+from miro import prefs
+from miro import app
 
 class WidgetStateStore(object):
+    # A CUSTOM_VIEW is a view which dynamically decides which view it is 
+    # supposed to be. The CUSTOM_VIEW has to be caught in widgetstatestore 
+    # and replaced with the proper view for the given situation.
+    CUSTOM_VIEW = 2 
     LIST_VIEW = 1
     STANDARD_VIEW = 0
     DEFAULT_VIEW_TYPE = {
@@ -42,7 +48,7 @@ class WidgetStateStore(object):
         u'device-audio': LIST_VIEW,
         u'device-video': STANDARD_VIEW,
         u'downloading': STANDARD_VIEW,
-        u'feed': STANDARD_VIEW,
+        u'feed': CUSTOM_VIEW,
         u'folder-contents': STANDARD_VIEW,
         u'music': LIST_VIEW,
         u'others': LIST_VIEW,
@@ -154,6 +160,13 @@ class WidgetStateStore(object):
 
     REPEAT_OFF, REPEAT_PLAYLIST, REPEAT_TRACK = range(3)
 
+    # Sorters that should always be enabled when available, and should not
+    # appear in the Sorts menu (#17696).
+    # This needs to be handled here, though analagous properties (e.g.
+    # NO_RESIZE_COLUMNS) are handled in widgetconst. #16783 refers to fixing the
+    # WSS mess - these shouldn't be sets of strings anyway.
+    MANDATORY_SORTERS = frozenset([u'name'])
+
     def __init__(self):
         self.displays = {}
         self.views = {}
@@ -209,6 +222,15 @@ class WidgetStateStore(object):
             view = display.selected_view
         else:
             view = WidgetStateStore.DEFAULT_VIEW_TYPE[display_type]
+            if view == WidgetStateStore.CUSTOM_VIEW:
+                if display_type == u'feed':
+                    view = app.config.get(prefs.PODCASTS_DEFAULT_VIEW)
+                else:
+                    app.widgetapp.handle_soft_failure("Getting default view",
+                                       "Unknown CUSTOM_VIEW, falling back to "
+                                       "STANDARD_VIEW",
+                                       with_exception=True)
+                    view = WidgetStateStore.STANDARD_VIEW
         return view
 
     def set_selected_view(self, display_type, display_id, selected_view):
@@ -295,6 +317,12 @@ class WidgetStateStore(object):
         if columns is None:
             columns = WidgetStateStore.DEFAULT_COLUMNS[display_type]
         available = WidgetStateStore.AVAILABLE_COLUMNS[display_type]
+        # If a column used to be disableable for a display but now is not,
+        # re-add it at the end of the list:
+        mandatory = available & WidgetStateStore.MANDATORY_SORTERS
+        columns.extend(mandatory.difference(columns))
+        # If a column used to be enableable for a display but now is not,
+        # filter it out:
         return [x for x in columns if x in available]
 
     def set_sorts_enabled(self, display_type, display_id, enabled):
@@ -351,6 +379,13 @@ class WidgetStateStore(object):
         self.global_info.guide_sidebar_expanded = expanded
         self._save_global_state()
 
+    def get_tabs_width(self):
+        return self.global_info.tabs_width
+
+    def set_tabs_width(self, width):
+        self.global_info.tabs_width = width
+        self._save_global_state()
+
 # Real ViewState properties:
 
     def get_scroll_position(self, display_type, display_id, view_type):
@@ -463,6 +498,10 @@ class WidgetStateStore(object):
     def get_columns():
         return WidgetStateStore.ALL_COLUMNS
 
+    @staticmethod
+    def get_toggleable_columns():
+        return WidgetStateStore.ALL_COLUMNS - WidgetStateStore.MANDATORY_SORTERS
+
     # displays:
 
     @staticmethod
