@@ -102,6 +102,11 @@ def init(languages=None, localedir=None):
         print "gtcache.init: setlocale failed.  setting locale to 'C'"
         locale.setlocale(locale.LC_ALL, 'C')
 
+    # bz:17713 - convert to str in utf-8 encoding before trying to use.
+    if languages:
+        languages = map(lambda lang: lang.encode('utf-8')
+                        if isinstance(lang, unicode) else lang, languages)
+
     _translation = _gt.translation("miro",
             localedir=localedir,
             languages=languages,
