@@ -65,7 +65,6 @@ class ChannelGuide(DDBObject, iconcache.IconCacheOwnerMixin):
         self.favicon = None
         self.firstTime = True
         self.store = self.STORE_NOT_STORE
-        self.type = u'guide'
         if url:
             self.historyLocation = 0
             self.history = [self.url]
@@ -73,6 +72,7 @@ class ChannelGuide(DDBObject, iconcache.IconCacheOwnerMixin):
             self.historyLocation = None
             self.history = []
 
+        self.setup_common()
         self.download_guide()
 
     def setup_restored(self):
@@ -82,6 +82,10 @@ class ChannelGuide(DDBObject, iconcache.IconCacheOwnerMixin):
         self.client = None
         if self.store is None:
             self.store = self.STORE_NOT_STORE
+        self.setup_common()
+
+    def setup_common(self):
+        self.type = u'guide'
 
     @classmethod
     def site_view(cls):
