@@ -60,6 +60,11 @@ from miro.plat import utils
 from miro.plat.resources import get_osname
 from miro.net import NetworkError, ConnectionError, ConnectionTimeout
 
+try:
+    import gzip
+except:
+    gzip = None
+
 REDIRECTION_LIMIT = 10
 MAX_AUTH_ATTEMPTS = 5
 
@@ -650,7 +655,18 @@ class CurlTransfer(object):
         info = self._make_callback_info()
         self.last_url = self.handle.getinfo(pycurl.EFFECTIVE_URL)
         if self.options.write_file is None:
-            info['body'] = self.buffer.getvalue()
+            if gzip and info.get('content-encoding', '') == 'gzip':
+                try:
+                    self.buffer.seek(0)
+                    info['body'] = gzip.GzipFile(
+                                       fileobj=self.buffer).read()
+                except IOError, e:
+                    info['body'] = self.buffer.getvalue()
+                    logging.warning("Received header with content-encoding "
+                                    "gzip, but content is not gzip encoded")
+            else:
+                info['body'] = self.buffer.getvalue()
+
         if self.check_response_code(info['status']):
             if not self.trying_head_request:
                 self.call_callback(info)
@@ -747,7 +763,13 @@ class CurlTransfer(object):
             return
         else:
             logging.warn("Unknown network error.  Code: %s", code)
-            error = NetworkError(_("Unknown"), unicode(handle.errstr()))
+            errstr = handle.errstr()
+            try:
+                description = errstr.decode('utf-8')
+            except UnicodeError, e:
+                logging.warn("error converting errstr: %s (%r)", e, errstr)
+                description = u'Unknown'
+            error = NetworkError(_("Unknown"), description)
         self.call_errback(error)
 
     def call_callback(self, info):
@@ -922,9 +944,15 @@ class LibCURLManager(eventloop.SimpleEventLoop):
     def check_finished(self):
         queued, finished, errors = self.multi.info_read()
         for handle in finished:
-            self.pop_transfer(handle).on_finished()
+            try:
+                self.pop_transfer(handle).on_finished()
+            except StandardError:
+                logging.stacktrace("Error calling on_finished()")
         for handle, code, message in errors:
-            self.pop_transfer(handle).on_error(code, handle)
+            try:
+                self.pop_transfer(handle).on_error(code, handle)
+            except StandardError:
+                logging.stacktrace("Error calling on_error()")
 
     def pop_transfer(self, handle):
         transfer = self.transfer_map.pop(handle)
