@@ -1769,24 +1769,18 @@ class Item(DDBObject, iconcache.IconCacheOwnerMixin, metadata.Store):
         # NOTE: it is very important (#7993) that there is no way to leave this
         # method without either:
         # - calling moviedata.movie_data_updater.request_update(self)
-        # - changing this item so that not self.in_incomplete_mdp_view
+        # - calling _handle_invalid_media_file
         try:
             self._check_media_file()
+        except IOError, e:
+            # shouldn't generally happen, but probably something we have no
+            # control over; likely another process has moved or deleted our file
+            logging.warn("check_media_file failed: %s", e)
+            self._handle_invalid_media_file()
         except CheckMediaError, e:
+            # filename is None - this should never happen
             app.controller.failed_soft("check_media_file", str(e), True)
-            # call path_processed since the movie data program won't run on us
-            path = self.get_filename()
-            if path is not None:
-                app.metadata_progress_updater.path_processed(path)
-            # Set our state to SKIPPED so we don't request another update.
-            self.mdp_state = moviedata.State.SKIPPED
-            self.file_type = u'other'
-            self.signal_change()
-            # The file may no longer be around.  Call check_delete() in an
-            # idle callback to handle this.  We don't want to call
-            # check_delete() now because it's not safe if we're called inside
-            # setup_new().  See #17344
-            eventloop.add_idle(self.check_deleted, 'checking item deleted')
+            self._handle_invalid_media_file()
         else:
             moviedata.movie_data_updater.request_update(self)
             if self.file_type is None:
@@ -1795,6 +1789,26 @@ class Item(DDBObject, iconcache.IconCacheOwnerMixin, metadata.Store):
                 self.file_type = u'other'
             self.signal_change()
 
+    def _handle_invalid_media_file(self):
+        """Failed to process a file in check_media_file; when this happens we:
+        - inform the metadata_progress_updater that we're done with the item
+        - make sure we don't try to process it again
+        - check whether the file no longer exists
+        """
+        # call path_processed since the movie data program won't run on us
+        path = self.get_filename()
+        if path is not None:
+            app.metadata_progress_updater.path_processed(path)
+        # Set our state to SKIPPED so we don't request another update.
+        self.mdp_state = moviedata.State.SKIPPED
+        self.file_type = u'other'
+        self.signal_change()
+        # The file may no longer be around.  Call check_delete() in an
+        # idle callback to handle this.  We don't want to call
+        # check_delete() now because it's not safe if we're called inside
+        # setup_new().  See #17344
+        _deleted_file_checker.schedule_check(self)
+
     def _check_media_file(self):
         """Does the work for check_media_file()
 
@@ -1807,14 +1821,8 @@ class Item(DDBObject, iconcache.IconCacheOwnerMixin, metadata.Store):
         filename = self.get_filename()
         if filename is None:
             raise CheckMediaError("item has no filename")
-        if (not fileutil.exists(filename) and not
-                self._allow_nonexistent_paths):
-            raise CheckMediaError("%s doesn't exist" % filename)
         self.file_type = filetypes.item_file_type_for_filename(filename)
-        try:
-            self.read_metadata()
-        except IOError:
-            raise CheckMediaError("IOError in read_metadata()")
+        self.read_metadata()
 
     def on_downloader_migrated(self, old_filename, new_filename):
         self.set_filename(new_filename)
@@ -1910,7 +1918,7 @@ class Item(DDBObject, iconcache.IconCacheOwnerMixin, metadata.Store):
             # In split_item() we found out that all our children were
             # deleted, so we were removed as well.  (#11979)
             return
-        eventloop.add_idle(self.check_deleted, 'checking item deleted')
+        _deleted_file_checker.schedule_check(self)
         if self.screenshot and not fileutil.exists(self.screenshot):
             logging.warn("file disappeared: %s", self.screenshot)
             self.screenshot = None
@@ -2210,7 +2218,6 @@ def filename_to_title(filename):
     title = filename_to_unicode(os.path.basename(filename))
     title = title.rsplit('.', 1)[0]
     title = title.replace('_', ' ')
-    title = title.lstrip('0123456789. -')
     t2 = []
     for word in title.split(' '):
         t2.append(word.capitalize())
@@ -2255,6 +2262,73 @@ class IncompleteMovieDataUpdator(object):
             eventloop.add_idle(self.do_some_updates,
                     'update incomplete movie data')
 
+class DeletedFileChecker(object):
+    """Utility class that manages calling Item.check_deleted().
+
+    This class ensures that we only schedule one idle callback at a time.
+    """
+    def __init__(self):
+        # track items that we should call check_deleted for
+        self.items_to_check = set()
+        # track if we have run_checks() scheduled as an idle callback
+        self.check_scheduled = False
+        # track if we should be checking yet
+        self.started = False
+
+    def schedule_check(self, item):
+        self.items_to_check.add(item)
+        self._ensure_run_checks_scheduled()
+
+    def start_checks(self):
+        """Start the deleted file checking."""
+        self.started = True
+        self._ensure_run_checks_scheduled()
+
+    def _ensure_run_checks_scheduled(self):
+        """Ensure that we run_checks() scheduled as an idle callback.
+
+        This method is a no-op if start_checks() hasn't been called yet.
+
+        If this method is called multiple times before run_checks() runs, then
+        it only schedules one callback.
+        """
+        if self.started and not self.check_scheduled:
+            eventloop.add_idle(self.run_checks, 'checking items deleted')
+            self.check_scheduled = True
+
+    def run_checks(self):
+        """Call check_deleted() for the items that are scheduled to check."""
+        self.check_scheduled = False
+        # Grab a limited number items at a time to prevent us from using too
+        # much time in for this idle callback.
+        # Update items_to_check immediately in case schedule_check() is called
+        # in response to us calling check_deleted()
+        items_this_pass = []
+        for x in xrange(100):
+            try:
+                to_check = self.items_to_check.pop()
+                items_this_pass.append(to_check)
+            except KeyError:
+                break # items_to_check is empty
+
+        app.bulk_sql_manager.start()
+        try:
+            for item in items_this_pass:
+                if item.id_exists():
+                    item.check_deleted()
+        finally:
+            app.bulk_sql_manager.finish()
+            if self.items_to_check:
+                self._ensure_run_checks_scheduled()
+
+
+def setup_deleted_checker():
+    global _deleted_file_checker
+    _deleted_file_checker = DeletedFileChecker()
+
+def start_deleted_checker():
+    _deleted_file_checker.start_checks()
+
 def fix_non_container_parents():
     """Make sure all items referenced by parent_id have isContainerItem set
 
