@@ -98,7 +98,8 @@ DEFAULT_DAAP_META = ('dmap.itemkind,dmap.itemid,dmap.itemname,' +
                      'com.apple.itunes.series-name')
 DEFAULT_DAAP_PLAYLIST_META = ('dmap.itemid,dmap.itemname,dmap.persistentid,' +
                               'daap.baseplaylist,dmap.itemcount,' +
-                              'dmap.parentcontainerid,dmap.persistentid')
+                              'dmap.parentcontainerid,dmap.persistentid,' + 
+                              'com.apple.itunes.is-podcast-playlist')
 
 class SessionObject(object):
     # Container object for a daap session.  Basically a heartbeat timeout
@@ -201,8 +202,6 @@ class DaapTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
                 del self.activeconn[s]
             except KeyError:
                 pass
-        if self.finished_callback:
-            self.finished_callback(s)
 
 class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
     protocol_version = 'HTTP/1.1'
@@ -673,19 +672,26 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                 reply = []
                 extra_headers = []
         except Exception, e:
-            self.log_message('Error: Exception occurred: ' + str(e))
-            if self.server.debug:
-                (typ, value, tb) = sys.exc_info()
-                print 'Exception: ' + str(typ)
-                print 'Traceback:\n'
-                traceback.print_tb(tb)
+            (typ, value, tb) = sys.exc_info()
+            parts = 'Error: Exception occurred: %s\nTraceback:\n'
+            parts += ''.join(traceback.format_list(traceback.extract_tb(tb)))
+            self.log_message(parts, e)
             # XXX should we end the connection on an exception occurence?
             rcode = DAAP_BADREQUEST
             reply = []
             extra_headers = []
-        content_encoding = self.reply_encoding()
-        self.do_send_reply(rcode, reply, extra_headers=extra_headers,
-                           content_encoding=content_encoding)
+        try:
+            content_encoding = self.reply_encoding()
+            self.do_send_reply(rcode, reply, extra_headers=extra_headers,
+                               content_encoding=content_encoding)
+        except Exception, e:
+            (typ, value, tb) = sys.exc_info()
+            parts = 'Error: Exception during response: %s\nTraceback:\n'
+            parts += ''.join(traceback.format_list(traceback.extract_tb(tb)))
+            self.log_message(parts, e)
+            # Let's cut them off ... no reply for you!  Re-raising the
+            # exception should make the caller do the right thing.
+            raise e
         if endconn:
             self.wfile.close()
 
@@ -974,7 +980,7 @@ class DaapClient(object):
             return True
         # We've been disconnected or there was a problem?
         except (socket.error, IOError, ValueError):
-            self.disconnect()
+            self.disconnect(polite=True)
             return False
         except httplib.BadStatusLine:
             self.disconnect()
@@ -994,7 +1000,8 @@ class DaapClient(object):
         # We've been disconnected or there was a problem?
         except (AttributeError, socket.error, IOError, ValueError,
                 httplib.BadStatusLine):
-            self.disconnect()
+            # Use of 'polite' flag: see description in connect().
+            self.disconnect(polite=True)
             return None
 
     def revision_query(self, update):
@@ -1071,25 +1078,34 @@ class DaapClient(object):
             self.disconnect()
             return None
 
-    def disconnect(self):
+    def disconnect(self, polite=False):
         try:
             self.timer.cancel()
-            # Don't use - you are just making life difficult for yourself to
-            # nuke the existing connection ...
-            #if polite:
-            #    self.conn.request('GET', self.sessionize('/logout', []))
+            # Don't use 'polite' unless absolutely necessary - you are just 
+            # making life difficult for yourself to nuke the existing
+            # connection.  Usually, we would use in connect() when the login
+            # fails, including when we cannot deal with a newer version of
+            # iTunes, in which case the login is okay but database fetch
+            # fails.  iTunes deals badly otherwise, thinking we are still 
+            # hanging around.
+            if polite:
+                self.conn.request('GET', self.sessionize('/logout', []))
         # Don't care since we are going away anyway.
         except (socket.error, ValueError, httplib.ResponseNotReady,
                 httplib.BadStatusLine, AttributeError, IOError):
             pass
         finally:
-            # self.conn may be invalid at this point but that's okay, finally
-            # block doesn't raise exception but just prints it out.
+            # self.conn may be invalid at this point but that's okay, 
+            # we'll catch it and also the socket error in case that gets
+            # raised.
             self.session = None
             if self.conn:
                 conn = self.conn
                 self.conn = None
-                conn.sock.shutdown(socket.SHUT_RDWR)
+                try:
+                    conn.sock.shutdown(socket.SHUT_RDWR)
+                except (AttributeError, socket.error), e:
+                    pass
                 conn.close()
 
     def daap_get_file_request(self, file_id, enclosure=None):
