@@ -458,16 +458,30 @@ class DownloadingItemsTracker(DatabaseSourceTrackerBase):
         self.view = item.Item.download_tab_view()
         DatabaseSourceTrackerBase.__init__(self)
 
+# We don't use the preferenced tracker superclass here because we want
+# to do it manually - have both audio and video prefs to take care of here.
 class SharingBackendItemsTracker(DatabaseSourceTrackerBase):
     type = u'sharing-backend'
     def __init__(self, id_):
-        self.id = id_
-        if self.id is None:
-            # All items
+        # NOTE: dodgy: we want to keep the existing api but we need a different
+        # database query for playlist and feed.  So ... track this info in
+        # the identifier and then chuck it away when we are done.
+        if id_ is None:
+            # All items.  Type is ignored in this case.
+            self.id = None
             self.view = item.Item.watchable_view()
         else:
-            # This playlist id only.
-            self.view = item.Item.playlist_view(self.id)
+            id_, podcast = id_
+            typ = 'feed' if podcast else 'playlist'
+            self.id = id_
+            if typ == 'playlist':
+                self.view = item.Item.playlist_view(self.id)
+            elif typ == 'feed':
+                self.view = item.Item.feed_view(self.id)
+            else:
+                # Future expansion?
+                logging.debug('SharingBackendTracker: unrecognized type %s',
+                              typ)
         DatabaseSourceTrackerBase.__init__(self)
 
 class PreferencedItemsTracker(DatabaseSourceTrackerBase):
@@ -535,14 +549,33 @@ class GuideSidebarTracker(DatabaseSourceTrackerBase):
 class SharingItemTracker(SourceTrackerBase):
     type = u'sharing'
     def __init__(self, share):
-        share_id = share.parent_id if share.parent_id else share.id
+        share_id = share.tracker_id
         self.id = share
         self.tracker = app.sharing_tracker.get_tracker(share_id)
+        prefdict = dict(video=prefs.SHOW_PODCASTS_IN_VIDEO,
+                        audio=prefs.SHOW_PODCASTS_IN_MUSIC)
         self.source = itemsource.SharingItemSource(self.tracker,
                                                    share.playlist_id)
-
+        try:
+            self.pref = prefdict[share.playlist_id]
+            self.prefvalue = app.config.get(self.pref)
+            self.source.include_podcasts = self.prefvalue
+        except KeyError:
+            self.pref = self.prefvalue = None
+        app.backend_config_watcher.connect_weak('changed',
+                                                self.on_config_changed)
         SourceTrackerBase.__init__(self)
 
+    def on_config_changed(self, obj, key, value):
+        if self.pref and key == self.pref.key:
+            self.prefvalue = value
+            self.source.include_podcasts = self.prefvalue
+            for source in self.trackers:
+                source.disconnect_all()
+            self.trackers = []
+            self.add_callbacks()
+            self.send_initial_list()
+
     def _make_changed_list(self, changed):
         retval = []
         for obj in changed:
