@@ -115,6 +115,7 @@ SHARE_NAME                  = Pref(key='ShareName',             default="Miro Sh
 SHARE_WARN_ON_QUIT          = Pref(key='ShareWarnOnQuit',       default=True, platformSpecific=False)
 SHARE_VIDEO                 = Pref(key='ShareVideo',            default=True, platformSpecific=False)
 SHARE_AUDIO                 = Pref(key='ShareAudio',            default=True, platformSpecific=False)
+SHARE_FEED                  = Pref(key='ShareFeed',             default=True, platformSpecific=False)
 MUSIC_TAB_CLICKED           = Pref(key='musicTabClicked',       default=False, platformSpecific=False)
 SHOW_PODCASTS_IN_VIDEO      = Pref(key='showPodcastsInVideo', default=True, platformSpecific=False)
 SHOW_PODCASTS_IN_MUSIC      = Pref(key='showPodcastsInMusic', default=False, platformSpecific=False)
@@ -170,11 +171,11 @@ SHARE_URL         = Pref(key='ShareURL',        default=default_share,
                          platformSpecific=False)
 # this is for final releases
 AUTOUPDATE_URL    = Pref(key='AutoupdateURL',   default=default_autoupdate,
-                         platformSpecific=False)
+                         platformSpecific=True)
 # this is for beta and release candidate releases
 AUTOUPDATE_BETA_URL = Pref(
     key='AutoupdateBetaURL', default=default_autoupdate_beta,
-    platformSpecific=False)
+    platformSpecific=True)
 DONATE_URL        = Pref(key='DonateURL', default=u"http://www.getmiro.com/donate/",
                          platformSpecific=False)
 TROUBLESHOOT_URL  = Pref(key='TroubleshootURL', default=u"http://manual.getmiro.com/troubleshooting.html",
