@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,8 +53,7 @@ Go to the bottom of this file for the current database schema.
 import datetime
 import time
 from types import NoneType
-from miro.plat.utils import FilenameType
-from miro.frontendstate import WidgetsFrontendState
+from miro.plat.utils import PlatformFilenameType
 
 class ValidationError(Exception):
     """Error thrown when we try to save invalid data."""
@@ -132,7 +132,7 @@ class SchemaBinary(SchemaSimpleItem):
 class SchemaFilename(SchemaSimpleItem):
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
-        self.validateType(data, FilenameType)
+        self.validateType(data, PlatformFilenameType)
 
 class SchemaURL(SchemaSimpleItem):
     def validate(self, data):
@@ -265,7 +265,7 @@ class SchemaStatusContainer(SchemaReprContainer):
                 self.validateType(value, str)
 
     def _binary_fields(self):
-        if FilenameType == unicode:
+        if PlatformFilenameType == unicode:
             return ('metainfo', 'fastResumeData')
         else:
             return ('channelName', 'shortFilename', 'filename', 'metainfo',
@@ -464,6 +464,9 @@ class ManualFeedImplSchema(FeedImplSchema):
     # no addition fields over FeedImplSchema
 
 class SingleFeedImplSchema(FeedImplSchema):
+    """
+    NB: This type of feed no longer exists as of version 125.
+    """
     klass = None
     classString = 'single-feed-impl'
     # no addition fields over FeedImplSchema
