@@ -40,6 +40,7 @@ from miro.gtcache import gettext as _
 import logging
 import os
 import config
+import sys
 import traceback
 import threading
 import time
@@ -86,6 +87,7 @@ from miro import conversions
 from miro import devices
 from miro import sharing
 from miro import transcode
+from miro import workerprocess
 from miro.plat import devicetracker
 
 DEBUG_DB_MEM_USAGE = False
@@ -117,6 +119,7 @@ def startup_function(func):
             # db attribute
             if ((isinstance(exc, (database.DatabaseException,
                                   database.DatabaseStandardError,
+                                  storedatabase.sqlite3.DatabaseError,
                                   storedatabase.sqlite3.OperationalError))
                  and app.db is not None)):
 
@@ -263,6 +266,8 @@ def load_extensions():
 @startup_function
 def finish_startup(obj, thread):
     database.set_thread(thread)
+    logging.info("Installing deleted file checker...")
+    item.setup_deleted_checker()
     logging.info("Restoring database...")
     start = time.time()
     app.db = storedatabase.LiveStorage()
@@ -348,18 +353,17 @@ def check_firsttime():
 def check_movies_gone():
     """Checks to see if the movies directory is gone.
     """
+
+    # callback is what the frontend will call if the user asks us to continue
     callback = lambda: eventloop.add_urgent_call(fix_movies_gone,
                                                "fix movies gone")
 
-    if is_movies_directory_gone():
-        logging.info("Movies directory is gone -- calling handler.")
-        _movies_directory_gone_handler(callback)
-        return
-
     movies_dir = fileutil.expand_filename(app.config.get(
         prefs.MOVIES_DIRECTORY))
+
     # if the directory doesn't exist, create it.
-    if not os.path.exists(movies_dir):
+    if (not os.path.exists(movies_dir) and
+            should_create_movies_directory(movies_dir)):
         try:
             fileutil.makedirs(movies_dir)
         except OSError:
@@ -375,6 +379,14 @@ def check_movies_gone():
     if not os.access(movies_dir, os.W_OK):
         _movies_directory_gone_handler(callback)
         return
+
+    # make sure that the directory is populated if we've downloaded stuff to
+    # it
+    if is_movies_directory_gone():
+        logging.info("Movies directory is gone -- calling handler.")
+        _movies_directory_gone_handler(callback)
+        return
+
     eventloop.add_urgent_call(finish_backend_startup, "reconnect downloaders")
 
 @startup_function
@@ -412,6 +424,10 @@ def on_frontend_started():
     # Delay running high CPU/IO operations for a bit
     eventloop.add_timeout(5, downloader.startup_downloader,
             "start downloader daemon")
+    eventloop.add_timeout(10, workerprocess.startup,
+            "start worker process")
+    eventloop.add_timeout(20, item.start_deleted_checker,
+            "start checking deleted items")
     eventloop.add_timeout(30, feed.start_updates, "start feed updates")
     eventloop.add_timeout(60, item.update_incomplete_movie_data,
             "update movie data")
@@ -452,6 +468,14 @@ def mark_first_time():
     # startup dialog pops up again
     app.config.save()
 
+def should_create_movies_directory(path):
+    """Figure out if we should create the movies directory if it's missing."""
+    if sys.platform == 'darwin' and path.startswith("/Volumes/"):
+        # Hack to fix #17826.  Don't try to create new directories in the
+        # mount points on OS X.
+        return False
+    return True
+
 def is_movies_directory_gone():
     """Checks to see if the MOVIES_DIRECTORY exists.
 
