@@ -1,23 +1,31 @@
 import os
 import unittest
-from datetime import datetime
 from time import sleep
 
-from miro import config
-from miro import feedparser
+from miro import app
 from miro import prefs
 from miro import dialogs
-from miro import database
-from miro import storedatabase
 from miro import feedparserutil
-from miro.plat import resources
 from miro.item import Item
 from miro.feed import validate_feed_url, normalize_feed_url, Feed
 
 from miro.test.framework import MiroTestCase, EventLoopTest
 
-class FakeDownloader:
-    pass
+class FakeDownloader(object):
+    def __init__(self):
+        self.current_size = 0
+        self.rate = 0
+        self.state = 'downloading'
+
+    def get_current_size(self):
+        return self.current_size
+
+    def get_rate(self):
+        return self.rate
+
+    def get_state(self):
+        return self.state
+
 
 class AcceptScrapeTestDelegate:
     def __init__(self):
@@ -120,7 +128,7 @@ class FeedTestCase(EventLoopTest):
 class SimpleFeedTestCase(FeedTestCase):
     def setUp(self):
         FeedTestCase.setUp(self)
-        # Based on 
+        # Based on
         # http://cyber.law.harvard.edu/blogs/gems/tech/rss2sample.xml
 
         # this rss feed has no enclosures.
@@ -141,26 +149,36 @@ class SimpleFeedTestCase(FeedTestCase):
       <item>
          <title>Star City</title>
          <link>http://liftoff.msfc.nasa.gov/news/2003/news-starcity.mov</link>
-         <description>How do Americans get ready to work with Russians aboard the International Space Station? They take a crash course in culture, language and protocol at Russia's &lt;a href="http://howe.iki.rssi.ru/GCTC/gctc_e.htm"&gt;Star City&lt;/a&gt;.</description>
+         <description>How do Americans get ready to work with Russians aboard \
+the International Space Station? They take a crash course in culture, \
+language and protocol at Russia's &lt;a href="http://howe.iki.rssi.ru/GCTC/\
+gctc_e.htm"&gt;Star City&lt;/a&gt;.</description>
          <pubDate>Tue, 03 Jun 2003 09:39:21 GMT</pubDate>
          <guid>http://liftoff.msfc.nasa.gov/2003/06/03.html#item573</guid>
       </item>
       <item>
-         <description>Sky watchers in Europe, Asia, and parts of Alaska and Canada will experience a &lt;a href="http://science.nasa.gov/headlines/y2003/30may_solareclipse.htm"&gt;partial eclipse of the Sun&lt;/a&gt; on Saturday, May 31st.</description>
+         <description>Sky watchers in Europe, Asia, and parts of Alaska and \
+Canada will experience a &lt;a href="http://science.nasa.gov/headlines/y2003/\
+30may_solareclipse.htm"&gt;partial eclipse of the Sun&lt;/a&gt; on Saturday, \
+May 31st.</description>
          <pubDate>Fri, 30 May 2003 11:06:42 GMT</pubDate>
          <guid>http://liftoff.msfc.nasa.gov/2003/05/30.html#item572</guid>
       </item>
       <item>
          <title>The Engine That Does More</title>
          <link>http://liftoff.msfc.nasa.gov/news/2003/news-VASIMR.asp</link>
-         <description>Before man travels to Mars, NASA hopes to design new engines that will let us fly through the Solar System more quickly.  The proposed VASIMR engine would do that.</description>
+         <description>Before man travels to Mars, NASA hopes to design new \
+engines that will let us fly through the Solar System more quickly.  The \
+proposed VASIMR engine would do that.</description>
          <pubDate>Tue, 27 May 2003 08:37:32 GMT</pubDate>
          <guid>http://liftoff.msfc.nasa.gov/2003/05/27.html#item571</guid>
       </item>
       <item>
          <title>Astronauts' Dirty Laundry</title>
          <link>http://liftoff.msfc.nasa.gov/news/2003/news-laundry.asp</link>
-         <description>Compared to earlier spacecraft, the International Space Station has many luxuries, but laundry facilities are not one of them.  Instead, astronauts have other options.</description>
+         <description>Compared to earlier spacecraft, the International Space \
+Station has many luxuries, but laundry facilities are not one of them.  \
+Instead, astronauts have other options.</description>
          <pubDate>Tue, 20 May 2003 08:56:02 GMT</pubDate>
          <guid>http://liftoff.msfc.nasa.gov/2003/05/20.html#item570</guid>
       </item>
@@ -214,7 +232,8 @@ class MultiFeedExpireTest(FeedTestCase):
    <channel>
       <title>Downhill Battle Pics</title>
       <link>http://downhillbattle.org/</link>
-      <description>Downhill Battle is a non-profit organization working to support participatory culture and build a fairer music industry.</description>
+      <description>Downhill Battle is a non-profit organization working to \
+support participatory culture and build a fairer music industry.</description>
       <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
 """)
 
@@ -241,8 +260,8 @@ class MultiFeedExpireTest(FeedTestCase):
 
         self.write_files(5, 10) # 5 feeds containing 10 identical items
         self.feed = self.make_feed()
-        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
-        config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 5)
+        app.config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
+        app.config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 5)
         self.rewrite_files(1) # now only 5 items in each feed
         self.update_feed(self.feed)
 
@@ -254,7 +273,8 @@ class EnclosureFeedTestCase(FeedTestCase):
    <channel>
       <title>Downhill Battle Pics</title>
       <link>http://downhillbattle.org/</link>
-      <description>Downhill Battle is a non-profit organization working to support participatory culture and build a fairer music industry.</description>
+      <description>Downhill Battle is a non-profit organization working to \
+support participatory culture and build a fairer music industry.</description>
       <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
       <item>
          <title>Bumper Sticker</title>
@@ -264,19 +284,22 @@ class EnclosureFeedTestCase(FeedTestCase):
       </item>
       <item>
          <title>T-shirt</title>
-         <enclosure url="http://downhillbattle.org/key/gallery/payola_tshirt.mpg" />
+         <enclosure url="http://downhillbattle.org/key/gallery/\
+payola_tshirt.mpg" />
       </item>
       <item>
          <enclosure url="http://downhillbattle.org/key/gallery/chriscE.mpg" />
          <description>Flyer in Yucaipa, CA</description>
       </item>
       <item>
-         <enclosure url="http://downhillbattle.org/key/gallery/jalabel_nov28.mpg" />
+         <enclosure url="http://downhillbattle.org/key/gallery/\
+jalabel_nov28.mpg" />
       </item>
       <item>
-         <enclosure url="http://downhillbattle.org/key/gallery/jalabel_nov28.jpg" />
+         <enclosure url="http://downhillbattle.org/key/gallery/\
+jalabel_nov28.jpg" />
       </item>
-      
+
    </channel>
 </rss>""")
 
@@ -297,8 +320,8 @@ class OldItemExpireTest(FeedTestCase):
         self.counter = 0
         self.write_new_feed()
         self.feed = self.make_feed()
-        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
-        config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 20)
+        app.config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
+        app.config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 20)
 
     def write_new_feed(self, entryCount=2):
         # make a feed with a new item and parse it
@@ -309,7 +332,8 @@ class OldItemExpireTest(FeedTestCase):
    <channel>
       <title>Downhill Battle Pics</title>
       <link>http://downhillbattle.org/</link>
-      <description>Downhill Battle is a non-profit organization working to support participatory culture and build a fairer music industry.</description>
+      <description>Downhill Battle is a non-profit organization working to \
+support participatory culture and build a fairer music industry.</description>
       <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
 """)
 
@@ -358,19 +382,20 @@ class OldItemExpireTest(FeedTestCase):
         self.check_guids(1, 2, 5, 6)
 
     def test_overflow_still_in_feed(self):
-        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 0)
+        app.config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 0)
         self.parse_new_feed(6)
         self.check_guids(3, 4, 5, 6, 7, 8)
 
     def test_overflow_with_replacement(self):
         # Keep item with guid-2 in the feed.
-        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 0)
+        app.config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 0)
         self.counter = 1
         self.parse_new_feed(5)
         self.check_guids(2, 3, 4, 5, 6)
 
     def test_overflow_with_max_old_items(self):
-        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 1000) # don't bother
+        app.config.set(
+            prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 1000) # don't bother
         self.assertEqual(Item.make_view().count(), 2)
         self.parse_new_feed()
         self.assertEquals(Item.make_view().count(), 4)
@@ -381,7 +406,7 @@ class OldItemExpireTest(FeedTestCase):
             self.processThreads()
             self.process_idles()
             sleep(0.1)
-        self.assertEquals(Item.make_view().count(), 6)            
+        self.assertEquals(Item.make_view().count(), 6)
         self.feed.set_max_old_items(2)
         self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
@@ -392,19 +417,20 @@ class OldItemExpireTest(FeedTestCase):
         self.check_guids(3, 4, 5, 6)
 
     def test_overflow_with_global_max_old_items(self):
-        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 1000) # don't bother
+        app.config.set(
+            prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 1000) # don't bother
         self.assertEqual(Item.make_view().count(), 2)
         self.parse_new_feed()
         self.assertEquals(Item.make_view().count(), 4)
         self.parse_new_feed()
-        config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 4)
+        app.config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 4)
         self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
             self.processThreads()
             self.process_idles()
             sleep(0.1)
         self.assertEquals(Item.make_view().count(), 6)
-        config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 2)
+        app.config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 2)
         self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
             self.processThreads()
@@ -427,7 +453,7 @@ class FeedParserAttributesTestCase(FeedTestCase):
             os.remove(self.tempdb)
         self.reload_database(self.tempdb)
         self.write_feed()
-        self.parsed_feed = feedparser.parse(self.filename)
+        self.parsed_feed = feedparserutil.parse(self.filename)
         self.make_feed()
         self.save_then_restore_db()
 
@@ -448,20 +474,23 @@ class FeedParserAttributesTestCase(FeedTestCase):
     <channel>
         <title>Downhill Battle Pics</title>
         <link>http://downhillbattle.org/</link>
-        <description>Downhill Battle is a non-profit organization working to support participatory culture and build a fairer music industry.</description>
+        <description>Downhill Battle is a non-profit organization working to \
+support participatory culture and build a fairer music industry.</description>
         <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
 
         <item>
             <title>Bumper Sticker</title>
             <link>http://downhillbattle.org/item</link>
             <comments>http://downhillbattle.org/item/comments</comments>
-            <creativeCommons:license>http://www.creativecommons.org/licenses/by-nd/1.0</creativeCommons:license>
+            <creativeCommons:license>http://www.creativecommons.org/licenses/\
+by-nd/1.0</creativeCommons:license>
             <guid>guid-1234</guid>
             <enclosure url="http://downhillbattle.org/key/gallery/movie.mpg"
                 length="1234"
                 type="video/mpeg"
                 />
-            <description>I'm a musician and I support filesharing.</description>
+            <description>I'm a musician and I support filesharing.\
+</description>
             <pubDate>Fri, 18 Mar 2005 12:03:42 EST</pubDate>
             <media:thumbnail url="%(thumburl)s" />
             <dtv:paymentlink url="http://www.example.com/payment.html" />
@@ -469,13 +498,14 @@ class FeedParserAttributesTestCase(FeedTestCase):
 
     </channel>
 </rss>
-""" % {'thumburl': resources.url("testdata/democracy-now-unicode-bug.xml")})
+""" % {'thumburl': "http://example.com/thumbnail.jpg"})
 
 
     def test_attributes(self):
         entry = self.parsed_feed.entries[0]
         self.assertEquals(self.item.get_rss_id(), entry.id)
-        self.assertEquals(self.item.get_thumbnail_url(), entry.thumbnail['url'])
+        self.assertEquals(self.item.get_thumbnail_url(),
+                          entry.thumbnail['url'])
         self.assertEquals(self.item.get_title(), entry.title)
         self.assertEquals(self.item.get_description(), entry.description)
         self.assertEquals(self.item.get_link(), entry.link)
