@@ -10,10 +10,12 @@ from miro import app
 from miro import config
 from miro import database
 from miro import eventloop
+from miro import feed
 from miro import feedparserutil
 from miro import downloader
 from miro import httpauth
 from miro import httpclient
+from miro import item
 from miro import iteminfocache
 from miro import moviedata
 from miro import util
@@ -23,6 +25,7 @@ from miro import signals
 from miro import storedatabase
 from time import sleep
 from miro import models
+from miro import workerprocess
 
 from miro.test import testhttpserver
 
@@ -261,12 +264,15 @@ class MiroTestCase(unittest.TestCase):
         app.in_unit_tests = True
         # Tweak Item to allow us to make up fake paths for FileItems
         models.Item._allow_nonexistent_paths = True
+        # setup the deleted file checker
+        item.setup_deleted_checker()
+        item.start_deleted_checker()
         # setup movie data stuff
         self.metadata_progress_updater = FakeMetadataProgressUpdater()
         app.metadata_progress_updater = self.metadata_progress_updater
         moviedata.movie_data_updater = moviedata.MovieDataUpdater()
-        # make feed parser updates happen immediately
-        feedparserutil._queue_parse_proccessor.WAIT_TIME = 0.0
+        # Skip worker proccess for feedparser
+        feed._RUN_FEED_PARSER_INLINE = True
         # reload config and initialize it to temprary
         config.load_temporary()
         self.platform = app.config.get(prefs.APP_PLATFORM)
@@ -294,6 +300,8 @@ class MiroTestCase(unittest.TestCase):
         return self.platform == "windows"
 
     def tearDown(self):
+        # shutdown workerprocess if we started it for some reason.
+        workerprocess.shutdown()
         self.reset_log_filter()
         signals.system.disconnect_all()
         util.chatter = True
