@@ -1,7 +1,6 @@
 from datetime import datetime
 import os
 import unittest
-from glob import glob
 import time
 
 from miro import app
@@ -12,21 +11,22 @@ from miro import downloader
 from miro import item
 from miro import feed
 from miro import folder
-from miro import frontendstate
+from miro import widgetstate
 from miro import guide
 from miro import schema
 from miro import signals
 from miro import tabs
 from miro import theme
+from miro.fileobject import FilenameType
 import shutil
 from miro import storedatabase
 from miro.plat import resources
-from miro.plat.utils import FilenameType
+from miro.plat.utils import PlatformFilenameType
 
-from miro.test.framework import MiroTestCase, EventLoopTest
+from miro.test.framework import EventLoopTest, skip_for_platforms
 from miro.schema import (SchemaString, SchemaInt, SchemaFloat,
                          SchemaReprContainer, SchemaList, SchemaDict,
-                         SchemaObject, SchemaBool, SchemaFilename,
+                         SchemaBool, SchemaFilename,
                          SchemaBinary)
 
 # create a dummy object schema
@@ -69,7 +69,9 @@ class PCFProgramer(Human):
 class SpecialProgrammer(PCFProgramer):
     def setup_new(self):
         PCFProgramer.setup_new(self, u'I.M. Special', 44, 2.1, [],
-                FilenameType('/home/specialdude/\u1234'.encode("utf-8")), True)
+                               PlatformFilenameType(
+                               '/home/specialdude/\u1234'.encode("utf-8")),
+                               True)
 
 class HumanSchema(schema.ObjectSchema):
     klass = Human
@@ -142,6 +144,9 @@ class StoreDatabaseTest(EventLoopTest):
             os.unlink(self.save_path)
 
     def tearDown(self):
+        # need to close the db before removing it from disk
+        from miro import app
+        app.db.close()
         self.remove_database()
         corrupt_path = os.path.join(os.path.dirname(self.save_path),
                                     'corrupt_database')
@@ -172,15 +177,14 @@ class DBUpgradeTest(StoreDatabaseTest):
     def tearDown(self):
         try:
             os.unlink(self.save_path2)
-        except:
+        except OSError:
             pass
         StoreDatabaseTest.tearDown(self)
 
+    @skip_for_platforms('win32')
     def test_indexes_same(self):
-        # FIXME - this fails on windows because it's using a non-Windows
+        # this fails on windows because it's using a non-Windows
         # database
-        if self.on_windows():
-            self.assert_(False, "test_indexes_same fails on windows")
         self.remove_database()
         self.reload_database()
         app.db.cursor.execute("SELECT name FROM sqlite_master "
@@ -194,11 +198,10 @@ class DBUpgradeTest(StoreDatabaseTest):
         upgraded_db_indexes = set(app.db.cursor)
         self.assertEquals(upgraded_db_indexes, blank_db_indexes)
 
+    @skip_for_platforms('win32')
     def test_schema_same(self):
-        # FIXME - this fails on windows because it's using a non-Windows
+        # this fails on windows because it's using a non-Windows
         # database
-        if self.on_windows():
-            self.assert_(False, "test_schema_same fails on windows")
         self.remove_database()
         self.reload_database()
         blank_column_types = self._get_column_types()
@@ -233,7 +236,8 @@ class FakeSchemaTest(StoreDatabaseTest):
         self.joe = RestorableHuman(u"joe", 14, 1.4, [self.lee], car=u'toyota',
                                    dog=u'scruffy')
         self.ben = PCFProgramer(u'ben', 25, 3.4, [self.joe],
-                                FilenameType('/home/ben/\u1234'.encode("utf-8")), True)
+                                PlatformFilenameType(
+                                '/home/ben/\u1234'.encode("utf-8")), True)
         self.db = [self.lee, self.joe, self.ben]
         databaseupgrade._upgrade_overide[1] = upgrade1
         databaseupgrade._upgrade_overide[2] = upgrade2
@@ -360,23 +364,40 @@ class DiskTest(FakeSchemaTest):
         self.allow_db_load_errors(False)
         self.assert_(os.path.exists(corrupt_path))
 
-    def start_fresh_on_error_dialog(self):
+    def handle_corrupt_db_dialogs(self, upgrade, corruption):
+        """Handle the dialogs that we pop up when we notice database errors.
+
+        :param upgrade: handle upgrade dialogs by clicking "start fresh"
+        :param corruption: handle database corrupt message boxes
+        """
         def dialog_handler(obj, dialog):
-            dialog.run_callback(dialogs.BUTTON_START_FRESH)
+            if upgrade and (dialogs.BUTTON_START_FRESH in dialog.buttons):
+                # handle database upgrade dialog
+                dialog.run_callback(dialogs.BUTTON_START_FRESH)
+            elif corruption and isinstance(dialog, dialogs.MessageBoxDialog):
+                # handle the load error dialog
+                dialog.run_callback(dialogs.BUTTON_OK)
+            else:
+                raise AssertionError("Don't know how to handle dialog: %s",
+                        dialog)
         signals.system.connect('new-dialog', dialog_handler)
 
     def test_upgrade_error(self):
-        self.start_fresh_on_error_dialog()
+        self.handle_corrupt_db_dialogs(upgrade=True, corruption=False)
         self.check_reload_error(version=2)
 
     def test_corrupt_database(self):
         app.db.close()
         open(self.save_path, 'wb').write("BOGUS DATA")
-        self.start_fresh_on_error_dialog()
+        # depending on the SQLite version, we will notice the error when we
+        # issup the PRAGMA journal_mode command, or when we do the upgrades.
+        # Handle the dialogs for both.
+        self.handle_corrupt_db_dialogs(upgrade=True, corruption=True)
         self.check_reload_error()
 
     def test_database_data_error(self):
         app.db.cursor.execute("DROP TABLE human")
+        self.handle_corrupt_db_dialogs(upgrade=False, corruption=True)
         self.check_reload_error()
 
     def test_bulk_insert(self):
@@ -435,6 +456,20 @@ class DiskTest(FakeSchemaTest):
         lee2 = self.reload_object(lee2)
         self.assertEqual(lee2.name, u'lee2-changed')
 
+    def test_bulk_insert_and_remove(self):
+        # test inserting, then removing an object while in bulk mode
+        app.bulk_sql_manager.start()
+        # nothing should be inserted yet
+        lee = Human(u'lee', 25, 1.4, [], {u'virtual bowling': 212})
+        lee.remove()
+        # lee was inserted, then removed while inside a bulk transaction,
+        # make sure things are cleaned up correctly (#17428)
+        app.bulk_sql_manager.finish()
+        self.assert_(not lee.id_exists())
+        self.assert_(lee.id not in app.db._object_map)
+        lee_view = Human.make_view("id=?", values=(lee.id,))
+        self.assertEquals(lee_view.count(), 0)
+
 class ObjectMemoryTest(FakeSchemaTest):
     def test_remove_remove_object_map(self):
         self.reload_test_database()
@@ -524,7 +559,9 @@ class ConverterTest(StoreDatabaseTest):
         self.assertEquals(val, {"updated_parsed":
                                 (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
 
-        test2 = """{'updated_parsed': time.struct_time(tm_year=2009, tm_mon=6, tm_mday=5, tm_hour=1, tm_min=30, tm_sec=0, tm_wday=4, tm_yday=156, tm_isdst=0)}"""
+        test2 = """{'updated_parsed': time.struct_time(tm_year=2009, \
+tm_mon=6, tm_mday=5, tm_hour=1, tm_min=30, tm_sec=0, tm_wday=4, tm_yday=156, \
+tm_isdst=0)}"""
         val = converter._convert_repr(test2)
         self.assertEquals(val, {"updated_parsed":
                                 (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
@@ -542,7 +579,7 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
         self.tab_order = tabs.TabOrder(u'channel')
         self.guide = guide.ChannelGuide(u'http://example.com/')
         self.theme_hist = theme.ThemeHistory()
-        self.widgets_frontend_state = frontendstate.WidgetsFrontendState()
+        self.display_state = widgetstate.DisplayState((u'testtype', u'testid'))
 
     def check_fixed_value(self, obj, column_name, value, disk_value=None):
         obj = self.reload_object(obj)
@@ -595,15 +632,21 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
 
     def test_corrupt_past_themes(self):
         app.db.cursor.execute("UPDATE theme_history "
-                "SET pastThemes='[1, 2; 3 ]' WHERE id=?", (self.theme_hist.id,))
+                "SET pastThemes='[1, 2; 3 ]' WHERE id=?",
+                              (self.theme_hist.id,))
         self.check_fixed_value(self.theme_hist, 'pastThemes', [])
 
-    def test_corrupt_list_view_displays(self):
-        app.db.cursor.execute("UPDATE widgets_frontend_state "
-                "SET list_view_displays='[1, 2; 3 ]' WHERE id=?",
-                (self.widgets_frontend_state.id,))
-        self.check_fixed_value(self.widgets_frontend_state,
-                'list_view_displays', [])
+    def test_corrupt_display_state(self):
+        app.db.cursor.execute("UPDATE display_state SET "
+                "active_filters=?, selected_view=?, "
+                "list_view_columns=?, list_view_widths=? WHERE id=?",
+                (5, 1, '{baddata', '{baddata', self.display_state.id))
+        self.check_fixed_value(self.display_state, 'list_view_columns', None)
+        self.check_fixed_value(self.display_state, 'list_view_widths', None)
+        # check that fields with valid values were salvaged
+        reloaded = self.reload_object(self.display_state)
+        self.assertEquals(reloaded.active_filters, 5)
+        self.assertEquals(reloaded.selected_view, 1)
 
     def test_corrupt_link_history(self):
         # TODO: should test ScraperFeedIpml.linkHistory, but it's not so easy
