@@ -322,6 +322,8 @@ class TranscodeObject(object):
 
         self.request_path_func = request_path_func
 
+        # note: nchunks is an estimate only.  We don't know how many
+        # chunks there are until we do the actual segmentation.
         self.nchunks = self.duration / TranscodeObject.segment_duration
         self.trailer = self.duration % TranscodeObject.segment_duration
         if self.trailer:
@@ -340,6 +342,7 @@ class TranscodeObject(object):
         self.chunk_lock = threading.Lock()
         self.chunk_sem = threading.Semaphore(0)
         self.tmp_file = tempfile.TemporaryFile()
+        self.finished = False
 
         self.transcode_gate = threading.Event()
 
@@ -458,18 +461,28 @@ class TranscodeObject(object):
         self.tmp_file.write(d)
         if not d:
             self.tmp_file.flush()
-            self.tmp_file.seek(0, os.SEEK_SET)
             with self.chunk_lock:
-                self.chunk_buffer.append(self.tmp_file)
-                chunk_buffer_size = len(self.chunk_buffer)
-                if (chunk_buffer_size >= 
-                  TranscodeObject.buffer_high_watermark):
-                    logging.debug('TranscodeObject: throttling')
-                    self.chunk_throttle.clear()
-            # Tell consumer there is stuff available
+                # This is empty ... we haven't actually written anything.
+                # This an end of transcode marker.
+                if not self.tmp_file.tell():
+                    logging.debug('Transcode: end-of-transcode marker')
+                    self.finished = True
+                else:
+                    self.tmp_file.seek(0, os.SEEK_SET)
+                    self.chunk_buffer.append(self.tmp_file)
+                    chunk_buffer_size = len(self.chunk_buffer)
+                    if (chunk_buffer_size >= 
+                      TranscodeObject.buffer_high_watermark):
+                        logging.debug('TranscodeObject: throttling')
+                        self.chunk_throttle.clear()
+
+            # Tell consumer there is stuff available.  We do this for the
+            # end of transcode marker too.  Why?  Because it may be stuck in
+            # the chunk_sem acquiring state.  This will kick start it.
             self.chunk_sem.release()
             # ready for next segment
             self.tmp_file = tempfile.TemporaryFile()
+           
 
     # Data consumer from segmenter.  Here, we listen for incoming request.
     # no need to handle quit signal - the sink should return a zero read
@@ -479,8 +492,6 @@ class TranscodeObject(object):
             try:
                 r, w, x = select.select([self.sink.fileno()], [], [])
                 self.chunk_throttle.wait()
-                if self.segmenter_handle.poll() is not None:
-                    return
                 try:
                     self.sink.handle_request()
                 except socket.error, (err, errstring):
@@ -497,7 +508,13 @@ class TranscodeObject(object):
                 raise
 
     def get_chunk(self):
-        # Consume an item ...
+        # End of transcode check: if the transcode returned not enough
+        # chunks, then send an empty file.
+        with self.chunk_lock:
+            if self.finished and not self.chunk_buffer:
+                return tempfile.TemporaryFile()
+
+        # Consume an item
         self.chunk_sem.acquire()
         with self.chunk_lock:
             # If we got woken up, and there is nothing there, maybe it's
@@ -536,26 +553,40 @@ class TranscodeObject(object):
         self.transcode_gate.wait()
         try:
             self.ffmpeg_handle.kill()
+        except (AttributeError, OSError), e:
+            logging.debug('transcode shutdown: ffmpeg kill %s', e)
+        try:
             self.segmenter_handle.kill()
+        except (AttributeError, OSError), e:
+            logging.debug('transcode shutdown: segmenter kill %s', e)
+        try:
             # Wait for segmenter to die so that poll() will return not None
             self.segmenter_handle.wait()
+        except (AttributeError, OSError), e:
+            logging.debug('transcode shutdown: segmenter wait %s', e)
+        try:
             self.ffmpeg_handle.wait()
-            logging.info('TranscodeObject reaping sink')
+        except (AttributeError, OSError), e:
+            logging.debug('transcode shutdown: ffmpeg wait %s', e)
+        logging.info('TranscodeObject reaping sink')
+        try:
             # Close the server, to make select() on the sink fd return
             self.sink.socket.close()
-            # Nobody is producing mpegts packets at this point.  So if we 
-            # unthrottle here nobody should undo our good work
-            self.chunk_throttle.set()
+        except (OSError, AttributeError), e:
+            logging.debug('transcode shutdown: sink socket close %s', e)
+        # Nobody is producing mpegts packets at this point.  So if we 
+        # unthrottle here nobody should undo our good work
+        self.chunk_throttle.set()
+        try:
             self.sink_thread.join()
-            # Ensure we unblock the get_chunk().
-            self.chunk_sem.release()
-            logging.info('TranscodeObject sink reaped')
-            # Set these last: sink thread relies on it.
-            self.ffmpeg_handle = None
-            self.segmenter_handle = None
-            self.sink_thread = None
-        # Catch AttributeError in case it's not actually there, or OSError,
-        # in case it doesn't exist anymore.  We don't really care.
-        # Catch RuntimeError in case sink_thread hasn't been started yet.
-        except (RuntimeError, OSError, AttributeError):
-            pass
+        except (OSError, AttributeError, RuntimeError), e:
+            # Catch RuntimeError in case sink_thread hasn't been started yet.
+            logging.debug('transcode shutdown: sink join %s', e)
+
+        # Ensure we unblock the get_chunk().
+        self.chunk_sem.release()
+        logging.info('TranscodeObject sink reaped')
+        # Set these last: sink thread relies on it.
+        self.ffmpeg_handle = None
+        self.segmenter_handle = None
+        self.sink_thread = None
