@@ -14,6 +14,8 @@
 yum groupinstall "Development Tools" "Development Libraries"
 
 yum install \
+   ffmpeg-devel \
+   avahi-compat-libdns_sd \
    Pyrex \
    python-devel \
    pygtk2-devel \
@@ -27,5 +29,6 @@ yum install \
    rb_libtorrent-python \
    pywebkitgtk \
    gstreamer-python \
+   python-mutagen \
    ffmpeg \
    ffmpeg2theora
