@@ -21,6 +21,10 @@ aptitude install \
     libsoup2.4-dev
 
 aptitude install \
+    libavcodec-dev \
+    libavformat-dev \
+    libavutil-dev \
+    libavahi-compat-libdnssd1 \
     libtorrent-rasterbar5 \
     python-libtorrent \
     libwebkit-1.0-2 \
@@ -28,6 +32,7 @@ aptitude install \
     python-gst0.10 \
     python-gconf \
     python-pycurl \
+    python-mutagen \
     gstreamer0.10-ffmpeg \
     gstreamer0.10-plugins-base \
     gstreamer0.10-plugins-good \
