@@ -8,10 +8,10 @@
 #
 # This script must be run with sudo.
 
-# Last updated:    9/16/2010
+# Last updated:    2/12/2011
 # Last updated by: Will Kahn-Greene
 
-aptitude install \
+apt-get install \
     build-essential \
     git-core \
     pkg-config \
@@ -20,7 +20,11 @@ aptitude install \
     libwebkit-dev \
     libsoup2.4-dev
 
-aptitude install \
+apt-get install \
+    libavcodec-dev \
+    libavformat-dev \
+    libavutil-dev \
+    libavahi-compat-libdnssd1 \
     libtorrent-rasterbar6 \
     python-libtorrent \
     libwebkit-1.0-2 \
@@ -28,6 +32,7 @@ aptitude install \
     python-gst0.10 \
     python-gconf \
     python-pycurl \
+    python-mutagen \
     gstreamer0.10-ffmpeg \
     gstreamer0.10-plugins-base \
     gstreamer0.10-plugins-good \
@@ -35,5 +40,5 @@ aptitude install \
     gstreamer0.10-plugins-ugly \
     ffmpeg \
     ffmpeg2theora \
-    libavcodec-unstripped-52 \
-    libfaac0
+    libfaac0 \
+    python-appindicator
