@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,5 +28,6 @@
 # statement from all source files in the program, then also delete it here.
 
 use_native_buttons = True
-use_custom_titlebar_background = False
+use_custom_tablist_font = False
+use_upside_down_sort = True
 file_navigator_name = ""
