@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,7 +37,6 @@ import sys
 import logging
 import os
 import thread
-import shutil
 from threading import Event
 
 import pygst
@@ -46,10 +46,9 @@ import gst.interfaces
 import gtk
 
 # not sure why this isn't in the gst module, but it's easy to define
-GST_PLAY_FLAG_TEXT          = (1 << 2)
+GST_PLAY_FLAG_TEXT = (1 << 2)
 
 from miro import app
-from miro import config
 from miro import prefs
 from miro.util import gather_subtitle_files, copy_subtitle_file
 from miro.gtcache import gettext as _
@@ -60,12 +59,15 @@ from miro.frontends.widgets import menus
 from miro.frontends.widgets.widgetconst import MAX_VOLUME
 from miro.frontends.widgets.gtk.threads import call_on_ui_thread
 
+
 def to_seconds(t):
     return t / gst.SECOND
 
+
 def from_seconds(s):
     return s * gst.SECOND
 
+
 class Sniffer:
     """Determines whether a file is "audio", "video", or "unplayable".
     """
@@ -83,7 +85,11 @@ class Sniffer:
         self.bus.add_signal_watch()
         self.watch_id = self.bus.connect("message", self.on_bus_message)
 
-        self.playbin.set_property("uri", "file://%s" % filename)
+        try:
+            url = filename.urlize()
+        except AttributeError:
+            url = 'file://' + filename
+        self.playbin.set_property("uri", url)
         self.playbin.set_state(gst.STATE_PAUSED)
 
     def result(self, success_callback, error_callback):
@@ -126,6 +132,7 @@ class Sniffer:
         del self.audiosink
         del self.videosink
 
+
 class Renderer:
     def __init__(self):
         logging.info("GStreamer version: %s", gst.version_string())
@@ -133,18 +140,18 @@ class Renderer:
         self.rate = 1.0
         self.select_callbacks = None
 
-        audiosink_name = config.get(options.GSTREAMER_AUDIOSINK)
+        audiosink_name = app.config.get(options.GSTREAMER_AUDIOSINK)
         try:
             gst.element_factory_make(audiosink_name, "audiosink")
 
         except gst.ElementNotFoundError:
-            logging.info("gstreamerrenderer: ElementNotFoundError '%s'",
+            logging.warn("gstreamerrenderer: ElementNotFoundError '%s'",
                          audiosink_name)
             audiosink_name = "autoaudiosink"
             gst.element_factory_make(audiosink_name, "audiosink")
 
         except Exception, e:
-            logging.info("gstreamerrenderer: Exception thrown '%s'" % e)
+            logging.warn("gstreamerrenderer: Exception thrown '%s'" % e)
             logging.exception("sink exception")
             audiosink_name = "alsasink"
             gst.element_factory_make(audiosink_name, "audiosink")
@@ -156,6 +163,7 @@ class Renderer:
         self.playbin = None
         self.bus = None
         self.watch_ids = []
+        self.enabled_track = None
 
     def build_playbin(self):
         self.playbin = gst.element_factory_make("playbin2", "player")
@@ -187,7 +195,7 @@ class Renderer:
         try:
             callback()
         except StandardError:
-            logging.stacktrace("Error calling renderer callback")
+            logging.exception("Error calling renderer callback")
 
     def on_bus_message(self, bus, message):
         """receives message posted on the GstBus"""
@@ -201,7 +209,8 @@ class Renderer:
                 logging.error("on_bus_message: gstreamer error: %s", err)
             else:
                 err, debug = message.parse_error()
-                logging.error("on_bus_message (after callbacks): gstreamer error: %s", err)
+                logging.error("on_bus_message (after callbacks): "
+                              "gstreamer error: %s", err)
         elif message.type == gst.MESSAGE_STATE_CHANGED:
             prev, new, pending = message.parse_state_changed()
             if ((new == gst.STATE_PAUSED
@@ -225,7 +234,11 @@ class Renderer:
 
         self.iteminfo = iteminfo
 
-        self.playbin.set_property("uri", "file://%s" % iteminfo.video_path)
+        try:
+            url = iteminfo.video_path.urlize()
+        except AttributeError:
+            url = 'file://' + iteminfo.video_path
+        self.playbin.set_property("uri", url)
 
     def finish_select_file(self):
         pass
@@ -243,11 +256,12 @@ class Renderer:
             return self.get_current_time(attempt + 1)
 
     def _seek(self, seconds):
-        event = gst.event_new_seek(1.0,
-                                   gst.FORMAT_TIME,
-                                   gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
-                                   gst.SEEK_TYPE_SET, from_seconds(seconds),
-                                   gst.SEEK_TYPE_NONE, 0)
+        event = gst.event_new_seek(
+            1.0,
+            gst.FORMAT_TIME,
+            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
+            gst.SEEK_TYPE_SET, from_seconds(seconds),
+            gst.SEEK_TYPE_NONE, 0)
         result = self.playbin.send_event(event)
         if not result:
             logging.error("seek failed")
@@ -322,25 +336,27 @@ class Renderer:
                               gst.SEEK_TYPE_SET,
                               position + (rate * gst.SECOND))
 
+
 class AudioRenderer(Renderer):
     pass
 
+
 class VideoRenderer(Renderer):
     def __init__(self):
         Renderer.__init__(self)
 
-        videosink_name = config.get(options.GSTREAMER_IMAGESINK)
+        videosink_name = app.config.get(options.GSTREAMER_IMAGESINK)
         try:
             gst.element_factory_make(videosink_name, "videosink")
 
         except gst.ElementNotFoundError:
-            logging.info("gstreamerrenderer: ElementNotFoundError '%s'",
+            logging.warn("gstreamerrenderer: ElementNotFoundError '%s'",
                          videosink_name)
             videosink_name = "xvimagesink"
             gst.element_factory_make(videosink_name, "videosink")
 
         except Exception, e:
-            logging.info("gstreamerrenderer: Exception thrown '%s'" % e)
+            logging.warn("gstreamerrenderer: Exception thrown '%s'" % e)
             logging.exception("sink exception")
             videosink_name = "ximagesink"
             gst.element_factory_make(videosink_name, "videosink")
@@ -354,7 +370,8 @@ class VideoRenderer(Renderer):
 
     def build_playbin(self):
         Renderer.build_playbin(self)
-        self.watch_ids.append(self.bus.connect('sync-message::element', self.on_sync_message))
+        self.watch_ids.append(self.bus.connect(
+                'sync-message::element', self.on_sync_message))
         self.videosink = gst.element_factory_make(
             self.videosink_name, "videosink")
         self.playbin.set_property("video-sink", self.videosink)
@@ -363,7 +380,8 @@ class VideoRenderer(Renderer):
                 self.textsink_name, "textsink")
             self.playbin.set_property("text-sink", self.textsink)
         except TypeError:
-            logging.warning("this platform has an old version of playbin2--no subtitle support.")
+            logging.warning("this platform has an old version of "
+                            "playbin2--no subtitle support.")
             self.supports_subtitles = False
 
     def destroy_playbin(self):
@@ -379,14 +397,14 @@ class VideoRenderer(Renderer):
 
     def _setup_initial_subtitles(self, sub_filename):
         sub_index = -1
-        if (config.get(prefs.ENABLE_SUBTITLES) and self.supports_subtitles
+        if (app.config.get(prefs.ENABLE_SUBTITLES) and self.supports_subtitles
                 and not sub_filename):
             tracks = self.get_subtitles()
-            if 100 in tracks: # Select default sidecar file
+            if 100 in tracks:  # Select default sidecar file
                 sub_filename = tracks[100][1]
                 sub_index = 0
                 self.enabled_track = 100
-            elif 0 in tracks: # Select default embedded subtitle track
+            elif 0 in tracks:  # Select default embedded subtitle track
                 sub_index = 0
                 self.enabled_track = 0
 
@@ -515,10 +533,13 @@ class VideoRenderer(Renderer):
         if not self.supports_subtitles:
             return []
         tracks = [(index, filename)
-                  for index, (filename, language) in self.get_subtitles().items()]
+                  for index, (filename, language)
+                  in self.get_subtitles().items()]
         return tracks
 
     def get_enabled_subtitle_track(self):
+        if not self.playbin:
+            return None
         if not self.supports_subtitles:
             return None
         if self.enabled_track is not None:
@@ -558,18 +579,22 @@ class VideoRenderer(Renderer):
             return
         flags = self.playbin.get_property('flags')
         self.playbin.set_property('flags', flags & ~GST_PLAY_FLAG_TEXT)
+        self.enabled_track = None
 
     def select_subtitle_file(self, iteminfo, sub_path,
                              handle_successful_select):
         if not self.supports_subtitles:
             return
         subtitle_encoding = self.playbin.get_property("subtitle-encoding")
+
         def handle_ok():
             self.playbin.set_property("subtitle-encoding", subtitle_encoding)
             handle_successful_select()
+
         def handle_err():
             app.playback_manager.stop()
-        filenames = [filename for lang, filename in self.get_subtitles().values()]
+        filenames = [filename
+                     for lang, filename in self.get_subtitles().values()]
         if sub_path not in filenames:
             sub_path = copy_subtitle_file(sub_path, iteminfo.video_path)
         self.select_file(iteminfo, handle_ok, handle_err, sub_path)
@@ -664,11 +689,13 @@ class VideoRenderer(Renderer):
     def select_subtitle_encoding(self, encoding):
         self.playbin.set_property("subtitle-encoding", encoding)
 
+
 def movie_data_program_info(movie_path, thumbnail_path):
     extractor_path = os.path.join(os.path.split(__file__)[0],
                                   "gst_extractor.py")
     return ((sys.executable, extractor_path, movie_path, thumbnail_path), None)
 
+
 def get_item_type(item_info, success_callback, error_callback):
     s = Sniffer(item_info.video_path)
     s.result(success_callback, error_callback)
