@@ -232,41 +232,6 @@ def package_exists(package_name):
     return subprocess.call(['pkg-config', '--exists', package_name]) == 0
 
 
-def generate_miro():
-    f = open(os.path.join(platform_dir, "miro"), "w")
-    f.write(
-"""#!/bin/sh
-# This file is generated by setup.py.
-GDBDEBUG=0
-
-for arg in $@
-do
-    case $arg in
-    "--gdb")       GDBDEBUG=1;;
-    esac
-done
-
-if [ $GDBDEBUG = 1 ]
-then
-    echo "GDB DEBUGGING MODE."
-    PYTHON=`which python`
-    GDB=`which gdb`
-
-    if [ -z $GDB ]
-    then
-        echo "gdb cannot be found on your path.  aborting....";
-        exit;
-    fi
-
-    $GDB -ex 'set breakpoint pending on' \
-        -ex 'run' --args $PYTHON ./miro.real --sync "$@"
-else
-    miro.real "$@"
-fi
-""")
-    f.close()
-
-
 #### Xlib Extension ####
 ngrams_ext = \
     Extension("miro.ngrams",
@@ -388,7 +353,6 @@ data_files += [
 # if we're not doing "python setup.py clean", then we can do a bunch
 # of things that have file-related side-effects
 if not "clean" in sys.argv:
-    generate_miro()
     # gzip the man page
     os.system("gzip -9 < %s > %s" % (
             os.path.join(platform_dir, 'miro.1'),
@@ -475,8 +439,7 @@ class build(distutils.command.build.build):
                                     output_dir=output_dir)
         cc.link_executable(segmenter_objs, 'miro-segmenter',
                            output_dir=output_dir)
-        segmenter_exe = os.path.join(platform_dir, output_dir,
-                                     'miro-segmenter')
+        segmenter_exe = os.path.join(output_dir, 'miro-segmenter')
         self.distribution.scripts.append(segmenter_exe)
 
     def run(self):
