@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,13 +27,10 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-import os
-import re
 import sys
 import struct
 import logging
 import platform
-import urlparse
 import traceback
 import time
 
@@ -40,34 +38,35 @@ from objc import YES, NO, nil, signature
 from AppKit import *
 from Foundation import *
 from PyObjCTools import Conversion
-from ExceptionHandling import NSExceptionHandler, NSLogAndHandleEveryExceptionMask, NSStackTraceKey
 
 from miro import app
 from miro import prefs
-from miro import config
 from miro import downloader
 from miro import messages
-from miro import filetypes
 from miro import eventloop
 from miro import moviedata
+from miro import signals
 from miro import httpclient
 from miro import commandline
 from miro.frontends.widgets import menus
 from miro.frontends.widgets.application import Application
-from miro.plat import utils
 from miro.plat import growl
 from miro.plat import bundle
 from miro.plat import _growlImage
 from miro.plat import migrateappname
 from miro.plat.utils import ensureDownloadDaemonIsTerminated, filename_type_to_os_filename, os_filename_to_filename_type
 from miro.plat.utils import begin_thread_loop, finish_thread_loop
-from miro.plat.frontends.widgets import quicktime, osxmenus, sparkleupdater, threads
+from miro.plat.frontends.widgets import (widgetupdates, quicktime, osxmenus,
+        sparkleupdater, threads)
 from miro.plat.frontends.widgets.rect import Rect
 from miro.gtcache import gettext as _
 
 GROWL_GENERIC_NOTIFICATION = u'Misc'
 GROWL_DOWNLOAD_COMPLETE_NOTIFICATION = u'Download Complete'
 
+def run_application():
+    OSXApplication().run()
+
 class MiroApplication(NSApplication):
     def sendEvent_(self, event):
         try:
@@ -75,11 +74,25 @@ class MiroApplication(NSApplication):
         except:
             # should we just catch StandardError here?
             app.widgetapp.exception_handler(*sys.exc_info())
+        try:
+            app.widgetapp.emit("event-processed")
+        except:
+            # should we just catch StandardError here?
+            app.widgetapp.exception_handler(*sys.exc_info())
 
-class OSXApplication(Application):
+    def sendAction_to_from_(self, action, to, from_):
+        try:
+            return NSApplication.sendAction_to_from_(self, action, to, from_)
+        except:
+            # should we just catch StandardError here?
+            app.widgetapp.exception_handler(*sys.exc_info())
+            # tell Cocoa that we handled the event
+            return YES 
 
+class OSXApplication(Application, signals.SignalEmitter):
     def __init__(self):
         Application.__init__(self)
+        signals.SignalEmitter.__init__(self, "event-processed")
         self.gotQuit = False
 
     def run(self):
@@ -104,6 +117,7 @@ class OSXApplication(Application):
         # singleton
         miroapp = MiroApplication.sharedApplication()
         miroapp.setDelegate_(self.app_controller)
+        app.size_request_manager = widgetupdates.SizeRequestManager()
         NSApplicationMain(sys.argv)        
 
     def connect_to_signals(self):
@@ -118,7 +132,6 @@ class OSXApplication(Application):
             lambda cm: cm.connect('begin-loop', self.beginLoop))
         httpclient.register_on_start(
             lambda cm: cm.connect('end-loop', self.endLoop))
-        config.add_change_callback(self.on_pref_changed)
 
     def startup_ui(self):
         migrateappname.migrateVideos('Democracy', 'Miro')
@@ -127,8 +140,20 @@ class OSXApplication(Application):
         app.menu_manager.connect('enabled-changed', osxmenus.on_menu_change)
         app.playback_manager.connect('did-start-playing', osxmenus.on_playback_change)
         app.playback_manager.connect('did-stop', osxmenus.on_playback_change)
+        # add the Amazon cookie to Safari
+        storage = NSHTTPCookieStorage.sharedHTTPCookieStorage()
+        if 'dmusic_download_manager_enabled' not in [
+            cookie.name() for cookie in storage.cookiesForURL_(
+                NSURL.URLWithString_('http://www.amazon.com'))]:
+            storage.setCookie_(NSHTTPCookie.cookieWithProperties_(
+                    {NSHTTPCookieName: 'dmusic_download_manager_enabled',
+                     NSHTTPCookieValue: '1.0.3',
+                     NSHTTPCookieDomain: '.amazon.com',
+                     NSHTTPCookiePath: '/',
+                     NSHTTPCookieMaximumAge: 3600 * 365 * 10}))
         quicktime.register_components()
         quicktime.warm_up()
+        self.emit("event-processed")
 
     # This callback should only be called once, after startup is done.
     # (see superclass implementation)
@@ -136,16 +161,7 @@ class OSXApplication(Application):
         Application.on_window_show(self, window)
         self.app_controller.finish_startup()
         
-    def on_pref_changed(self, key, value):
-        """Any time a preference changes, this gets notified so that we
-        can adjust things
-
-        The callback happens on the backend thread, use call_on_ui_thread() to
-        move the call to the frontend thread.
-        """
-        self._on_pref_changed(key, value)
-
-    def _on_pref_changed(self, key, value):
+    def on_config_changed(self, obj, key, value):
         if key == prefs.RUN_AT_STARTUP.key:
             self.set_launch_at_startup(bool(value))
 
@@ -160,9 +176,9 @@ class OSXApplication(Application):
         if self.window is not None:
             windowFrame = self.window.nswindow.frame()
             windowFrame.size.height -= 22
-            config.set(prefs.LEFT_VIEW_SIZE, self.window.splitter.get_left_width())
-            config.set(prefs.MAIN_WINDOW_FRAME, NSStringFromRect(windowFrame))
-            config.save()
+            app.config.set(prefs.LEFT_VIEW_SIZE, self.window.splitter.get_left_width())
+            app.config.set(prefs.MAIN_WINDOW_FRAME, NSStringFromRect(windowFrame))
+            app.config.save()
         Application.do_quit(self)
 
     def quit_ui(self):
@@ -191,6 +207,17 @@ class OSXApplication(Application):
     def open_file(self, fn):
         filename = filename_type_to_os_filename(fn)
         ws = NSWorkspace.sharedWorkspace()
+        # Let's only open files that are of content type - we don't want to
+        # automatically launch executables here.
+        typ, error = ws.typeOfFile_error_(filename, None)
+        if not typ or error:
+            logging.warn('Cannot determine UTI of file %s' % fn)
+            return
+
+        if ws.type_conformsToType_(typ, 'public.executable'):
+            logging.warn('Not opening executable file %s' % fn)
+            return
+
         ok, externalApp, movieType = ws.getInfoForFile_application_type_(filename, None, None)
         if ok:
             if externalApp == bundle.getBundlePath():
@@ -202,9 +229,20 @@ class OSXApplication(Application):
             logging.warn("movie %s could not be externally opened" % fn)
     
     def get_main_window_dimensions(self):
-        windowFrame = config.get(prefs.MAIN_WINDOW_FRAME)
+        windowFrame = app.config.get(prefs.MAIN_WINDOW_FRAME)
         if windowFrame is None:
-            windowFrame = (0,0,800,600)
+            # windowFrame is None on first run.  in that case, we want
+            # to put Miro in the middle.
+            mainscreen = NSScreen.mainScreen()
+            rect = mainscreen.frame()
+
+            width = min(1200, rect.size.width)
+            height = min(800, rect.size.height)
+
+            x = (rect.size.width - width) / 2
+            y = (rect.size.height - height) / 2
+
+            windowFrame = (x, y, width, height)
         else:
             rect = NSRectFromString(windowFrame)
             windowFrame = (rect.origin.x, rect.origin.y, rect.size.width, rect.size.height)
@@ -217,7 +255,11 @@ class OSXApplication(Application):
     def handle_download_complete(self, obj, item):
         title = _('Download Completed')
         body = _('Download of video \'%s\' is finished.') % item.get_title()
-        icon = _growlImage.Image.imageFromPath(item.get_thumbnail())
+        icon = None
+        try:
+            icon = _growlImage.Image.imageFromPath(item.get_thumbnail())
+        except TypeError:
+            pass
         self.app_controller.growl_notifier.notify(GROWL_DOWNLOAD_COMPLETE_NOTIFICATION, title, body, icon=icon)
 
     def handle_unwatched_count_changed(self):
@@ -304,14 +346,12 @@ class AppController(NSObject):
         return self
 
     def setup_growl_notifier(self):
-        app_name = config.get(prefs.LONG_APP_NAME)
+        app_name = app.config.get(prefs.LONG_APP_NAME)
         notifications = [GROWL_GENERIC_NOTIFICATION, GROWL_DOWNLOAD_COMPLETE_NOTIFICATION]
         self.growl_notifier = growl.GrowlNotifier(app_name, notifications)
         self.growl_notifier.register()
 
     def applicationWillFinishLaunching_(self, notification):
-        NSExceptionHandler.defaultExceptionHandler().setExceptionHandlingMask_(NSLogAndHandleEveryExceptionMask)
-        NSExceptionHandler.defaultExceptionHandler().setDelegate_(self)
 
         man = NSAppleEventManager.sharedAppleEventManager()
         man.setEventHandler_andSelector_forEventClass_andEventID_(
@@ -353,7 +393,7 @@ class AppController(NSObject):
     def downloaderDaemonDidTerminate_(self, notification):
         task = notification.object()
         status = task.terminationStatus()
-        logging.info("Downloader daemon has been terminated (status: %d)" % status)
+        logging.warn("Downloader daemon has been terminated (status: %d)", status)
 
     def applicationWillTerminate_(self, notification):
         # Reset the application icon to its default state
@@ -363,28 +403,6 @@ class AppController(NSObject):
         ensureDownloadDaemonIsTerminated()    
         app.controller.on_shutdown()
 
-    def exceptionHandler_shouldLogException_mask_(self, handler, exception, mask):
-        logging.warn("Unhandled exception: %s", exception.name())
-        if os.path.exists("/usr/bin/atos"):
-            stack = exception.userInfo().objectForKey_(NSStackTraceKey)
-            if stack is None:
-                print "No stack available"
-            else:
-                pid = NSNumber.numberWithInt_(NSProcessInfo.processInfo().processIdentifier()).stringValue()
-                args = NSMutableArray.arrayWithCapacity_(20)
-                args.addObject_("-p")
-                args.addObject_(pid);
-                args.addObjectsFromArray_(stack.componentsSeparatedByString_("  "))
-            
-                task = NSTask.alloc().init()
-                task.setLaunchPath_("/usr/bin/atos");
-                task.setArguments_(args);
-                task.launch();
-        else:
-            import traceback
-            traceback.print_stack()
-        return NO
-
     def applicationShouldHandleReopen_hasVisibleWindows_(self, appl, flag):
         if app.widgetapp is not None and app.widgetapp.window is not None:
             mainWindow = app.widgetapp.window.nswindow
@@ -414,7 +432,7 @@ class AppController(NSObject):
                     self.pausedDownloaders.append(dl)
             dlCount = len(self.pausedDownloaders)
             if dlCount > 0:
-                logging.info("System is going to sleep, suspending %d download(s)." % dlCount)
+                logging.debug("System is going to sleep, suspending %d download(s).", dlCount)
                 for dl in self.pausedDownloaders:
                     dl.pause()
         dc = eventloop.add_urgent_call(lambda:pauseRunningDownloaders(), "Suspending downloaders for sleep")
@@ -427,7 +445,7 @@ class AppController(NSObject):
         def restartPausedDownloaders(self=self):
             dlCount = len(self.pausedDownloaders)
             if dlCount > 0:
-                logging.info("System is awake from sleep, resuming %s download(s)." % dlCount)
+                logging.debug("System is awake from sleep, resuming %s download(s).", dlCount)
                 try:
                     for dl in self.pausedDownloaders:
                         dl.start()
@@ -447,10 +465,14 @@ class AppController(NSObject):
         item = menus.osx_menu_structure.get(action)
 
         label = item.label
-        for state, actions in app.menu_manager.states.items():
-            if action in actions:
-                label = item.state_labels.get(state, item.label)
-                break
+        if action == 'PlayPauseItem':
+            label = item.state_labels.get(app.menu_manager.play_pause_state,
+                                          item.label)
+        else:
+            for state, actions in app.menu_manager.states.items():
+                if action in actions:
+                    label = item.state_labels.get(state, item.label)
+                    break
         menuitem.setTitleWithMnemonic_(label.replace("_", "&"))
 
         group_names = item.groups
