@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,7 +37,6 @@ from AppKit import *
 from Foundation import *
 from objc import YES, NO, nil
 
-from miro.plat import utils
 from miro.plat.frontends.widgets import drawing
 
 INFINITE = 1000000 # size of an "infinite" dimension
@@ -44,7 +44,7 @@ INFINITE = 1000000 # size of an "infinite" dimension
 class MiroLayoutManager(NSLayoutManager):
     """Overide NSLayoutManager to draw better underlines."""
 
-    def drawUnderlineForGlyphRange_underlineType_baselineOffset_lineFragmentRect_lineFragmentGlyphRange_containerOrigin_(self, glyph_range, type, offset, line_rect, line_glyph_range, container_origin):
+    def drawUnderlineForGlyphRange_underlineType_baselineOffset_lineFragmentRect_lineFragmentGlyphRange_containerOrigin_(self, glyph_range, typ, offset, line_rect, line_glyph_range, container_origin):
         container, _ = self.textContainerForGlyphAtIndex_effectiveRange_(glyph_range.location, None)
         rect = self.boundingRectForGlyphRange_inTextContainer_(glyph_range, container)
         x = container_origin.x + rect.origin.x
@@ -429,7 +429,8 @@ class StyledButton(object):
         x = x + self.PAD_HORIZONTAL
         y = y + (c_height - t_height) / 2
         if self.image is not None:
-            self.image.drawAtPoint_fromRect_operation_fraction_(NSPoint(x, y+3), NSZeroRect, NSCompositeSourceOver, 1.0)
+            self.image.drawAtPoint_fromRect_operation_fraction_(
+                NSPoint(x, y+3), NSZeroRect, NSCompositeSourceOver, 1.0)
             x += self.image.size().width + self.ICON_PAD
         else:
             y += 0.5
