@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,16 +29,17 @@
 
 """tablemodel.py -- Model classes for TableView.  """
 
-import itertools
+import logging
 
 from AppKit import NSDragOperationNone, NSTableViewDropOn, protocols
-from Foundation import NSObject, NSNotFound
+from Foundation import NSObject, NSNotFound, NSMutableIndexSet
 from objc import YES, NO, nil
 
 from miro import fasttypes
+from miro import infolist
 from miro import signals
+from miro.errors import WidgetActionError
 from miro.plat.frontends.widgets import wrappermap
-from miro.plat.frontends.widgets.simple import Image
 
 def list_from_nsindexset(index_set):
     rows = list()
@@ -89,9 +91,9 @@ class RowList(object):
 
     def nth_iter(self, index):
         if index < 0:
-            raise IndexError()
+            raise IndexError(index)
         elif index >= len(self):
-            return None
+            raise LookupError()
         if len(self.iter_cache) == 0:
             self.iter_cache.append(self.firstIter())
         try:
@@ -112,27 +114,28 @@ class TableModelBase(signals.SignalEmitter):
         self.row_list = RowList()
         self.column_types = column_types
         self.create_signal('row-changed')
-        self.create_signal('row-added')
-        self.create_signal('row-will-be-removed')
+        self.create_signal('structure-will-change')
 
     def check_column_values(self, column_values):
         if len(self.column_types) != len(column_values):
             raise ValueError("Wrong number of columns")
         # We might want to do more typechecking here
 
+    def get_column_data(self, row, column):
+        attr_map = column.identifier()
+        return dict((name, row[index]) for name, index in attr_map.items())
+
     def update_value(self, iter, index, value):
-        old_row = list(iter.value().values)
         iter.value().values[index] = value
-        self.emit('row-changed', iter, old_row)
+        self.emit('row-changed', iter)
 
     def update(self, iter, *column_values):
-        old_row = list(iter.value().values)
         iter.value().update_values(column_values)
-        self.emit('row-changed', iter, old_row)
+        self.emit('row-changed', iter)
 
     def remove(self, iter):
+        self.emit('structure-will-change')
         row_list = self.containing_list(iter)
-        self.emit('row-will-be-removed', iter)
         rv = row_list.remove(iter)
         if rv == row_list.lastIter():
             rv = None
@@ -192,7 +195,8 @@ class TableModel(TableModelBase):
         if row not in self.row_indexes:
             self.row_indexes[row] = index
 
-    def get_index_of_row(self, row):
+    def row_of_iter(self, tableview, iter):
+        row = iter.value()
         try:
             return self.row_indexes[row]
         except KeyError:
@@ -211,9 +215,9 @@ class TableModel(TableModelBase):
         return self.row_list
 
     def append(self, *column_values):
+        self.emit('structure-will-change')
         self.row_indexes = {}
         retval = self.row_list.append(TableRow(column_values))
-        self.emit('row-added', retval)
         return retval
 
     def remove(self, iter):
@@ -221,15 +225,84 @@ class TableModel(TableModelBase):
         return TableModelBase.remove(self, iter)
 
     def insert_before(self, iter, *column_values):
+        self.emit('structure-will-change')
         self.row_indexes = {}
         row = TableRow(column_values)
         retval = self.row_list.insertBefore(iter, row)
-        self.emit('row-added', retval)
         return retval
 
     def iter_for_row(self, tableview, row):
         return self.row_list.nth_iter(row)
 
+class InfoListModel(infolist.InfoList, signals.SignalEmitter):
+    # Wrap InfoList into to a model that we can use.
+    #
+    # Note: iterators here are just row indexes (type int)
+
+    def __init__(self, *args, **kwargs):
+        infolist.InfoList.__init__(self, *args, **kwargs)
+        signals.SignalEmitter.__init__(self, 'structure-will-change', 'row-changed')
+
+    def first_iter(self):
+        if len(self) > 0:
+            return 0
+        else:
+            raise IndexError(0)
+
+    def add_infos(self, *args, **kwargs):
+        self.emit('structure-will-change')
+        infolist.InfoList.add_infos(self, *args, **kwargs)
+
+    def remove_ids(self, *args, **kwargs):
+        self.emit('structure-will-change')
+        infolist.InfoList.remove_ids(self, *args, **kwargs)
+
+    def update_infos(self, *args, **kwargs):
+        self.emit('structure-will-change')
+        # HACK: update_infos might only change rows, so we maybe we only need
+        # to emit 'row-changed' here.  But most of the time we will re-sort
+        # things so it seems simpler just to emit structure-will-change
+        infolist.InfoList.update_infos(self, *args, **kwargs)
+
+    def remove_all(self, *args, **kwargs):
+        self.emit('structure-will-change')
+        infolist.InfoList.remove_all(self, *args, **kwargs)
+
+    def move_before(self, *args, **kwargs):
+        self.emit('structure-will-change')
+        infolist.InfoList.move_before(self, *args, **kwargs)
+
+    def change_sort(self, *args, **kwargs):
+        self.emit('structure-will-change')
+        infolist.InfoList.change_sort(self, *args, **kwargs)
+
+    def set_attr(self, id_, name, value):
+        infolist.InfoList.set_attr(self, id_, name, value)
+        row = self.index_of_id(id_)
+        self.emit('row-changed', row)
+
+    def unset_attr(self, id_, name):
+        infolist.InfoList.unset_attr(self, id_, name)
+        row = self.index_of_id(id_)
+        self.emit('row-changed', row)
+
+    def iter_for_row(self, tableview, row):
+        if row < 0:
+            raise IndexError(row)
+        elif row >= len(self):
+            raise WidgetActionError("iter past end of table")
+        else:
+            return row # iterators are just the row index
+
+    def row_of_iter(self, tableview, iter):
+        return iter # iterators are just the row index
+
+    def iter_for_id(self, id_):
+        return self.index_of_id(id_) # iterators are just the row index
+
+    def get_column_data(self, row, column):
+        return row
+
 class TreeNode(NSObject, TableRow):
     """A row in a TreeTableModel"""
 
@@ -259,6 +332,7 @@ class TreeTableModel(TableModelBase):
         return self.row_list_for_iter(iter.value().parent)
 
     def row_list_for_iter(self, iter):
+        """Return the rows of all direct children of iter."""
         if iter is None:
             return self.row_list
         else:
@@ -269,8 +343,8 @@ class TreeTableModel(TableModelBase):
         return iter
 
     def append(self, *column_values):
+        self.emit('structure-will-change')
         retval = self.row_list.append(TreeNode.create_(column_values, None))
-        self.emit('row-added', retval)
         return self.remember_iter(retval)
 
     def forget_iter_for_item(self, item):
@@ -279,19 +353,20 @@ class TreeTableModel(TableModelBase):
             self.forget_iter_for_item(child)
 
     def remove(self, iter):
+        self.emit('structure-will-change')
         self.forget_iter_for_item(iter.value())
         return TableModelBase.remove(self, iter)
 
     def insert_before(self, iter, *column_values):
+        self.emit('structure-will-change')
         row = TreeNode.create_(column_values, self.parent_iter(iter))
         retval = self.containing_list(iter).insertBefore(iter, row)
-        self.emit('row-added', retval)
         return self.remember_iter(retval)
 
     def append_child(self, iter, *column_values):
+        self.emit('structure-will-change')
         row_list = self.row_list_for_iter(iter)
         retval = row_list.append(TreeNode.create_(column_values, iter))
-        self.emit('row-added', retval)
         return self.remember_iter(retval)
 
     def child_iter(self, iter):
@@ -306,7 +381,7 @@ class TreeTableModel(TableModelBase):
         return row_list.nth_iter(index)
 
     def has_child(self, iter):
-        return  len(iter.value().children) > 0
+        return len(iter.value().children) > 0
 
     def children_count(self, iter):
         if iter is not None:
@@ -321,17 +396,35 @@ class TreeTableModel(TableModelBase):
         return iter.value().parent
 
     def iter_for_row(self, tableview, row):
-        return self.iter_for_item[tableview.itemAtRow_(row)]
+        item = tableview.itemAtRow_(row)
+        if item in self.iter_for_item:
+            return self.iter_for_item[item]
+        elif item == -1:
+            raise WidgetActionError("no item at row %s" % row)
+        else:
+            raise WidgetActionError("no iter for item %s at row %s" %
+                                   (repr(item), row))
 
-def get_column_data(row, column):
-    attr_map = column.identifier()
-    return dict((name, row[index]) for name, index in attr_map.items())
+    def row_of_iter(self, tableview, iter):
+        item = iter.value()
+        row = tableview.rowForItem_(item)
+        if row == -1:
+            raise LookupError("%s is not in this table" % repr(item))
+        return row
+
+    def get_path(self, iter_):
+        """Not implemented (yet?) for Cocoa. Currently the only place this is
+        needed is tablistmanager, where the situation that uses paths results
+        from GTK peculiarities.
+        """
+        return NotImplemented
 
 class DataSourceBase(NSObject):
     def initWithModel_(self, model):
         self.model = model
         self.drag_source = None
         self.drag_dest = None
+        return self
 
     def setDragSource_(self, drag_source):
         self.drag_source = drag_source
@@ -347,8 +440,8 @@ class DataSourceBase(NSObject):
         if not drag_data:
             return NO
         pasteboard.declareTypes_owner_(drag_data.keys(), self)
-        for type, value in drag_data.items():
-            pasteboard.setString_forType_(value, type)
+        for typ, value in drag_data.items():
+            pasteboard.setString_forType_(repr(value), typ)
         return YES
 
     def calcType_(self, drag_info):
@@ -361,10 +454,10 @@ class DataSourceBase(NSObject):
 
     def validateDrop_dragInfo_parentIter_position_(self, view, drag_info, 
             parent, position):
-        type = self.calcType_(drag_info)
-        if type:
+        typ = self.calcType_(drag_info)
+        if typ:
             wrapper = wrappermap.wrapper(view)
-            return self.drag_dest.validate_drop(wrapper, self.model, type,
+            return self.drag_dest.validate_drop(wrapper, self.model, typ,
                     drag_info.draggingSourceOperationMask(), parent,
                     position)
         else:
@@ -372,13 +465,14 @@ class DataSourceBase(NSObject):
 
     def acceptDrop_dragInfo_parentIter_position_(self, view, drag_info,
             parent, position):
-        type = self.calcType_(drag_info)
-        if type:
-            data = drag_info.draggingPasteboard().stringForType_(type)
+        typ = self.calcType_(drag_info)
+        if typ:
+            ids = eval(drag_info.draggingPasteboard().stringForType_(typ))
             wrapper = wrappermap.wrapper(view)
-            return self.drag_dest.accept_drop(wrapper, self.model, type, 
-                    drag_info.draggingSourceOperationMask(), parent,
-                    position, data)
+            self.drag_dest.accept_drop(wrapper, self.model, typ, 
+                drag_info.draggingSourceOperationMask(), parent,
+                position, ids)
+            return YES
         else:
             return NO
 
@@ -389,7 +483,7 @@ class MiroTableViewDataSource(DataSourceBase, protocols.NSTableDataSource):
     def tableView_objectValueForTableColumn_row_(self, table_view, column, row):
         node = self.model.nth_iter(row).value()
         self.model.remember_row_at_index(node, row)
-        return get_column_data(node.values, column)
+        return self.model.get_column_data(node.values, column)
 
     def tableView_writeRowsWithIndexes_toPasteboard_(self, tableview, rowIndexes,
             pasteboard):
@@ -400,7 +494,7 @@ class MiroTableViewDataSource(DataSourceBase, protocols.NSTableDataSource):
 
     def translateRow_operation_(self, row, operation):
         if operation == NSTableViewDropOn:
-            return (self.model.nth_iter(row),), -1
+            return self.model.nth_iter(row), -1
         else:
             return None, row
 
@@ -416,6 +510,23 @@ class MiroTableViewDataSource(DataSourceBase, protocols.NSTableDataSource):
         return self.acceptDrop_dragInfo_parentIter_position_(tableview, 
                 drag_info, parent, position)
 
+class MiroInfoListDataSource(MiroTableViewDataSource):
+    def translateRow_operation_(self, row, operation):
+        if operation == NSTableViewDropOn:
+            return row, -1
+        else:
+            return None, row
+
+    def tableView_objectValueForTableColumn_row_(self, table_view, column, row):
+        return self.model[row]
+
+    def tableView_writeRowsWithIndexes_toPasteboard_(self, tableview,
+            rowIndexes, pasteboard):
+        indexes = list_from_nsindexset(rowIndexes)
+        data = [self.model[i] for i in indexes]
+        return self.view_writeColumnData_ToPasteboard_(tableview, data,
+                pasteboard)
+
 class MiroOutlineViewDataSource(DataSourceBase, protocols.NSOutlineViewDataSource):
     def outlineView_child_ofItem_(self, view, child, item):
         if item is nil:
@@ -438,7 +549,7 @@ class MiroOutlineViewDataSource(DataSourceBase, protocols.NSOutlineViewDataSourc
 
     def outlineView_objectValueForTableColumn_byItem_(self, view, column,
             item):
-        return get_column_data(item.values, column)
+        return self.model.get_column_data(item.values, column)
 
     def outlineView_writeItems_toPasteboard_(self, outline_view, items, 
             pasteboard):
