@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,17 +29,21 @@
 
 """miro.plat.frontends.widgets.window -- Top-level Window class.  """
 
+import logging
+
 from AppKit import *
 from Foundation import *
 from objc import YES, NO, nil
 from PyObjCTools import AppHelper
 
 from miro import signals
+from miro import app
+from miro import prefs
 from miro.frontends.widgets import widgetconst
 from miro.plat.frontends.widgets import osxmenus
 from miro.plat.frontends.widgets import wrappermap
 from miro.plat.frontends.widgets.helpers import NotificationForwarder
-from miro.plat.frontends.widgets.base import Widget, Container, FlippedView
+from miro.plat.frontends.widgets.base import Widget, FlippedView
 from miro.plat.frontends.widgets.layout import VBox, HBox, Alignment
 from miro.plat.frontends.widgets.control import Button
 from miro.plat.frontends.widgets.simple import Label
@@ -49,28 +54,121 @@ from miro.plat.utils import filename_to_unicode
 # object stay alive as long as the window is alive.
 alive_windows = set()
 
-class MiroWindow(NSWindow):
-    def handle_keyDown(self, event):
+class MiroResponderInterceptor(NSResponder):
+    """Intercepts cocoa events and gives our wrappers and chance to handle
+    them first.
+    """
+    
+    def initWithResponder_(self, responder):
+        """Initialize a MiroResponderInterceptor
+
+        We will give the wrapper for responder a chance to handle the event,
+        then pass it along to responder.
+        """
+        self.responder = responder
+
+    def keyDown_(self, event):
+        if self.sendKeyDownToWrapper_(event):
+            return # signal handler returned True, stop processing
+
+        # If our responder is the last in the chain, we can stop intercepting
+        if self.responder.nextResponder() is None:
+            self.responder.keyDown_(event)
+            return
+
+        # Here's the tricky part, we want to call keyDown_ on our responder,
+        # but if it doesn't handle the event, then it will pass it along to
+        # it's next responder.  We need to set things up so that we will
+        # intercept that call.
+
+        # Make a new MiroResponderInterceptor whose responder is the next
+        # responder down the chain.
+        next_intercepter = MiroResponderInterceptor.alloc()
+        next_intercepter.initWithResponder_(self.responder.nextResponder())
+        # Install the interceptor
+        self.responder.setNextResponder_(next_intercepter)
+        # Send event along
+        self.responder.keyDown_(event)
+        # Restore old nextResponder value
+        self.responder.setNextResponder_(next_intercepter.responder)
+
+    def sendKeyDownToWrapper_(self, event):
+        """Give a keyDown event to the wrapper for our responder
+        
+        Return True if the wrapper handled the event
+        """
         key = event.charactersIgnoringModifiers()
-        if len(key) != 1 or not key.isalpha():
+        if len(key) != 1 or not key.isalnum():
             key = osxmenus.REVERSE_KEYS_MAP.get(key)
         mods = osxmenus.translate_event_modifiers(event)
-        responder = self.firstResponder()
-        while responder is not None:
-            wrapper = wrappermap.wrapper(responder)
-            if isinstance(wrapper, Widget) or isinstance(wrapper, Window):
-                if wrapper.emit('key-press', key, mods):
-                    return True # signal handler returned True, stop processing
-            responder = responder.nextResponder()
+        wrapper = wrappermap.wrapper(self.responder)
+        if isinstance(wrapper, Widget) or isinstance(wrapper, Window):
+            if wrapper.emit('key-press', key, mods):
+                return True
+        return False
+
+class MiroWindow(NSWindow):
+    def initWithContentRect_styleMask_backing_defer_(self, rect, mask,
+            backing, defer):
+        self = NSWindow.initWithContentRect_styleMask_backing_defer_(self,
+                rect, mask, backing, defer)
+        self._last_focus_chain = None
+        return self
+
+    def handleKeyDown_(self, event):
+        if self.handle_tab_navigation(event):
+            return
+        interceptor = MiroResponderInterceptor.alloc()
+        interceptor.initWithResponder_(self.firstResponder())
+        interceptor.keyDown_(event)
+
+    def handle_tab_navigation(self, event):
+        """Handle tab navigation through the window.
+
+        :returns: True if we handled the event
+        """
+        keystr = event.charactersIgnoringModifiers()
+        if keystr[0] == NSTabCharacter:
+            # handle cycling through views with Tab.
+            self.focusNextKeyView_(True)
+            return True
+        elif keystr[0] == NSBackTabCharacter:
+            self.focusNextKeyView_(False)
+            return True
+        return False
+
+    def acceptsMouseMovedEvents(self):
+        # HACK: for some reason calling setAcceptsMouseMovedEvents_() doesn't
+        # work, we have to forcefully override this method.
+        return NO
 
     def sendEvent_(self, event):
         if event.type() == NSKeyDown:
-            # We grab the KeyDown event here instead of with a keyDown handler
-            # because some keys (notable Cmd-> and Cmd-< aren't caught by that
-            # method.
-            if self.handle_keyDown(event):
-                return
-        return NSWindow.sendEvent_(self, event)
+            self.handleKeyDown_(event)
+        else:
+            NSWindow.sendEvent_(self, event)
+
+    def _calc_current_focus_wrapper(self):
+        responder = self.firstResponder()
+        while responder:
+            wrapper = wrappermap.wrapper(responder)
+            # check if we have a wrapper for the view, if not try the parent
+            # view
+            if wrapper is not None:
+                return wrapper
+            responder = responder.superview()
+        return None
+
+    def focusNextKeyView_(self, is_forward):
+        current_focus = self._calc_current_focus_wrapper()
+        my_wrapper = wrappermap.wrapper(self)
+        next_focus = my_wrapper.get_next_tab_focus(current_focus, is_forward)
+        if next_focus is not None:
+            next_focus.focus()
+
+class MainMiroWindow(MiroWindow):
+    def isMovableByWindowBackground(self):
+        return YES
 
 class Window(signals.SignalEmitter):
     """See https://develop.participatoryculture.org/index.php/WidgetAPI for a description of the API for this class."""
@@ -82,7 +180,8 @@ class Window(signals.SignalEmitter):
         self.create_signal('key-press')
         self.create_signal('show')
         self.create_signal('hide')
-        self.nswindow = MiroWindow.alloc().initWithContentRect_styleMask_backing_defer_(
+        self.create_signal('on-shown')
+        self.nswindow = MainMiroWindow.alloc().initWithContentRect_styleMask_backing_defer_(
                 rect.nsrect,
                 self.get_style_mask(),
                 NSBackingStoreBuffered,
@@ -104,12 +203,27 @@ class Window(signals.SignalEmitter):
         wrappermap.add(self.nswindow, self)
         alive_windows.add(self)
 
+    def get_next_tab_focus(self, current, is_forward):
+        """Return the next widget to cycle through for keyboard focus
+
+        Subclasses can override this to for find-grained control of keyboard
+        focus.
+
+        :param current: currently-focused widget
+        :param is_forward: are we tabbing forward?
+        """
+        return None
+
     def get_style_mask(self):
-        return NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask | NSResizableWindowMask
+        return (NSTitledWindowMask | NSClosableWindowMask |
+                NSMiniaturizableWindowMask | NSResizableWindowMask)
 
     def set_title(self, title):
         self.nswindow.setTitle_(title)
 
+    def get_title(self):
+        return self.nswindow.title()
+
     def on_frame_change(self, notification):
         self.place_child()
 
@@ -123,23 +237,34 @@ class Window(signals.SignalEmitter):
         self.emit('did-move')
 
     def on_will_close(self, notification):
+        # unset the first responder.  This allows text entry widgets to get
+        # the NSControlTextDidEndEditingNotification
+        self.nswindow.makeFirstResponder_(nil)
         self.emit('will-close')
         self.emit('hide')
 
     def is_active(self):
         return self.nswindow.isMainWindow()
 
+    def is_visible(self):
+        return self.nswindow.isVisible()
+
     def show(self):
         if self not in alive_windows:
             raise ValueError("Window destroyed")
         self.nswindow.makeKeyAndOrderFront_(nil)
         self.nswindow.makeMainWindow()
         self.emit('show')
+        # Cocoa doesn't apply default selections as forcefully as GTK, so
+        # currently there's no need for on-shown to actually wait until the
+        # window has been shown here
+        self.emit('on-shown')
 
     def close(self):
         self.nswindow.close()
 
     def destroy(self):
+        self.close()
         self.window_notifications.disconnect()
         self.view_notifications.disconnect()
         self.nswindow.setContentView_(nil)
@@ -174,7 +299,20 @@ class Window(signals.SignalEmitter):
 
     def update_size_constraints(self):
         width, height = self.content_widget.get_size_request()
-        self.nswindow.setContentMinSize_(NSSize(width, height))
+        # It is possible the window is torn down between the size invalidate
+        # request and the actual size invalidation invocation.  So check
+        # to see if nswindow is there if not then do not do anything.
+        if self.nswindow:
+            # FIXME: I'm not sure that this code does what we want it to do.
+            # It enforces the min-size when the user drags the window, but I
+            # think it should also call setContentSize_ if the window is
+            # currently too small to fit the content - BDK
+            self.nswindow.setContentMinSize_(NSSize(width, height))
+            rect = self.nswindow.contentRectForFrameRect_(self.nswindow.frame())
+            if rect.size.width < width or rect.size.height < height:
+                logging.warn("Content widget too large for this window "
+                        "size available: %dx%d widget size: %dx%d",
+                        rect.size.width, rect.size.height, width, height)
 
     def get_content_widget(self):
         return self.content_widget
@@ -192,6 +330,7 @@ class MainWindow(Window):
     def __init__(self, title, rect):
         Window.__init__(self, title, rect)
         self.nswindow.setReleasedWhenClosed_(NO)
+
     def close(self):
         self.nswindow.orderOut_(nil)
 
@@ -201,6 +340,10 @@ class DialogBase(object):
     def set_transient_for(self, window):
         self.sheet_parent = window
 
+class MiroPanel(NSPanel):
+    def cancelOperation_(self, event):
+        wrappermap.wrapper(self).end_with_code(-1)
+
 class Dialog(DialogBase):
     def __init__(self, title, description=None):
         DialogBase.__init__(self)
@@ -218,6 +361,9 @@ class Dialog(DialogBase):
         self.buttons.append(button)
 
     def on_button_clicked(self, button, code):
+        self.end_with_code(code)
+
+    def end_with_code(self, code):
         if self.sheet_parent is not None:
             NSApp().endSheet_returnCode_(self.window, code)
         else:
@@ -255,7 +401,7 @@ class Dialog(DialogBase):
         self.content_widget = self.build_content()
         width, height = self.content_widget.get_size_request()
         width = max(width, 400)
-        window = NSPanel.alloc()
+        window = MiroPanel.alloc()
         window.initWithContentRect_styleMask_backing_defer_(
                 NSMakeRect(400, 400, width, height),
                 NSTitledWindowMask, NSBackingStoreBuffered, NO)
@@ -268,11 +414,45 @@ class Dialog(DialogBase):
             self.buttons[0].make_default()
         return window
 
+    def hookup_content_widget_signals(self):
+        self.size_req_handler = self.content_widget.connect(
+                'size-request-changed',
+                self.on_content_widget_size_request_change)
+
+    def unhook_content_widget_signals(self):
+        self.content_widget.disconnect(self.size_req_handler)
+        self.size_req_handler = None
+
+    def on_content_widget_size_request_change(self, widget, old_size):
+        width, height = self.content_widget.get_size_request()
+        # It is possible the window is torn down between the size invalidate
+        # request and the actual size invalidation invocation.  So check
+        # to see if nswindow is there if not then do not do anything.
+        if self.window and (width, height) != old_size:
+            self.change_content_size(width, height)
+
+    def change_content_size(self, width, height):
+        content_rect = self.window.contentRectForFrameRect_(
+                self.window.frame())
+        # Cocoa's coordinate system is funky, adjust y so that the top stays
+        # in place
+        content_rect.origin.y += (content_rect.size.height - height)
+        # change our frame to fit the new content.  It would be nice to
+        # animate the change, but timers don't work when we are displaying a
+        # modal dialog
+        content_rect.size = NSSize(width, height)
+        new_frame = self.window.frameRectForContentRect_(content_rect)
+        self.window.setFrame_display_(new_frame, NO)
+
     def run(self):
         self.window = self.build_window()
+        wrappermap.add(self.window, self)
+        self.hookup_content_widget_signals()
         self.running = True
         if self.sheet_parent is None:
             response = NSApp().runModalForWindow_(self.window)
+            if self.window:
+                self.window.close()
         else:
             delegate = SheetDelegate.alloc().init()
             NSApp().beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo_(
@@ -284,6 +464,7 @@ class Dialog(DialogBase):
                 # the dialog
                 self.window.orderOut_(nil)
         self.running = False
+        self.unhook_content_widget_signals()
 
         if response < 0:
             return -1
@@ -293,8 +474,9 @@ class Dialog(DialogBase):
         if self.running:
             NSApp().stopModalWithCode_(-1)
 
-        self.window.setContentView_(None)
-        self.window.close()
+        if self.window is not None:
+            self.window.setContentView_(None)
+            self.window.close()
         self.window = None
         self.buttons = None
         self.extra_widget = None
@@ -368,6 +550,9 @@ class FileSaveDialog(FileDialogBase):
     def destroy(self):
         self._panel = None
 
+    set_path = set_filename
+    get_path = get_filename
+
 class FileOpenDialog(FileDialogBase):
     def __init__(self, title):
         FileDialogBase.__init__(self)
@@ -395,9 +580,15 @@ class FileOpenDialog(FileDialogBase):
             self._types += t
 
     def get_filename(self):
+        if self._filenames is None:
+            # canceled
+            return None
         return self.get_filenames()[0]
 
     def get_filenames(self):
+        if self._filenames is None:
+            # canceled
+            return []
         # Use encode('utf-8') instead of unicode_to_filename, because
         # unicode_to_filename has code to make sure nextFilename works, but it's
         # more important here to not change the filename.
@@ -414,6 +605,9 @@ class FileOpenDialog(FileDialogBase):
     def destroy(self):
         self._panel = None
 
+    set_path = set_filename
+    get_path = get_filename
+
 class DirectorySelectDialog(FileDialogBase):
     def __init__(self, title):
         FileDialogBase.__init__(self)
@@ -443,9 +637,16 @@ class DirectorySelectDialog(FileDialogBase):
     def destroy(self):
         self._panel = None
 
+    set_path = set_directory
+    get_path = get_directory
+
 class AboutDialog(DialogBase):
     def run(self):
-        NSApplication.sharedApplication().orderFrontStandardAboutPanel_(nil)
+        optionsDictionary = dict()
+        revision = app.config.get(prefs.APP_REVISION_NUM)
+        if revision:
+            optionsDictionary['Version'] = revision
+        NSApplication.sharedApplication().orderFrontStandardAboutPanelWithOptions_(optionsDictionary)
     def destroy(self):
         pass
 
@@ -463,15 +664,15 @@ class AlertDialog(DialogBase):
     def destroy(self):
         self._nsalert = nil
 
-class PreferenceItem (NSToolbarItem):
+class PreferenceItem(NSToolbarItem):
 
     def setPanel_(self, panel):
         self.panel = panel
 
-class ToolbarDelegate (NSObject):
+class PreferenceToolbarDelegate(NSObject):
 
     def initWithPanels_identifiers_window_(self, panels, identifiers, window):
-        self = super(ToolbarDelegate, self).init()
+        self = super(PreferenceToolbarDelegate, self).init()
         self.panels = panels
         self.identifiers = identifiers
         self.window = window
@@ -486,11 +687,13 @@ class ToolbarDelegate (NSObject):
     def toolbarSelectableItemIdentifiers_(self, toolbar):
         return self.identifiers
 
-    def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_(self, toolbar, itemIdentifier, flag):
+    def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_(self, toolbar,
+                                                                 itemIdentifier,
+                                                                 flag):
         panel = self.panels[itemIdentifier]
         item = PreferenceItem.alloc().initWithItemIdentifier_(itemIdentifier)
         item.setLabel_(panel[1])
-        item.setImage_(NSImage.imageNamed_(u"pref_item_%s" % itemIdentifier))
+        item.setImage_(NSImage.imageNamed_(u"pref_tab_%s" % itemIdentifier))
         item.setAction_("switchPreferenceView:")
         item.setTarget_(self)
         item.setPanel_(panel[0])
@@ -502,7 +705,19 @@ class ToolbarDelegate (NSObject):
     def switchPreferenceView_(self, sender):
         self.window.do_select_panel(sender.panel, YES)
 
-class PreferencesWindow (Window):
+class DialogWindow(Window):
+    def __init__(self, title, rect, allow_miniaturize=False):
+        self.allow_miniaturize = allow_miniaturize
+        Window.__init__(self, title, rect)
+        self.nswindow.setShowsToolbarButton_(NO)
+
+    def get_style_mask(self):
+        mask = (NSTitledWindowMask | NSClosableWindowMask)
+        if self.allow_miniaturize:
+            mask |= NSMiniaturizableWindowMask
+        return mask
+
+class PreferencesWindow(Window):
     def __init__(self, title):
         Window.__init__(self, title, Rect(0, 0, 640, 440))
         self.panels = dict()
@@ -519,14 +734,15 @@ class PreferencesWindow (Window):
         self.app_notifications.disconnect()
 
     def get_style_mask(self):
-        return NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask
+        return (NSTitledWindowMask | NSClosableWindowMask |
+                NSMiniaturizableWindowMask)
  
     def append_panel(self, name, panel, title, image_name):
         self.panels[name] = (panel, title)
         self.identifiers.append(name)
 
     def finish_panels(self):
-        self.tbdelegate = ToolbarDelegate.alloc().initWithPanels_identifiers_window_(self.panels, self.identifiers, self)
+        self.tbdelegate = PreferenceToolbarDelegate.alloc().initWithPanels_identifiers_window_(self.panels, self.identifiers, self)
         toolbar = NSToolbar.alloc().initWithIdentifier_(u"Preferences")
         toolbar.setAllowsUserCustomization_(NO)
         toolbar.setDelegate_(self.tbdelegate)
@@ -541,8 +757,8 @@ class PreferencesWindow (Window):
     def do_select_panel(self, panel, animate):
         wframe = self.nswindow.frame()
         vsize = list(panel.get_size_request())
-        if vsize[0] < 500:
-            vsize[0] = 500
+        if vsize[0] < 650:
+            vsize[0] = 650
         if vsize[1] < 200:
             vsize[1] = 200
 
@@ -561,3 +777,16 @@ class PreferencesWindow (Window):
 
     def on_app_quit(self, notification):
         self.close()
+
+def get_first_time_dialog_coordinates(width, height):
+    """Returns the coordinates for the first time dialog.
+    """
+    # windowFrame is None on first run.  in that case, we want
+    # to put Miro in the middle.
+    mainscreen = NSScreen.mainScreen()
+    rect = mainscreen.frame()
+
+    x = (rect.size.width - width) / 2
+    y = (rect.size.height - height) / 2
+
+    return x, y
