@@ -343,6 +343,18 @@ def kill_process(pid):
         except:
             logging.exception ("error killing process")
 
+def _app_command_line():
+    """Get the command line to lanch the Miro.app bundle. """
+    exe = NSBundle.mainBundle().executablePath()
+
+    os_info = os.uname()
+    os_version = int(os_info[2].split('.')[0])
+    if os_version < 9:
+        return [exe]
+    else:
+        arch = subprocess.Popen("/usr/bin/arch", stdout=subprocess.PIPE).communicate()[0].strip()
+        return ['/usr/bin/arch', '-%s' % arch, exe]
+
 @on_ui_thread
 def launch_download_daemon(oldpid, env):
     kill_process(oldpid)
@@ -352,17 +364,9 @@ def launch_download_daemon(oldpid, env):
     env["MIRO_APP_VERSION"] = app.config.get(prefs.APP_VERSION)
     env.update(os.environ)
 
-    exe = NSBundle.mainBundle().executablePath()
-
-    os_info = os.uname()
-    os_version = int(os_info[2].split('.')[0])
-    if os_version < 9:
-        launch_path = exe
-        launch_arguments = [u'--download-daemon']
-    else:
-        arch = subprocess.Popen("arch", stdout=subprocess.PIPE).communicate()[0].strip()
-        launch_path = '/usr/bin/arch'
-        launch_arguments = ['-%s' % arch, exe, u'--download-daemon']
+    command_line = _app_command_line()
+    launch_path = command_line[0]
+    launch_arguments = command_line[1:] + [u'--download-daemon']
 
     global dlTask
     dlTask = NSTask.alloc().init()
@@ -407,11 +411,17 @@ def exit_miro(return_code):
 
 ###############################################################################
 
-def movie_data_program_info(movie_path, thumbnail_path):
+def _get_cmd_line_and_env_for_script(script_name):
+    """Get command line and env variables for a script.
+
+    script_name must be located in the resources directory.
+
+    Returns the tuple (cmd_line, env)
+    """
     main_bundle = NSBundle.mainBundle()
     bundle_path = main_bundle.bundlePath()
     rsrc_path = main_bundle.resourcePath()
-    script_path = os.path.join(rsrc_path, 'qt_extractor.py')
+    script_path = os.path.join(rsrc_path, script_name)
     options = main_bundle.infoDictionary().get('PyOptions')
     env = None
     if options['alias'] == 1:
@@ -433,7 +443,19 @@ def movie_data_program_info(movie_path, thumbnail_path):
             check_b(env[k])
         except:
             env[k] = env[k].encode('utf-8')
-    return ((py_exe_path, script_path, movie_path, thumbnail_path), env)
+    return ((py_exe_path, script_path), env)
+
+def movie_data_program_info(movie_path, thumbnail_path):
+    cmd_line, env = _get_cmd_line_and_env_for_script('qt_extractor.py')
+    return (cmd_line + (movie_path, thumbnail_path), env)
+
+def miro_helper_program_info():
+    cmd_line = _app_command_line() + [u'--miro-helper']
+    env = {
+            'VERSIONER_PYTHON_PREFER_32_BIT': 'yes',
+            'MIRO_APP_VERSION': app.config.get(prefs.APP_VERSION)
+    }
+    return cmd_line, env
 
 ###############################################################################
 
@@ -515,13 +537,15 @@ def get_logical_cpu_count():
     try:
         import multiprocessing
         return multiprocessing.cpu_count()
-    except ImportError:
+    except (NotImplementedError, ImportError):
         try:
             ncpus = os.sysconf("SC_NPROCESSORS_ONLN")
             if isinstance(ncpus, int) and ncpus > 0:
                 return ncpus
-        except:
-            return int(os.popen2("sysctl -n hw.ncpu")[1].read())
+        # NB: No need to catch AttributeError for os.sysconf, or TypeError in
+        # int(cpus) conversion.
+        except ValueError:
+            return int(os.popen2("/usr/sbin/sysctl -n hw.ncpu")[1].read())
     return 1
 
 ###############################################################################
