@@ -1,7 +1,8 @@
 #!/bin/sh
 
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,4 +30,9 @@
 # statement from all source files in the program, then also delete it here.
 
 
-xgettext -k_ -o messages.pot `find ../../ -name '*.py' -and -not -path '*testdata*' -and -not -path '*.git*' -and -not -path '*.svn*' -and -not -path '*build*' -and -not -path '*dist*' -and -not -path '*feedparser.py' -and -not -path '*lib/test/*'`
+xgettext --package-name="Miro" --msgid-bugs-address="http://bugzilla.pculture.org/" --copyright-holder="Participatory Culture Foundation" -o messages.pot `find ../../ -name '*.py' -and -not -path '*testdata*' -and -not -path '*.git*' -and -not -path '*.svn*' -and -not -path '*build*' -and -not -path '*dist*' -and -not -path '*feedparser.py' -and -not -path '*amazon.py' -and -not -path '*lib/test/*'`
+
+# generates faux translation
+cp messages.pot swch.po
+python poxx.py swch.po
+msgfmt swch.po -o swch.mo
