@@ -1,32 +1,33 @@
 /*
- * Miro - an RSS based video player application
- * Copyright (C) 2005-2010 Participatory Culture Foundation
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * In addition, as a special exception, the copyright holders give
- * permission to link the code of portions of this program with the OpenSSL
- * library.
- *
- * You must obey the GNU General Public License in all respects for all of
- * the code used other than OpenSSL. If you modify file(s) with this
- * exception, you may extend this exception to your version of the file(s),
- * but you are not obligated to do so. If you do not wish to do so, delete
- * this exception statement from your version. If you delete this exception
- * statement from all source files in the program, then also delete it here.
-**/
+# Miro - an RSS based video player application
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+*/
 
 /*
  * MiroBrowserEmbed.cpp
@@ -39,7 +40,6 @@
 #include "nsCOMPtr.h"
 #include "nsComponentManagerUtils.h"
 #include "nsEmbedCID.h"
-#include "nsEmbedString.h"
 #include "nsIDOMWindow.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsISupportsImpl.h"
@@ -125,6 +125,16 @@ nsresult MiroBrowserEmbed::enable()
     nsresult rv;
     nsCOMPtr<nsIBaseWindow> browserBaseWindow(do_QueryInterface(mWebBrowser));
 
+    // Check if we haven't loaded a URI yet and load about:blank if so,
+    // otherwise Repaint() fails
+    nsIURI *aURI;
+
+    rv = mWebNavigation->GetCurrentURI(&aURI);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if(!aURI) {
+        loadURI("about:blank");
+    }
+
     rv = browserBaseWindow->SetVisibility(PR_TRUE);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = browserBaseWindow->SetEnabled(PR_TRUE);
@@ -151,7 +161,8 @@ PRBool MiroBrowserEmbed::is_enabled()
 // Load a URI into the browser
 nsresult MiroBrowserEmbed::loadURI(const char* uri)
 {
-    mWebNavigation->LoadURI(NS_ConvertASCIItoUTF16(uri).get(),
+    mCurrentURI = NS_ConvertASCIItoUTF16(uri);
+    mWebNavigation->LoadURI(PromiseFlatString(mCurrentURI).get(),
             nsIWebNavigation::LOAD_FLAGS_NONE, 0, 0, 0);
     return NS_OK;
 }
@@ -164,6 +175,10 @@ nsresult MiroBrowserEmbed::getCurrentURI(char ** uri)
 
     rv = mWebNavigation->GetCurrentURI(&aURI);
     NS_ENSURE_SUCCESS(rv, rv);
+    if(!aURI) {
+        *uri = nsnull;
+        return NS_OK;
+    }
 
     rv = aURI->GetSpec(specString);
     NS_ENSURE_SUCCESS(rv, rv);
