@@ -29,20 +29,15 @@
 
 import os.path
 import os
-import shutil
 import time
 import socket
-import copy
 import sys
 import string
 import subprocess
 import zipfile as zip
 from glob import glob, iglob
-from xml.sax.saxutils import escape
-from distutils import sysconfig
 from distutils.core import Command
 import distutils.command.install_data
-from distutils.ccompiler import new_compiler
 from distutils import file_util, dir_util
 
 
@@ -103,6 +98,7 @@ XULRUNNER_SDK_BIN_PATH = os.path.join(XULRUNNER_SDK_PATH, 'bin')
 VLC_PATH = os.path.join(BINARY_KIT_ROOT, 'libvlc')
 LIBTORRENT_PATH = os.path.join(BINARY_KIT_ROOT, 'libtorrent')
 MUTAGEN_PATH = os.path.join(BINARY_KIT_ROOT, 'mutagen-1.20')
+WINSPARKLE_PATH = os.path.join(BINARY_KIT_ROOT, 'winsparkle')
 
 SEGMENTER_PATH = os.path.join(BINARY_KIT_ROOT, 'segmenter')
 FFMPEG_PATH = os.path.join(
@@ -112,6 +108,10 @@ FFMPEG2THEORA_PATH = os.path.join(BINARY_KIT_ROOT, 'ffmpeg2theora')
 VCREDIST71_PATH = os.path.join(BINARY_KIT_ROOT, 'vc71redist')
 VCREDIST90_PATH = os.path.join(BINARY_KIT_ROOT, 'vc90redist')
 
+# so py2exe can find DLLs
+sys.path.append(VCREDIST71_PATH)
+sys.path.append(os.path.join(VCREDIST90_PATH, 'Microsoft.VC90.CRT'))
+
 def find_data_files(dest_path_base, source_path):
     retval = []
     for path, dirs, files in os.walk(source_path):
@@ -134,13 +134,11 @@ PYTHON_BINARY = "python"
 
 from distutils.core import setup
 from distutils.extension import Extension
-from distutils.core import Command
 from distutils import log
 import py2exe
 import py2exe.build_exe
 import os
 import sys
-import re
 from Pyrex.Distutils import build_ext
 
 # The name of this platform.
@@ -291,6 +289,7 @@ data_files.extend(find_data_files(
         'vlc-plugins', os.path.join(VLC_PATH, 'vlc-plugins')))
 data_files.append(('', [os.path.join(VLC_PATH, 'libvlc.dll')]))
 data_files.append(('', [os.path.join(VLC_PATH, 'libvlccore.dll')]))
+data_files.append(('', [os.path.join(WINSPARKLE_PATH, 'WinSparkle.dll')]))
 data_files.append(('', [os.path.join(LIBTORRENT_PATH, 'libtorrent.pyd')]))
 data_files.append(('', [
             os.path.join(SEGMENTER_PATH, 'miro-segmenter.exe'),
@@ -473,7 +472,6 @@ class bdist_nsis(Command):
     user_options = [
         ('generic', None, 'Build a generic installer instead of the Miro-branded installer.'),
         ('nozugo', None, 'Do not include the silent Zugo toolbar installer.'),
-        ('mozilla', None, 'Do not show the toolbar option to international users.'),
         ('install-icon=', None, 'ICO file to use for the installer.'),
         ('install-image=', None, 'BMP file to use for the welcome/finish pages.')
         ]
@@ -481,7 +479,6 @@ class bdist_nsis(Command):
     def initialize_options(self):
         self.generic = False
         self.nozugo = False
-        self.mozilla = False
         self.install_icon = None
         self.install_image = None
 
@@ -505,7 +502,7 @@ class bdist_nsis(Command):
 
         self.copy_file(os.path.join(platform_dir, 'Miro.nsi'), self.dist_dir)
         if not self.nozugo:
-            self.copy_file(os.path.join(platform_dir, 'zugo-silent.exe'), self.dist_dir)
+            self.copy_file(os.path.join(platform_dir, 'toolbar-silent.exe'), self.dist_dir)
         self.copy_file(self.install_icon, self.dist_dir)
         self.copy_file(self.install_image, self.dist_dir)
 
@@ -530,11 +527,10 @@ class bdist_nsis(Command):
         nsis_vars['MIRO_INSTALL_IMAGE'] = self.install_image
         nsis_vars['CONFIG_BINARY_KIT'] = BINARY_KIT_ROOT
         if not self.nozugo:
-            nsis_vars['MIROBAR_EXE'] = 'zugo-silent.exe'
+            nsis_vars['MIROBAR_EXE'] = 'toolbar-silent.exe'
         if self.generic:
             nsis_vars['GENERIC_INSTALLER'] = '1'
-        if self.mozilla:
-            nsis_vars['MOZILLA_INSTALLER'] = '1'
+        nsis_vars['MOZILLA_INSTALLER'] = '1'
 
         output_file = '%s-%s'
         # One stage installer
@@ -542,8 +538,6 @@ class bdist_nsis(Command):
             output_file = "%s-generic" % output_file
         if self.nozugo:
             output_file = "%s-nozugo" % output_file
-        if self.mozilla:
-            output_file = '%s-mozilla' % output_file
 
         output_file = (output_file %
                        (template_vars['shortAppName'], template_vars['appVersion']))
@@ -558,7 +552,7 @@ class bdist_nsis(Command):
         if subprocess.call([NSIS_PATH] + nsis_args) != 0:
             print "ERROR creating the 1 stage installer, quitting"
             return
-
+        
         # Two stage installer
         if self.generic:
             output_file = '%s-%s-generic-twostage.exe'
@@ -683,6 +677,8 @@ if __name__ == "__main__":
                 'packages': [
                     'encodings'
                     ],
+                # don't include stray DLLs from Win2008
+                'dll_excludes': ['DNSAPI.DLL', 'MSIMG32.DLL'],
                 'includes': ('cairo, pango, pangocairo, atk, gobject, '
                              'gio, libtorrent, mutagen'),
                 },
